/*****************************************************************************
 *  Rohde&Schwarz RTM Instrument Driver                               
 *  LabWindows/CVI Instrument Driver                                     
 *  Original Release: 2010-12-17                                 
 *  By: Jiri Kominek, Martin Krcmar
 *                                                                           
 *  Modification History:                                                    
 *                                                                           
 *       See ChangeLog
 *                                                                           
 *****************************************************************************/

#include <string.h>
#include <stdio.h>  
#include <stdlib.h>
#include <math.h>
#include "rsrtm.h"

ViStatus _VI_FUNC  rsrtm_RsClose (ViSession vi);

/*****************************************************************************
 *- Value Definition and Range Tables ---------------------------------------*
 *****************************************************************************/

 static ViString autoMeasTypeArr[] = {"Average", "Deviation", "Minimum", "Maximum", VI_NULL};
 static ViString signalSourceArr[] = {"NONE","CH1","CH2","CH3","CH4","MA1","MA2","MA3","MA4","MA5",
										 									"RE1","RE2","RE3","RE4","TRIG",VI_NULL};
 static ViString protocolI2CFrameStatus_Arr[] = {"INC", "OK", "UNEX", "INS", "ADD", VI_NULL};
 static ViString protocolI2CAddressType_Arr[] = {"BIT7", "BIT10", VI_NULL};
 static ViString protocolI2CRWBit_Arr[] = {"EITH", "READ", "WRIT", "INC", "UNDF", VI_NULL};
 static ViString protocolI2CAckBit_Arr[] = {"EITH", "ACK", "NACK", "INC", VI_NULL};
 static ViString protocolSPIFrameStatus_Arr[] = {"OK", "INCF", "INCL", "INS", VI_NULL};
 static ViString protocolUARTLine_Arr[] = {"RX", "TX", VI_NULL}; 
 static ViString protocolUARTStatus_Arr[] = {"OK", "FRST", "FREN", "FRME", "STER", "SPER", "PRER", "INS", "BRE", VI_NULL};
 static ViString protocolLINVersion_Arr[] = {"UNK", "V1X", "V2X", VI_NULL};
 static ViString protocolLINIDStatus_Arr[] = {"OK", "PRER", "UNDF", "UVAL", "INS", VI_NULL};
 static ViString protocolLINFieldStatus_Arr[] = {"OK", "ERR", "UNDF", VI_NULL};
 static ViString protocolLINDataStatus_Arr[] = {"OK", "INS", "UART", VI_NULL};
 static ViString protocolCANFrameStatus_Arr[] = {"OK", "BTST", "CRCD", "CRC", "EOFD", "ACKD", "NOACK", "INS", VI_NULL};
 static ViString protocolCANFrameType_Arr[] = {"DATA", "REM", "ERR", "OVLD", VI_NULL};
 static ViString protocolCANIDType_Arr[] = {"B11", "B29", "ANY", VI_NULL};
 static ViString protocolCANFieldStatus_Arr[] = {"OK", "UNDF", VI_NULL};
 static ViString languageArr[] = {"ENGL","GERM","FREN","SPAN","RUSS","SCH","TCH","JAP","KOR",VI_NULL};
 
/*****************************************************************************
 *- User-Callable Functions (Exportable Functions) --------------------------*
 *****************************************************************************/

/*****************************************************************************
 * Function: rsrtm_init
 * Purpose:  VXIplug&play required function. Calls the
 *           rsrtm_InitWithOptions function.
 *****************************************************************************/
ViStatus _VI_FUNC rsrtm_init (ViRsrc resourceName, ViBoolean IDQuery,
                                 ViBoolean resetDevice, ViSession *newInstrSession)
{
    ViStatus    error = VI_SUCCESS;

    if (newInstrSession == VI_NULL)
    {
        (void) Rs_SetErrorInfo (VI_NULL, VI_FALSE, RS_ERROR_INVALID_PARAMETER,
                          VI_ERROR_PARAMETER4, "Null address for Instrument Handle");
        checkErr( RS_ERROR_INVALID_PARAMETER);
    }

    checkErr( rsrtm_InitWithOptions (resourceName, IDQuery, resetDevice,
                                      "QueryInstrStatus=1", newInstrSession));

Error:
    return error;
}

/*****************************************************************************
 * Function: rsrtm_InitWithOptions
 * Purpose:  This function creates a new RS session and calls the
 *           RsInit function.
 *****************************************************************************/
ViStatus _VI_FUNC rsrtm_InitWithOptions(
    ViRsrc          resourceName,
    ViBoolean       IDQuery,
    ViBoolean       resetDevice,
    ViString        optionString,
    ViSession      *newInstrSession
)
{
    ViStatus                error = VI_SUCCESS;
    ViChar                  newResourceName[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar                  newOptionString[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViBoolean               isLogicalName = 0;
    ViSession               instrSession = 0, rmSession = 0;
    RsAttrPropertiesPtr     *attrList=NULL;
    RsRepCapPtr             repCapTable = NULL;
    
    if (newInstrSession == VI_NULL)
        {
        (void) Rs_SetErrorInfo (VI_NULL, VI_FALSE, RS_ERROR_INVALID_PARAMETER,
                         VI_ERROR_PARAMETER5, "Null address for Instrument Handle");
        checkErr( RS_ERROR_INVALID_PARAMETER);
        }

    /* Prepare resource manager session handle */
    checkErr (viOpenDefaultRM (&rmSession));

    /* Establish communication session with the instrument */

    viCheckErr( viOpen (rmSession, resourceName, VI_NULL, VI_NULL, &instrSession));
    
    /* Configure VISA Formatted I/O */
    viCheckErr( viSetAttribute (instrSession, VI_ATTR_TMO_VALUE, 5000 ));
    viCheckErr( viSetBuf (instrSession, (ViUInt16) (VI_READ_BUF | VI_WRITE_BUF), 4000));
    viCheckErr( viSetAttribute (instrSession, VI_ATTR_WR_BUF_OPER_MODE, VI_FLUSH_ON_ACCESS));
    viCheckErr( viSetAttribute (instrSession, VI_ATTR_RD_BUF_OPER_MODE, VI_FLUSH_ON_ACCESS));
    viCheckErr( viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_TRUE));

    checkErr( Rs_GetInfoFromResourceName (resourceName,
                                          (ViString)optionString,
                                          newResourceName,
                                          newOptionString,
                                          &isLogicalName));
    
    ATTRLIST(attrList,rsrtm);  
    checkErr( Rs_SpecificDriverNew (instrSession, "rsrtm", newOptionString, attrList));
    
    /* Build repeated capability table */
    REPCAPTABLE(repCapTable,rsrtm);
    checkErr( Rs_BuildRepCapTable (instrSession, repCapTable));

    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_VISA_RM_SESSION, 0, &rmSession));
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_IO_SESSION, 0, &instrSession));

    /* With remote control via Ethernet, remote control is not automatically
       set by means of a command. The instrument must be explicitly set to
       the REMOTE state, e.g. by sending the intf command &GTR
    */
    checkErr (viWrite (instrSession, (ViBuf) "&GTR\n", 5, NULL));
    
    /*- Reset instrument ----------------------------------------------------*/
    if (resetDevice == VI_TRUE)
    {
        checkErr( rsrtm_reset (instrSession));
    }
    else  /*- Send Default Instrument Setup ---------------------------------*/
        checkErr( rsrtm_DefaultInstrSetup (instrSession));

    /*- Identification Query ------------------------------------------------*/
    if (IDQuery == VI_TRUE)
        {
        ViChar rdBuffer[RSRTM_IO_BUFFER_SIZE] = "";

        checkErr( Rs_GetAttribute (instrSession, VI_NULL, (ViAttr) RSRTM_ATTR_ID_QUERY_RESPONSE,
                                   0, RSRTM_IO_BUFFER_SIZE, rdBuffer));

        if (strncmp (rdBuffer, RSRTM_VALID_ID_RESPONSE_STRING, strlen(RSRTM_VALID_ID_RESPONSE_STRING)) != 0)
            {
            viCheckErr( VI_ERROR_FAIL_ID_QUERY);
            }
        }

    if (isLogicalName == VI_FALSE)
        {
        ViInt32 oldFlag = 0;

        checkErr (Rs_GetAttributeFlags (instrSession, RS_ATTR_IO_RESOURCE_DESCRIPTOR, &oldFlag));
        checkErr (Rs_SetAttributeFlags (instrSession, RS_ATTR_IO_RESOURCE_DESCRIPTOR, oldFlag & 0xfffb | 0x0010));
        checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_IO_RESOURCE_DESCRIPTOR, 0, newResourceName));
        }
    
    if (isLogicalName == VI_TRUE)
        checkErr( Rs_ApplyDefaultSetup (instrSession));

    /* Instrument session accessible by the end user is native VISA session,
       or when simulated virtual instrument (instrSession) session. */
    checkErr( rsrtm_CheckStatus (instrSession));

Error:

    if (error < VI_SUCCESS)
    {
        ViSession   rmSess = VI_NULL;
        
        (void) Rs_GetAttribute (instrSession, VI_NULL, RS_ATTR_VISA_RM_SESSION, 0, (ViInt32) sizeof (rmSess), &rmSess);
        (void) Rs_Dispose (instrSession); 
        if (instrSession != 0)
            (void) viClose (instrSession);
        (void) viClose (rmSess);
    }
    else
    {
        *newInstrSession = instrSession;
    }
    
    return error;
    
}

/// HIFN This function returns one statistical value from the indicated buffer place.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects the measurement.
/// HIPAR bufferPosition/Buffer position.
/// HIPAR value/Statistical value.
ViStatus _VI_FUNC rsrtm_QueryAutomaticMeasurementSingleStatisticalValue (ViSession instrSession,
                                                                         ViInt32 measurement,
                                                                         ViInt32 bufferPosition,
                                                                         ViReal64 *value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar      rep_cap[10] = "";
    
	checkErr (Rs_LockSession (instrSession, VI_NULL));
	
    if (rsrtm_invalidViInt32Range (measurement, RSRTM_VAL_MEASUREMENT_1, RSRTM_VAL_MEASUREMENT_4) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Measurement");
    }
	
	sprintf (rep_cap, "M%ld", measurement);
	
	viCheckParm(Rs_SetAttribute(instrSession, "",
        RSRTM_ATTR_AUTOMATIC_MEASUREMENT_RESULT_INDEX, RS_VAL_SET_CACHE_ONLY, (void *) &bufferPosition), 3, "Buffer Position");
	viCheckParm(rsrtm_GetAttributeViReal64(instrSession, rep_cap,
        RSRTM_ATTR_AUTOMATIC_MEASUREMENT_SINGLE_STATISTICAL_VALUE, value), 4, "Value");
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN Document your function here.
/// HIFN You may use multiple lines for documentation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects the measurement.
/// HIPAR arraySize/Pass the number of elements in 'Results' parameter.
/// HIPAR results/All values from the statistics buffer.
ViStatus _VI_FUNC rsrtm_QueryAutomaticMeasurementAllStatisticalValues (ViSession instrSession,
                                                                       ViInt32 measurement,
                                                                       ViInt32 arraySize,
                                                                       ViReal64 results[])
{
	ViStatus	error = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    ViChar *extBuf=VI_NULL;
    ViChar *pbuffer;
    ViInt32  i = 0;

	checkErr (Rs_LockSession (instrSession, VI_NULL));
	
    if (rsrtm_invalidViInt32Range (measurement, RSRTM_VAL_MEASUREMENT_1, RSRTM_VAL_MEASUREMENT_4) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Measurement");
    }

    viCheckErr (viPrintf (instrSession, "MEAS%ld:STAT:VAL:ALL?\n", measurement));
    viCheckErr (Rs_ReadDataUnknownLength(instrSession, &extBuf, &retCnt)); 

    pbuffer = strtok(extBuf, ",");
    i=0;
    while (pbuffer) 
    {
        if (i < arraySize)
        {
            results[i++] = atof (pbuffer);
        }   
        pbuffer = strtok(NULL, ",");        
    }
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    if (extBuf) free(extBuf);
    return error;
	
}

/// HIFN  This function returns the sample mode the oscilloscope is currently 
/// HIFN  using.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR sampleMode/Returns the method of adding waveform points to the samples of the ADC 
/// HIPAR sampleMode/in order to fill the record length.
ViStatus _VI_FUNC rsrtm_SampleMode(
    ViSession   instrSession,
    ViInt32*    sampleMode
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsrtm_GetAttributeViInt32 (instrSession, "",
                                               RSRTM_ATTR_SAMPLE_MODE,
                                               sampleMode), 2, "SampleMode");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function returns the effective sample rate of the acquired 
/// HIFN  waveform using the current configuration in samples per second.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR sampleRate/Return the sample rate, that is the number of recorded waveform 
/// HIPAR sampleRate/samples per second.
ViStatus _VI_FUNC rsrtm_ActualSampleRate(
    ViSession   instrSession,
    ViReal64*   sampleRate
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsrtm_GetAttributeViReal64 (instrSession, "",
                                                RSRTM_ATTR_HORZ_SAMPLE_RATE,
                                                sampleRate), 2, "Sample Rate");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}


/// HIFN  Queries the attenuation of the probe.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR autoProbeSenseValue/Queries the attenuation of the probe.
ViStatus _VI_FUNC rsrtm_AutoProbeSenseValue(
    ViSession   instrSession,
    ViInt32 channel,
    ViInt32 bufferSize,
    ViChar   autoProbeSenseValue[]
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar  rep_cap[15] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsrtm_GetAttributeRepCapName (instrSession,
                                                 RSRTM_ATTR_PROBE_ATTENUATION,
                                                 "Channel",
                                                 channel,
                                                 15,
                                                 rep_cap), 2, "Channel");

    viCheckParm (rsrtm_GetAttributeViString (instrSession, rep_cap,
                                                RSRTM_ATTR_PROBE_ATTENUATION,
                                                bufferSize,
                                                autoProbeSenseValue), 3, "Auto Probe Sense Value");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function performs an autoset process: analyzes the enabled channel signals, and obtains appropriate horizontal, vertical, and trigger settings to display stable waveforms.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.  The handle identifies a particular instrument session.
ViStatus _VI_FUNC rsrtm_Autoset(
    ViSession   instrSession
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckErr(rsrtm_SetAttributeViString(instrSession, "",RSRTM_ATTR_AUTOSCALE, VI_NULL));
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function configures how the oscilloscope acquires data and fills the waveform record.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR acquisitionType/Specify the manner in which you want the oscilloscope to acquire data and fill the waveform record.
ViStatus _VI_FUNC rsrtm_ConfigureAcquisitionType(
    ViSession instrSession,
    ViInt32   channel,
    ViInt32   acquisitionType
)
{
    ViStatus error = VI_SUCCESS;
    ViChar   rep_cap[15] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsrtm_GetAttributeRepCapName (instrSession,
                                                 RSRTM_ATTR_ACQUISITION_TYPE,
                                                 "Channel",
                                                 channel,
                                                 15,
                                                 rep_cap), 2, "Channel");

    viCheckParm(rsrtm_SetAttributeViInt32(instrSession, rep_cap,
        RSRTM_ATTR_ACQUISITION_TYPE, acquisitionType), 4, "Acquisition Type");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function configures the common attributes of the acquisition subsystem.  These attributes are the time per record and acquisition start time.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR timePerRecord/Pass the time in seconds that corresponds to the record length.  
/// HIPAR acquisitionStartTime_Offset/Specifies the length of time from the trigger event to the first point in the waveform record.  If the value is positive, the first point in the waveform record occurs after the trigger event.  If the 
///           value is negative, the first point in the waveform record occurs before the trigger event.
ViStatus _VI_FUNC rsrtm_ConfigureAcquisitionRecord(
    ViSession   instrSession,
    ViReal64    timePerRecord,
    ViReal64    acquisitionStartTime_Offset
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm(rsrtm_SetAttributeViReal64(instrSession, "",
        RSRTM_ATTR_HORZ_TIME_PER_RECORD, timePerRecord), 2, "Time Per Record");
    
    viCheckParm(rsrtm_SetAttributeViReal64(instrSession, "",
        RSRTM_ATTR_ACQUSITION_START_TIME, acquisitionStartTime_Offset), 3, "Acqusition Start Time");  
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function configures the basic settings for the time axis.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR timeScale/Sets the horizontal scale - the time per division on the x-axis - for all channel and math waveforms.
/// HIPAR referencePoint/Sets the reference point of the time scale in % of the display.
ViStatus _VI_FUNC rsrtm_ConfigureTimeBase(
    ViSession   instrSession,
    ViReal64    timeScale,
    ViInt32     referencePoint
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm(rsrtm_SetAttributeViReal64(instrSession, "",
        RSRTM_ATTR_HORZ_SCALE, timeScale), 2, "Time Scale");

    viCheckParm(rsrtm_SetAttributeViInt32(instrSession, "",
        RSRTM_ATTR_HORZ_REFERENCE, referencePoint), 3, "Reference Point");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN Selects a range of samples that will be returned
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR recordLength/Selects a range of samples that will be returned
ViStatus _VI_FUNC rsrtm_ConfigureRecordLength (ViSession instrSession,
                                               ViInt32 recordLength)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm(rsrtm_SetAttributeViInt32(instrSession, "",
        RSRTM_ATTR_HORZ_RECORD_LENGTH, recordLength), 2, "Record Length");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function configures the roll mode settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR mode/Activates the automatic roll mode.
ViStatus _VI_FUNC rsrtm_ConfigureRollMode(
    ViSession   instrSession,
    ViBoolean mode
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm(rsrtm_SetAttributeViBoolean(instrSession, "",
        RSRTM_ATTR_ROLL_MODE, mode), 2, "Roll Mode");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function queries the number of horizontal divisions on the screen.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR divisions/Queries the number of horizontal divisions on the screen
ViStatus _VI_FUNC rsrtm_QueryDivisions(
    ViSession   instrSession,
    ViInt32*    divisions
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm(rsrtm_GetAttributeViInt32(instrSession, "",
        RSRTM_ATTR_HORZ_DIVISIONS, divisions), 2, "Divisions");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function configures the sample rate, that is the number of recorded waveform samples per second.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR sampleRate/Sets the sample rate, that is the number of recorded waveform samples per second.
ViStatus _VI_FUNC rsrtm_QuerySampleRate(
    ViSession   instrSession,
    ViReal64*    sampleRate
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm(rsrtm_GetAttributeViReal64(instrSession, NULL,
        RSRTM_ATTR_HORZ_SAMPLE_RATE, sampleRate), 2, "Sample Rate");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN Queries the required acquisition time.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR acquisitionTime/Queries the required acquisition time
ViStatus _VI_FUNC rsrtm_QueryAcquisitionTime (ViSession instrSession,
                                              ViReal64 *acquisitionTime)
{
    ViStatus    error = VI_SUCCESS;
    
    viCheckParm(rsrtm_GetAttributeViReal64(instrSession, NULL,
        RSRTM_ATTR_ACQUISITION_TIME, acquisitionTime), 2, "Acquisition Time");
Error:
    return error;
}

/// HIFN  This function defines the mode to set the sample rate
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR waveformRate/Sets the mode to set the sample rate.
ViStatus _VI_FUNC rsrtm_ConfigureWaveformAcquisitionRate (ViSession instrSession,
                                                        ViInt32 waveformRate)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm(rsrtm_SetAttributeViInt32(instrSession, "",
        RSRTM_ATTR_WAVEFORM_RATE, waveformRate), 2, "Waveform Rate");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function queries the sample rate of the ADC, that is the number of points that are sampled by the ADC in one second.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR adcSampleRate/Returns the sample rate of the ADC, that is the number of points that are sampled by the ADC in one second.
ViStatus _VI_FUNC rsrtm_QueryADCSampleRate(
    ViSession   instrSession,
    ViReal64*   adcSampleRate
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm(rsrtm_GetAttributeViReal64(instrSession, "",
        RSRTM_ATTR_HORZ_ADC_SAMPLE_RATE, adcSampleRate), 2, "ADC Sample Rate");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function configures the time of one acquisition, that is the time across the 10 divisions of the diagram.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR acquisitionTime/Sets the time of one acquisition.  
ViStatus _VI_FUNC rsrtm_ConfigureAcquisitionOneTime (ViSession instrSession,
                                                       ViReal64 acquisitionTime)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm(rsrtm_SetAttributeViReal64(instrSession, NULL,
        RSRTM_ATTR_ACQUISITION_TIME, acquisitionTime), 2, "Acquisition Time");    

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function configures the method of adding waveform points to the samples of the ADC in order to fill the record length.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR resolutionEnhancement/Selects the method of adding waveform points to the samples of the ADC in order to fill the record length.
ViStatus _VI_FUNC rsrtm_ConfigureResolutionEnhancement(
    ViSession   instrSession,
    ViInt32     resolutionEnhancement
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm(rsrtm_SetAttributeViInt32(instrSession, "",
        RSRTM_ATTR_SAMPLE_MODE, resolutionEnhancement), 2, "Resolution Enhancement");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function configures the interpolation method the oscilloscope uses when it cannot sample a voltage for a point in the waveform record.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR interpolation/Pass the interpolation method you want the oscilloscope to 
ViStatus _VI_FUNC rsrtm_ConfigureInterpolation(
    ViSession   instrSession,
    ViInt32     interpolation
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm(rsrtm_SetAttributeViInt32(instrSession, "",
        RSRTM_ATTR_INTERPOLATION, interpolation), 2, "Interpolation");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function configures the method to reduce the data stream of the ADC to a stream of waveform points with lower sample rate.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR arithmetic/Selects the method to reduce the data stream of the ADC to a stream of waveform points with lower sample rate.
ViStatus _VI_FUNC rsrtm_ConfigureWaveformArithmetic(
    ViSession   instrSession,
    ViInt32     channel,
    ViInt32     arithmetic
)
{
    ViStatus error = VI_SUCCESS;
    ViChar   rep_cap[15] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsrtm_GetAttributeRepCapName (instrSession,
                                                 RSRTM_ATTR_WAVEFORM_ACQUISITION_TYPE,
                                                 "Channel",
                                                 channel,
                                                 15,
                                                 rep_cap), 2, "Channel");

    viCheckParm(rsrtm_SetAttributeViInt32(instrSession, rep_cap,
        RSRTM_ATTR_WAVEFORM_ACQUISITION_TYPE, arithmetic), 4, "Arithmetic");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}


/// HIFN  Sets the limit frequency if Waveform arithmetics is set to "Smooth".
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.e.
/// HIPAR filterFrequency/Sets the limit frequency if Waveform Arithmetics  is set to "Smooth".
ViStatus _VI_FUNC rsrtm_ConfigureFilterFrequency (ViSession instrSession,
                                                  ViReal64 filterFrequency)
{
	ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));
   
    viCheckParm (rsrtm_SetAttributeViReal64 (instrSession, "",
        RSRTM_ATTR_FILTER_FREQUENCY, filterFrequency), 2, "Filter Frequency");    
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function selects a range of samples.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR samples/Selects a range of samples.
ViStatus _VI_FUNC rsrtm_QueryWaveformSamplesRange (ViSession instrSession,
                                                         ViInt32 channel,
                                                         ViInt32* samples)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[15] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsrtm_GetAttributeRepCapName (instrSession,
                                                 RSRTM_ATTR_WAVEFORM_SAMPLES,
                                                 "Channel",
                                                 channel,
                                                 15,
                                                 rep_cap), 2, "Channel");
    
    viCheckParm (rsrtm_GetAttributeViInt32 (instrSession, rep_cap,
        RSRTM_ATTR_WAVEFORM_SAMPLES, samples), 3, "Samples");    
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function configures the number of waveforms that the oscilloscope  acquires and averages.  After the oscilloscope acquires as many waveforms as you specify, it returns to the idle state.  
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR numberofAverages/Pass the number of waveforms you want the oscilloscope to acquire and to average. After the oscilloscope acquires as many waveforms as you specify, it returns to the idle state.
ViStatus _VI_FUNC rsrtm_ConfigureNumberOfAverages(
    ViSession    instrSession,
    ViInt32      numberofAverages
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm(rsrtm_SetAttributeViInt32(instrSession, "",
        RSRTM_ATTR_NUM_AVERAGES, numberofAverages), 2, "Number of Averages");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
} 


/// HIFN  This function configures the number of waveforms acquired.  
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.
/// HIPAR numberOfWaveforms/Sets the number of waveforms acquired
ViStatus _VI_FUNC rsrtm_ConfigureNumberOfWaveforms (ViSession instrSession,
                                                    ViInt32 numberOfWaveforms)
{
	ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));
   
    viCheckParm (rsrtm_SetAttributeViInt32 (instrSession, "",
        RSRTM_ATTR_NUMBER_OF_WAVEFORMS, numberOfWaveforms), 2, "Number Of Waveforms");    
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function configures the common attributes of the oscilloscope's channel subsystem for a particular channel.  These attributes are the vertical range, vertical offset, coupling, probe attenuation, 
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR verticalRange/Sets the voltage range across the 10 vertical divisions of the diagram.
/// HIPAR verticalOffset/Pass the location of the center of the range that you specify with for the Vertical Range parameter. Express the value with respect to ground.
/// HIPAR verticalCoupling/Specify how you want the oscilloscope to couple the input signal for the channel.
/// HIPAR channelEnabled/Switches the channel signal on or off.
ViStatus _VI_FUNC rsrtm_ConfigureChannel(
    ViSession    instrSession,
    ViInt32      channel,
    ViBoolean    channelEnabled,
    ViReal64     verticalRange,
    ViReal64     verticalOffset,
    ViInt32      verticalCoupling
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar  rep_cap[15] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsrtm_GetAttributeRepCapName (instrSession,
                                                 RSRTM_ATTR_CHANNEL_ENABLED,
                                                 "Channel",
                                                 channel,
                                                 15,
                                                 rep_cap), 2, "Channel");

    viCheckParm(rsrtm_SetAttributeViBoolean(instrSession, rep_cap,
        RSRTM_ATTR_CHANNEL_ENABLED, channelEnabled), 3, "Channel Enabled");
    
    viCheckParm(rsrtm_SetAttributeViReal64(instrSession, rep_cap,
        RSRTM_ATTR_VERTICAL_RANGE, verticalRange), 4, "Vertical Range");
    
    viCheckParm(rsrtm_SetAttributeViReal64(instrSession, rep_cap,
        RSRTM_ATTR_VERTICAL_OFFSET, verticalOffset), 5, "Vertical Offset"); 

    viCheckParm(rsrtm_SetAttributeViInt32(instrSession, rep_cap,
        RSRTM_ATTR_VERTICAL_COUPLING, verticalCoupling), 6, "Vertical Coupling");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function configures the offset voltage. The offset is subtracted to correct an offset-affected signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR offset/Sets the offset voltage.
ViStatus _VI_FUNC rsrtm_ConfigureChannelOffset(
    ViSession   instrSession,
    ViInt32     channel,
    ViReal64    offset
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[15] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsrtm_GetAttributeRepCapName (instrSession,
                                                 RSRTM_ATTR_CHANNEL_OFFSET,
                                                 "Channel",
                                                 channel,
                                                 15,
                                                 rep_cap), 2, "Channel");

    viCheckParm(rsrtm_SetAttributeViReal64(instrSession, rep_cap,
        RSRTM_ATTR_CHANNEL_OFFSET, offset), 3, "Offset");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function configures the vertical scale for the indicated channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR verticalScale/Sets the vertical scale for the indicated channel.
ViStatus _VI_FUNC rsrtm_ConfigureChannelVerticalScale(
    ViSession    instrSession,
    ViInt32      channel,
    ViReal64     verticalScale
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar  rep_cap[15] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsrtm_GetAttributeRepCapName (instrSession,
                                                 RSRTM_ATTR_VERTICAL_SCALE,
                                                 "Channel",
                                                 channel,
                                                 15,
                                                 rep_cap), 2, "Channel");

    viCheckParm(rsrtm_SetAttributeViReal64(instrSession, rep_cap,
        RSRTM_ATTR_VERTICAL_SCALE, verticalScale), 3, "Vertical Scale");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}


/// HIFN  This function sets threshold value for digitization of analog signals.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR threshold/Threshold value for digitization of analog signals.
ViStatus _VI_FUNC rsrtm_ConfigureChannelThreshold (ViSession instrSession,
                                                     ViInt32 channel,
                                                     ViReal64 threshold)
{
    ViStatus    error = VI_SUCCESS;
    ViChar  rep_cap[15] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsrtm_GetAttributeRepCapName (instrSession,
                                                 RSRTM_ATTR_CHANNEL_THRESHOLD,
                                                 "Channel",
                                                 channel,
                                                 15,
                                                 rep_cap), 2, "Channel");

    viCheckParm(rsrtm_SetAttributeViReal64(instrSession, rep_cap,
        RSRTM_ATTR_CHANNEL_THRESHOLD, threshold), 3, "Threshold");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function configures the bandwidth limit for the indicated channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR bandwidth/Selects the bandwidth limit for the indicated channel.
ViStatus _VI_FUNC rsrtm_ConfigureChannelBandwidth(
    ViSession    instrSession,
    ViInt32      channel,
    ViInt32      bandwidth
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[15] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsrtm_GetAttributeRepCapName (instrSession,
                                                 RSRTM_ATTR_CHANNEL_BANDWIDTH_LIMIT,
                                                 "Channel",
                                                 channel,
                                                 15,
                                                 rep_cap), 2, "Channel");

    viCheckParm(rsrtm_SetAttributeViInt32(instrSession, rep_cap,
        RSRTM_ATTR_CHANNEL_BANDWIDTH_LIMIT, bandwidth), 3, "Bandwidth");  
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}


/// HIFN  This function turns the inversion of the signal amplitude on or off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR polarity/Selects polarity.
ViStatus _VI_FUNC rsrtm_ConfigureChannelPolarity (ViSession instrSession,
                                                  ViInt32 channel,
                                                  ViInt32 polarity)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[15] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsrtm_GetAttributeRepCapName (instrSession,
                                                 RSRTM_ATTR_CHANNEL_POLARITY,
                                                 "Channel",
                                                 channel,
                                                 15,
                                                 rep_cap), 2, "Channel");

    viCheckParm(rsrtm_SetAttributeViInt32(instrSession, rep_cap,
        RSRTM_ATTR_CHANNEL_POLARITY, polarity), 3, "Polarity");  
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function configures the overload status of the specified channel from the status bit.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR overload/Retrieves the overload status of the specified channel from the status bit. When the overload problem is solved, the command resets the status bit. Use VI_FALSE to clear the overload condition.
ViStatus _VI_FUNC rsrtm_ConfigureChannelOverload(
    ViSession   instrSession,
    ViInt32     channel,
    ViBoolean   overload
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[15] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsrtm_GetAttributeRepCapName (instrSession,
                                                 RSRTM_ATTR_CHANNEL_OVERLOAD,
                                                 "Channel",
                                                 channel,
                                                 15,
                                                 rep_cap), 2, "Channel");

    viCheckParm(rsrtm_SetAttributeViBoolean(instrSession, rep_cap,
        RSRTM_ATTR_CHANNEL_OVERLOAD, overload), 3, "Overload");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function compensates delay differences between channels
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR deskewValue/Compensates delay differences between channels.
ViStatus _VI_FUNC rsrtm_ConfigureChannelSkew (ViSession instrSession,
                                                ViInt32 channel,
                                                ViReal64 deskewValue)
{
    ViStatus    error = VI_SUCCESS;
    ViChar  rep_cap[15] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsrtm_GetAttributeRepCapName (instrSession,
                                                 RSRTM_ATTR_CHANNEL_SKEW,
                                                 "Channel",
                                                 channel,
                                                 15,
                                                 rep_cap), 2, "Channel");

    viCheckParm(rsrtm_SetAttributeViReal64(instrSession, rep_cap,
        RSRTM_ATTR_CHANNEL_SKEW, deskewValue), 3, "Deskew Value");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function configures the probe settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR channelVisible/Switches the channel signal on or off.
/// HIPAR offset/Sets the offset voltage.
ViStatus _VI_FUNC rsrtm_ConfigureProbe(
    ViSession    instrSession,
    ViInt32        channel,
    ViBoolean    channelVisible,
    ViReal64    offset
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar    rep_cap[10] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    if (rsrtm_invalidViInt32Range (channel, RSRTM_VAL_CHANNEL_1, RSRTM_VAL_CHANNEL_4) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel");
    }

    viCheckParm (rsrtm_GetAttributeRepCapName (instrSession,
                                                 RSRTM_ATTR_CHANNEL_ENABLED,
                                                 "Channel",
                                                 channel,
                                                 10,
                                                 rep_cap), 2, "Channel"); 
    
    viCheckParm (rsrtm_SetAttributeViBoolean (instrSession, rep_cap,
                                                 RSRTM_ATTR_CHANNEL_ENABLED,
                                                 channelVisible), 3, "Channel Visible");

    viCheckParm (rsrtm_SetAttributeViReal64 (instrSession, rep_cap,
                                                RSRTM_ATTR_CHANNEL_OFFSET,
                                                offset), 4, "Offset");    

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function returns the probe characteristics.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the Name 
/// HIPAR bufferSize/and Attenuation values.
/// HIPAR type/Queries the type of the probe.
/// HIPAR name/Queries the name of the probe.
/// HIPAR inputImpedance/Queries the termination of the probe.
/// HIPAR inputCapacitance/Queries the input capacitance of the probe.
/// HIPAR attenuation/Queries the attenuation of the probe.
/// HIPAR bandwidth/Queries the bandwidth of the probe.
ViStatus _VI_FUNC rsrtm_QueryProbeCharacteristics(
    ViSession    instrSession,
    ViInt32        channel,
    ViInt32        bufferSize,
    ViChar        type[],
    ViChar        name[],
    ViReal64*    inputImpedance,
    ViReal64*    inputCapacitance,
    ViChar        attenuation[],
    ViReal64*    bandwidth
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar    rep_cap[10] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    if (rsrtm_invalidViInt32Range (channel, RSRTM_VAL_CHANNEL_1, RSRTM_VAL_CHANNEL_4) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel");
    }

    viCheckParm (rsrtm_GetAttributeRepCapName (instrSession,
                                                 RSRTM_ATTR_PROBE_TYPE,
                                                 "Channel",
                                                 channel,
                                                 10,
                                                 rep_cap), 2, "Channel"); 
    
    viCheckParm (rsrtm_GetAttributeViString (instrSession, rep_cap,
                                               RSRTM_ATTR_PROBE_TYPE,
                                               bufferSize,
                                               type), 4, "Type");
    
    viCheckParm (rsrtm_GetAttributeViString (instrSession, rep_cap,
                                                RSRTM_ATTR_PROBE_NAME,
                                                bufferSize,
                                                name), 5, "Name");
    
    viCheckParm (rsrtm_GetAttributeViReal64 (instrSession, rep_cap,
                                                RSRTM_ATTR_PROBE_INPUT_IMPEDANCE,
                                                inputImpedance), 6, "Input Impedance");
    
    viCheckParm (rsrtm_GetAttributeViReal64 (instrSession, rep_cap,
                                                RSRTM_ATTR_PROBE_INPUT_CAPACITANCE,
                                                inputCapacitance), 7, "Input Capacitance");
    
    viCheckParm (rsrtm_GetAttributeViString (instrSession, rep_cap,
                                                RSRTM_ATTR_PROBE_ATTENUATION,
                                                bufferSize,
                                                attenuation), 8, "Attenuation");    
    
    viCheckParm (rsrtm_GetAttributeViReal64 (instrSession, rep_cap,
                                                RSRTM_ATTR_PROBE_BANDWIDTH,
                                                bandwidth), 9, "Bandwidth");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function configures the action that is started with the micro 
/// HIFN  button on the probe head.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR microButtonAction/Select the action that is started with the micro button on the probe 
/// HIPAR microButtonAction/head.
ViStatus _VI_FUNC rsrtm_ConfigureProbeMicroButtonAction(
    ViSession    instrSession,
    ViInt32        channel,
    ViInt32        microButtonAction
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar    rep_cap[10] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    if (rsrtm_invalidViInt32Range (channel, RSRTM_VAL_CHANNEL_1, RSRTM_VAL_CHANNEL_4) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel");
    }

    viCheckParm (rsrtm_GetAttributeRepCapName (instrSession,
                                                 RSRTM_ATTR_PROBE_MICRO_BUTTON_ACTION,
                                                 "Channel",
                                                 channel,
                                                 10,
                                                 rep_cap), 2, "Channel"); 

    viCheckParm (rsrtm_SetAttributeViInt32 (instrSession, rep_cap,
                                               RSRTM_ATTR_PROBE_MICRO_BUTTON_ACTION,
                                               microButtonAction), 3, "Micro Button Action");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  Retrieves the DC voltage that is measured by the integrated voltmeter of R&S active probes.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR DCOffset/Returns the the DC voltage.
ViStatus _VI_FUNC rsrtm_QueryProbeDCOffset (ViSession instrSession,
                                         ViInt32 channel, 
                                         ViReal64 *DCOffset)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[15] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsrtm_GetAttributeRepCapName (instrSession,
                                                 RSRTM_ATTR_PROBE_DC_OFFSET,
                                                 "Channel",
                                                 channel,
                                                 15,
                                                 rep_cap), 2, "Channel");

    viCheckParm (rsrtm_GetAttributeViReal64 (instrSession, rep_cap,
        RSRTM_ATTR_PROBE_DC_OFFSET, DCOffset), 3, "DC Offset");
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This Function sets an additional probe offset.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR probeOffset/Sets an additional probe offset.
ViStatus _VI_FUNC rsrtm_ConfigureProbeAdditionalOffset (ViSession instrSession,
                                                 ViInt32 channel,
                                                 ViReal64 probeOffset)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[15] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsrtm_GetAttributeRepCapName (instrSession,
                                                 RSRTM_ATTR_PROBE_ADDITIONAL_OFFSET,
                                                 "Channel",
                                                 channel,
                                                 15,
                                                 rep_cap), 2, "Channel");

    viCheckParm (rsrtm_SetAttributeViReal64 (instrSession, rep_cap,
        RSRTM_ATTR_PROBE_ADDITIONAL_OFFSET, probeOffset), 3, "Probe Offset");
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function switches the integrated voltmeter of an R&S active probe on or off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR probeActive/This Control switches the integrated voltmeter of an R&S active probe on or off.
ViStatus _VI_FUNC rsrtm_ConfigureProbeVoltmeterState (ViSession instrSession,
                                               ViInt32 channel,
                                               ViBoolean probeActive)
{ 
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[15] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsrtm_GetAttributeRepCapName (instrSession,
                                                 RSRTM_ATTR_PROBE_VOLTMETER_STATE,
                                                 "Channel",
                                                 channel,
                                                 15,
                                                 rep_cap), 2, "Channel");

    viCheckParm (rsrtm_SetAttributeViBoolean (instrSession, rep_cap,
        RSRTM_ATTR_PROBE_VOLTMETER_STATE, probeActive), 3, "Probe Active"); 
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN This function selects the unit that the probe can measure.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR unit/Selects the unit that the probe can measure.
ViStatus _VI_FUNC rsrtm_ConfigureProbeAttenuationUnit (ViSession instrSession,
                                                       ViInt32 channel,
                                                       ViInt32 unit)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[15] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
	if (rsrtm_invalidViInt32Range (channel, RSRTM_VAL_CHANNEL_1, RSRTM_VAL_CHANNEL_4) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel");
    }

    viCheckParm (rsrtm_GetAttributeRepCapName (instrSession,
                                                 RSRTM_ATTR_PROBE_ATTENUATION_UNIT,
                                                 "Channel",
                                                 channel,
                                                 15,
                                                 rep_cap), 2, "Channel");

    viCheckParm (rsrtm_SetAttributeViInt32 (instrSession, rep_cap,
        RSRTM_ATTR_PROBE_ATTENUATION_UNIT, unit), 3, "Unit"); 
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN This function sets the attenuation or gain of the probe if the probe was not detected by the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR attenuation/Sets the attenuation or gain of the probe if the probe was not detected by the instrument.
ViStatus _VI_FUNC rsrtm_ConfigureProbeAttenuationManual (ViSession instrSession,
                                                         ViInt32 channel,
                                                         ViReal64 attenuation)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[15] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
	if (rsrtm_invalidViInt32Range (channel, RSRTM_VAL_CHANNEL_1, RSRTM_VAL_CHANNEL_4) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel");
    }

    viCheckParm (rsrtm_GetAttributeRepCapName (instrSession,
                                                 RSRTM_ATTR_PROBE_ATTENUATION_MANUAL,
                                                 "Channel",
                                                 channel,
                                                 15,
                                                 rep_cap), 2, "Channel");

    viCheckParm (rsrtm_SetAttributeViReal64 (instrSession, rep_cap,
        RSRTM_ATTR_PROBE_ATTENUATION_MANUAL, attenuation), 3, "Attenuation"); 
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function returns the probe attributes.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the 
/// HIPAR bufferSize/Software Version, Production Date, Part Number and Serial Number 
/// HIPAR bufferSize/values.
/// HIPAR softwareVersion/Queries the version of the probe firmware.
/// HIPAR productionDate/Queries the production date of the probe.
/// HIPAR partNumber/Queries the R&S part number of the probe.
/// HIPAR serialNumber/Queries the serial number of the probe.
ViStatus _VI_FUNC rsrtm_QueryProbeAttributes(
    ViSession    instrSession,
    ViInt32        channel,
    ViInt32        bufferSize,
    ViChar      softwareVersion[],
    ViChar      productionDate[],
    ViChar      partNumber[],
    ViChar      serialNumber[]
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar    rep_cap[10] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    if (rsrtm_invalidViInt32Range (channel, RSRTM_VAL_CHANNEL_1, RSRTM_VAL_CHANNEL_4) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel");
    }

    viCheckParm (rsrtm_GetAttributeRepCapName (instrSession,
                                                 RSRTM_ATTR_PROBE_SOFTWARE_VERSION,
                                                 "Channel",
                                                 channel,
                                                 10,
                                                 rep_cap), 2, "Channel"); 

    viCheckParm (rsrtm_GetAttributeViString (instrSession, rep_cap,
                                                RSRTM_ATTR_PROBE_SOFTWARE_VERSION,
                                                bufferSize,
                                                softwareVersion), 4, "Software Version");
    viCheckParm (rsrtm_GetAttributeViString (instrSession, rep_cap,
                                                RSRTM_ATTR_PROBE_PRODUCTION_DATE,
                                                bufferSize,
                                                productionDate), 5, "Production Date");
    viCheckParm (rsrtm_GetAttributeViString (instrSession, rep_cap,
                                                RSRTM_ATTR_PROBE_PART_NUMBER,
                                                bufferSize,
                                                partNumber), 6, "Part Number");
    viCheckParm (rsrtm_GetAttributeViString (instrSession, rep_cap,
                                                RSRTM_ATTR_PROBE_SERIAL_NUMBER,
                                                bufferSize,
                                                serialNumber), 7, "Serial Number");
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  Queries the build number of the probe software.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR buildNumber/Returns the build number.
ViStatus _VI_FUNC rsrtm_QueryProbeBuildNumber (ViSession instrSession,
                                            ViInt32 channel,
                                            ViInt32 *buildNumber)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[15] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsrtm_GetAttributeRepCapName (instrSession,
                                                 RSRTM_ATTR_PROBE_BUILD_NUMBER,
                                                 "Channel",
                                                 channel,
                                                 15,
                                                 rep_cap), 2, "Channel");

    viCheckParm (rsrtm_GetAttributeViInt32 (instrSession, rep_cap,
        RSRTM_ATTR_PROBE_BUILD_NUMBER, buildNumber), 3, "Build Number");
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function turns an additional 5 kHz or 100 MHz low-pass filter in the trigger path
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR filterType/Selects the filter type.
/// HIPAR filterState/Activates or deactivates a low-pass filter in the trigger path.
ViStatus _VI_FUNC rsrtm_ConfigureEdgeTriggerFilter (ViSession instrSession,
                                                   ViInt32 filterType,
                                                   ViBoolean filterState)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[15] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsrtm_GetAttributeRepCapName (instrSession,
                                                 RSRTM_ATTR_TRIGGER_FILTER,
                                                 "TriggerFilter",
                                                 filterType,
                                                 15,
                                                 rep_cap), 3, "Filter Type");

    viCheckParm(rsrtm_SetAttributeViBoolean (instrSession, rep_cap,
        RSRTM_ATTR_TRIGGER_FILTER, filterState), 4, "Filter State");    
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function sets the time the instrument waits after an A-event until it recognizes B-events.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR delayTime/Sets the time the instrument waits after an A-event until it recognizes B-events.
ViStatus _VI_FUNC rsrtm_ConfigureTriggerDelay (ViSession instrSession,
                                           ViReal64 delayTime)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm(rsrtm_SetAttributeViReal64(instrSession, NULL,
        RSRTM_ATTR_TRIGGER_DELAY, delayTime), 2, "Delay Time");    
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}



/// HIFN Sets the coupling for the external trigger input.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.
/// HIPAR externalCoupling/Sets the coupling for the external trigger input.
ViStatus _VI_FUNC rsrtm_ConfigureExternalTriggerCoupling (ViSession instrSession,
                                                          ViInt32 externalCoupling)
{
	ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm(rsrtm_SetAttributeViInt32(instrSession, NULL,
        RSRTM_ATTR_EXTERNAL_TRIGGER_COUPLING, externalCoupling), 2, "External Coupling");    
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN This function defines the holdoff time.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR holdoffTime/Defines the holdoff time.
ViStatus _VI_FUNC rsrtm_ConfigureTriggerHoldoffTime (ViSession instrSession,
                                                     ViReal64 holdoffTime)
{
	ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm(rsrtm_SetAttributeViReal64 (instrSession, NULL,
							        RSRTM_ATTR_TRIGGER_HOLDOFF_TIME, 
									holdoffTime), 2, "Holdoff Time");    
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function activates or deactivates the second trigger. The instrument triggers if both trigger event conditions (A and B) are fulfilled.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR state/This Control activates or deactivates the second trigger.
ViStatus _VI_FUNC rsrtm_ConfigureTriggerBState (ViSession instrSession,
                                           ViBoolean state)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm(rsrtm_SetAttributeViBoolean (instrSession, NULL,
        RSRTM_ATTR_TRIGGER_STATE, state), 2, "State");    
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  Sets a number of B-trigger events that fulfill all B-trigger conditions but do not cause the trigger. The oscilloscope triggers on the n-th event (the last of the specified number of events).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR eventCount/This Control sets a number of B-trigger events.
ViStatus _VI_FUNC rsrtm_ConfigureTriggerEventCount (ViSession instrSession,
                                                ViInt32 eventCount)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm(rsrtm_SetAttributeViInt32(instrSession, NULL,
        RSRTM_ATTR_TRIGGER_EVENT_COUNT, eventCount), 4, "Event Count");    
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN This function configures the label.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions
/// HIPAR instrSession/function. The handle identifies a particular instrument session.
/// HIPAR channel/String with max. 8 characters, only ASCII characters can be used.
/// HIPAR labelState/Switches the channel name on or off.
/// HIPAR channelName/String with max. 8 characters, only ASCII characters can be used.
ViStatus _VI_FUNC rsrtm_ConfigureLabel (ViSession instrSession,
                                        ViInt32 channel, ViBoolean labelState,
                                        ViString channelName)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[15] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsrtm_GetAttributeRepCapName (instrSession,
                                                 RSRTM_ATTR_LABEL_STATE,
                                                 "Channel", channel,
                                                 15, rep_cap), 2, "Channel");

    viCheckParm (rsrtm_SetAttributeViBoolean (instrSession, rep_cap,
        			RSRTM_ATTR_LABEL_STATE, labelState), 3, "Label State");
	
	viCheckParm (rsrtm_SetAttributeViString (instrSession, rep_cap,
        			RSRTM_ATTR_LABEL_NAME, channelName), 4, "Channel Name");
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function configures the common attributes of the trigger subsystem.  These attributes are the trigger type and holdoff.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR triggerType/Sets the trigger type.
ViStatus _VI_FUNC rsrtm_ConfigureTrigger(
    ViSession   instrSession,
    ViInt32     trigger,
    ViInt32     triggerType
)
{
    ViStatus    error = VI_SUCCESS;
	ViChar      rep_cap[15] = "";
	
    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
	viCheckParm (rsrtm_GetAttributeRepCapName (instrSession,
                                                 RSRTM_ATTR_TRIGGER_TYPE,
                                                 "Trigger",
                                                 trigger,
                                                 15,
                                                 rep_cap), 2, "Trigger");
	
    viCheckParm(rsrtm_SetAttributeViInt32(instrSession, rep_cap,
        RSRTM_ATTR_TRIGGER_TYPE, triggerType), 3, "Trigger Type");    
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function configures the TV trigger.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR standard/This Control sets the color television standard.
/// HIPAR polarity/This Control sets the polarity.
/// HIPAR field/Selects the frame or line.
/// HIPAR line/This Control sets an exact line number.
ViStatus _VI_FUNC rsrtm_ConfigureVideoTriggerSource (ViSession instrSession,
                                                ViInt32 standard, 
                                                ViInt32 polarity,
                                                ViInt32 field, 
                                                ViInt32 line)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    checkErr (rsrtm_ConfigureTrigger (instrSession, RSRTM_VAL_TRIGGER_A, RSRTM_VAL_VIDEO_TRIGGER));
    viCheckParm(rsrtm_SetAttributeViInt32(instrSession, NULL,
        RSRTM_ATTR_TV_STANDARD, standard), 2, "Standard");    
    
    viCheckParm(rsrtm_SetAttributeViInt32(instrSession, NULL,
        RSRTM_ATTR_TV_TRIGGER_POLARITY, polarity), 3, "Polarity");    
        
    viCheckParm(rsrtm_SetAttributeViInt32(instrSession, NULL,
        RSRTM_ATTR_TV_TRIGGER_EVENT, field), 4, "Field");    
    
    viCheckParm(rsrtm_SetAttributeViInt32(instrSession, NULL,
        RSRTM_ATTR_TV_TRIGGER_LINE_NUMBER, line), 5, "Line");    
                                                
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function configures the trigger.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR channel/This control selects the channel.
/// HIPAR triggerSource/selects the channel.
/// HIPAR triggerLevel/This control sets the trigger threshold voltage.
/// HIPAR triggerSlope/Selects the slope.
ViStatus _VI_FUNC rsrtm_ConfigureEdgeTriggerSource (ViSession instrSession,
                                            ViInt32  trigger,
											ViInt32  channel,
                                            ViInt32  triggerSource,
                                            ViReal64 triggerLevel,
                                            ViInt32  triggerSlope)
{
    ViStatus    error = VI_SUCCESS;
    ViChar   rep_cap_trigger[15] = "";
    ViChar   rep_cap_channel[15] = "";
    ViChar   rep_cap[50] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsrtm_GetAttributeRepCapName (instrSession,
                                                 RSRTM_ATTR_TRIGGER_SOURCE,
                                                 "Trigger",
                                                 trigger,
                                                 15,
                                                 rep_cap_trigger), 2, "Trigger");
    
    
    viCheckParm(rsrtm_SetAttributeViInt32(instrSession, rep_cap,
        RSRTM_ATTR_TRIGGER_SOURCE, triggerSource), 4, "Trigger Source");
	
	viCheckParm (rsrtm_GetAttributeRepCapName (instrSession,
                                                 RSRTM_ATTR_TRIGGER_SLOPE,
                                                 "Trigger",
                                                 trigger,
                                                 15,
                                                 rep_cap_trigger), 2, "Trigger");
	
	viCheckParm(rsrtm_SetAttributeViInt32(instrSession, rep_cap,
        RSRTM_ATTR_TRIGGER_SLOPE, triggerSlope), 6, "Trigger Slope");
	
	viCheckParm (rsrtm_GetAttributeRepCapName (instrSession,
                                                 RSRTM_ATTR_TRIGGER_LEVEL,
                                                 "Trigger",
                                                 trigger,
                                                 15,
                                                 rep_cap_trigger), 2, "Trigger");
	
	viCheckParm (rsrtm_GetAttributeRepCapName (instrSession,
                                                 RSRTM_ATTR_TRIGGER_LEVEL,
                                                 "Channel",
                                                 channel,
                                                 15,
                                                 rep_cap_channel), 2, "Channel");
	
	strcpy (rep_cap, rep_cap_trigger);
    strcat (rep_cap, ",");
    strcat (rep_cap, rep_cap_channel);
    
    viCheckParm(rsrtm_SetAttributeViReal64(instrSession, rep_cap,
        RSRTM_ATTR_TRIGGER_LEVEL, triggerLevel), 5, "Trigger Level");
    
    
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  Sets the trigger source for the selected trigger type.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR source/Selects the source.
ViStatus _VI_FUNC rsrtm_ConfigureTriggerSource (ViSession instrSession,
                                            ViInt32 trigger, 
                                            ViInt32 source)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[15] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsrtm_GetAttributeRepCapName (instrSession,
                                                 RSRTM_ATTR_TRIGGER_SOURCE,
                                                 "Trigger",
                                                 trigger,
                                                 15,
                                                 rep_cap), 2, "Trigger");

    viCheckParm(rsrtm_SetAttributeViInt32(instrSession, rep_cap,
        RSRTM_ATTR_TRIGGER_SOURCE, source), 3, "Source");    
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  Sets the coupling for the trigger source.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR coupling/Selects coupling event for the trigger source.
ViStatus _VI_FUNC rsrtm_ConfigureEdgeTriggerCoupling (ViSession instrSession,
                                                ViInt32 coupling)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm(rsrtm_SetAttributeViInt32(instrSession, NULL,
        RSRTM_ATTR_TRIGGER_COUPLING, coupling), 2, "Coupling");    
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  Sets the trigger mode. The trigger mode determines the behaviour of the instrument if no trigger occurs.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR triggerModifier/Sets the trigger mode. The trigger mode determines the behaviour of the instrument if no trigger occurs.
ViStatus _VI_FUNC rsrtm_ConfigureTriggerModifier(
    ViSession   instrSession,
    ViInt32     trigger,
    ViInt32     triggerModifier
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[15] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsrtm_GetAttributeRepCapName (instrSession,
                                                 RSRTM_ATTR_TRIGGER_MODIFIER,
                                                 "Trigger",
                                                 trigger,
                                                 15,
                                                 rep_cap), 2, "Trigger");

    viCheckParm(rsrtm_SetAttributeViInt32(instrSession, rep_cap,
        RSRTM_ATTR_TRIGGER_MODIFIER, triggerModifier), 3, "Trigger Modifier");    
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  Sets the trigger level automatically. The attribute is not available for an external trigger source.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR trigger/Selects trigger event.
ViStatus _VI_FUNC rsrtm_TriggerLevelAutoFind(
    ViSession   instrSession,
    ViInt32     trigger
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[15] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsrtm_GetAttributeRepCapName (instrSession,
                                                 RSRTM_ATTR_TRIGGER_LEVEL_AUTO_FIND,
                                                 "Trigger",
                                                 trigger,
                                                 15,
                                                 rep_cap), 2, "Trigger");

    viCheckParm(rsrtm_SetAttributeViString(instrSession, rep_cap,
        RSRTM_ATTR_TRIGGER_LEVEL_AUTO_FIND, VI_NULL), 3, "Trigger");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN This function configures selected trigger level.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR channel/Sets the trigger source for the selected trigger type.
/// HIPAR triggerLevel/Sets the trigger level for the specified event and channel.
ViStatus _VI_FUNC rsrtm_ConfigureTriggerChannelLevel (ViSession instrSession,
                                                        ViInt32 trigger,
                                                        ViInt32 channel,
                                                        ViReal64 triggerLevel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   rep_cap_trigger[15] = "";
    ViChar   rep_cap_channel[15] = "";
    ViChar   rep_cap[50] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsrtm_GetAttributeRepCapName (instrSession,
                                                 RSRTM_ATTR_TRIGGER_LEVEL,
                                                 "Trigger",
                                                 trigger,
                                                 15,
                                                 rep_cap_trigger), 2, "Trigger");
    viCheckParm (rsrtm_GetAttributeRepCapName (instrSession,
                                                 RSRTM_ATTR_TRIGGER_LEVEL,
                                                 "Channel",
                                                 channel,
                                                 15,
                                                 rep_cap_channel), 2, "Channel");
    strcpy (rep_cap, rep_cap_trigger);
    strcat (rep_cap, ",");
    strcat (rep_cap, rep_cap_channel);
    
    viCheckParm(rsrtm_SetAttributeViReal64(instrSession, rep_cap,
        RSRTM_ATTR_TRIGGER_LEVEL, triggerLevel), 4, "Trigger Level");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function configures the width trigger.  A width trigger occurs 
/// HIFN  when the oscilloscope detects a positive or negative pulse with a width 
/// HIFN  between, or optionally outside, the width thresholds.  You specify the 
/// HIFN  width thresholds with the Width Low Threshold and Width High Threshold 
/// HIFN  parameters.  You specify whether the oscilloscope triggers on pulse 
/// HIFN  widths that are within or outside the width thresholds with the Width 
/// HIFN  Condition parameter.  You specify the polarity of the pulse with the 
/// HIFN  Width Polarity parameter.  The trigger does not actually occur until 
/// HIFN  the edge of a pulse that corresponds to the Width Low Threshold, Width 
/// HIFN  High Threshold, Width Condition, and Width Polarity crosses the 
/// HIFN  threshold you specify in the Trigger Level parameter.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR width/For the ranges "Within" and "Outside", the width defines the center of 
/// HIPAR width/ a range which is defined by the limits "+/-Delta". For the ranges 
/// HIPAR width/ "Shorter" and "Longer", the width defines the maximum and minimum pulse 
/// HIPAR width/ width, respectively.
/// HIPAR widthDelta/Defines a range around the width value.
/// HIPAR widthPolarity/Sets the polarity of the pulse.
/// HIPAR triggerSource/Sets the trigger source for the selected trigger type.
/// HIPAR widthCondition/Sets how the measured pulse width is compared with the given limit(s).
ViStatus _VI_FUNC rsrtm_ConfigureWidthTriggerSource(
    ViSession   instrSession,
    ViInt32     triggerSource,
    ViReal64    triggerLevel,
    ViReal64    width,
    ViReal64    widthDelta,
    ViInt32     widthPolarity,
    ViInt32     widthCondition
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsrtm_ConfigureTrigger (instrSession, RSRTM_VAL_TRIGGER_A,
                                      RSRTM_VAL_WIDTH_TRIGGER));
	
    viCheckParm(rsrtm_SetAttributeViInt32(instrSession, "TrigA",
        RSRTM_ATTR_TRIGGER_SOURCE, triggerSource), 3, "Trigger Source");

    viCheckParm(rsrtm_ConfigureTriggerChannelLevel (instrSession,
                                                    RSRTM_VAL_TRIGGER_A,
                                                    triggerSource,
                                                    triggerLevel), 4, "Trigger Level");
    
    viCheckParm(rsrtm_SetAttributeViReal64(instrSession, NULL,
        RSRTM_ATTR_WIDTH, width), 5, "Width");

    viCheckParm(rsrtm_SetAttributeViReal64(instrSession, NULL,
        RSRTM_ATTR_WIDTH_DELTA, widthDelta), 6, "Width Delta");   

    viCheckParm(rsrtm_SetAttributeViInt32(instrSession, NULL,
        RSRTM_ATTR_WIDTH_POLARITY, widthPolarity), 7, "Width Polarity");  

    viCheckParm(rsrtm_SetAttributeViInt32(instrSession, NULL,
        RSRTM_ATTR_WIDTH_CONDITION, widthCondition), 8, "Width Condition");   

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function configures persistence.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR enable/If enabled, each new data point in the diagram area remains on the 
/// HIPAR enable/screen for the defined duration. If disabled, the signal value is only 
/// HIPAR enable/displayed as long as it actually occurs.
/// HIPAR infinite/If persistence is enabled, each new data point in the diagram area 
/// HIPAR infinite/remains on the screen infinitely until is disabled.
/// HIPAR time/If persistence is enabled, each new data point in the diagram area 
/// HIPAR time/remains on the screen for the duration defined here.
ViStatus _VI_FUNC rsrtm_ConfigurePersistence(
    ViSession    instrSession,
    ViBoolean    enable,
    ViBoolean    infinite,
    ViReal64    time
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsrtm_SetAttributeViBoolean (instrSession, "",
                                                 RSRTM_ATTR_PERSISTENCE_STATE,
                                                 enable), 2, "Enable");
    
    if (enable == VI_TRUE)
    {
        viCheckParm (rsrtm_SetAttributeViBoolean (instrSession, "",
                                                     RSRTM_ATTR_INFINITE_PERSISTENCE,
                                                     infinite), 3, "Infinite");
        if (infinite == VI_FALSE)
            {
                viCheckParm (rsrtm_SetAttributeViReal64 (instrSession, "",
                                                            RSRTM_ATTR_PERSISTENCE_TIME,
                                                            time), 4, "Time");
            }
    }
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function removes the displayed persistent waveform from the screen.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsrtm_ClearPersistence (ViSession instrSession)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsrtm_SetAttributeViString (instrSession, "",
                                                RSRTM_ATTR_PERSISTENCE_CLEAR,
                                                VI_NULL
                                                ), 1, "Session");
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function configures optimal persistence time.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR autoPersist/If enabled, sets optimal persistence time.
ViStatus _VI_FUNC rsrtm_ConfigurePersistenceTimeAuto (ViSession instrSession,
                                               ViBoolean autoPersist)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm(rsrtm_SetAttributeViBoolean(instrSession, "",
        RSRTM_ATTR_PERSISTENCE_TIME_AUTO, autoPersist),
        2, "AutoPersist");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function configures signal intensity and style.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR style/Select the style in which the waveform is displayed.
ViStatus _VI_FUNC rsrtm_ConfigureSignalStyle(
    ViSession    instrSession,
    ViInt32        style
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsrtm_SetAttributeViInt32 (instrSession, "",
                                               RSRTM_ATTR_SIGNAL_STYLE,
                                               style), 2, "Style");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN Document your function here.
/// HIFN You may use multiple lines for documentation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR element/Selects the element.
/// HIPAR intensity/This value defines the intensity of the lighting of the selected display element
ViStatus _VI_FUNC rsrtm_ConfigureDisplayIntensity (ViSession instrSession,
                                                   ViInt32 element,
                                                   ViReal64 intensity)
{
    ViStatus    error = VI_SUCCESS;
    ViAttr      attribute = 0;

    switch (element)
    {
        case RSRTM_VAL_DISPLAY_BACKGROUND:
            attribute = RSRTM_ATTR_BACKGROUND_INTENSITY;
            break;
        case RSRTM_VAL_DISPLAY_GRID:
            attribute = RSRTM_ATTR_GRID_INTENSITY;
            break;
        case RSRTM_VAL_DISPLAY_WAVEFORM:
            attribute = RSRTM_ATTR_WAVEFORM_INTENSITY;
            break;
        default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Element");
    }
    
    viCheckParm (rsrtm_SetAttributeViReal64 (instrSession, NULL, attribute, intensity), 3, "Intensity");
Error:
    return error;
}

/// HIFN  This function sets the color and brightness of the displayed waveform samples depending on their cumulative occurance.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR palette/Selects the color and brightness of the displayed waveform samples.
ViStatus _VI_FUNC rsrtm_ConfigureColorAndBrightness (ViSession instrSession,
                                              ViInt32 palette)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsrtm_SetAttributeViInt32 (instrSession, "", 
        RSRTM_ATTR_COLOR_AND_BRIGHTNESS, palette), 2, "Palette");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function sets the language.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR language/Selects the language.
ViStatus _VI_FUNC rsrtm_ConfigureDisplayLanguage (ViSession instrSession,
                                           ViInt32 language)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsrtm_SetAttributeViInt32 (instrSession, "", 
        RSRTM_ATTR_DISPLAY_LANGUAGE, language), 2, "Language");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN This function installs the specified language on the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR language/Selects the language.
/// HIPAR source/This control specifies the string parameter containing source path and file
ViStatus _VI_FUNC rsrtm_DisplayAddLanguage (ViSession instrSession,
                                            ViInt32 language, ViString source)
{
	ViStatus	error = VI_SUCCESS;
	ViChar      cmd [RSRTM_IO_BUFFER_SIZE] = "";
    
	checkErr (Rs_LockSession (instrSession, VI_NULL));
	
	if (rsrtm_invalidViInt32Range (language, 
        RSRTM_VAL_DISPLAY_ENGLISH, RSRTM_VAL_DISPLAY_KOREAN) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Language");
    }
	sprintf (cmd, "DISP:LANG:ADD %s,\"%s\"", languageArr[language], source);
	checkErr (viWrite (instrSession, (ViBuf) cmd, (ViUInt32) strlen (cmd), NULL));
	checkErr (rsrtm_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN Removes the specified language from the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR language/Selects the language
ViStatus _VI_FUNC rsrtm_DisplayRemoveLanguage (ViSession instrSession,
                                               ViInt32 language)
{
	ViStatus	error = VI_SUCCESS;
	viCheckParm(rsrtm_SetAttributeViInt32(instrSession, "",
        RSRTM_ATTR_REMOVE_DISPLAY_LANGUAGE, language), 2, "Language");
Error:
	return error;
}

/// HIFN  This function sets the diagram mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR mode/Selects the diagram mode.
ViStatus _VI_FUNC rsrtm_ConfigureDiagramMode (ViSession instrSession, ViInt32 mode)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsrtm_SetAttributeViInt32 (instrSession, "", 
        RSRTM_ATTR_DIAGRAM_MODE, mode), 2, "Mode");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function defines how the grid is displayed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR style/Selects the grid style.
ViStatus _VI_FUNC rsrtm_ConfigureDisplayGridStyle (ViSession instrSession,
                                            ViInt32 style)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsrtm_SetAttributeViInt32 (instrSession, "",
                                               RSRTM_ATTR_DIAGRAM_GRID_STYLE,
                                               style
                                               ), 3, "Style");
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function defines the source to be displayed in x direction in an XY-diagram, replacing the usual time base.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR source/Defines the source to be displayed.
ViStatus _VI_FUNC rsrtm_ConfigureSourceX (ViSession instrSession, ViInt32 source)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsrtm_SetAttributeViInt32 (instrSession, "", 
        RSRTM_ATTR_XY_X_SOURCE, source), 2, "Source");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}


/// HIFN  This function defines the (first) source to be displayed in y direction in an XY-diagram.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR source/Defines the source to be displayed.
ViStatus _VI_FUNC rsrtm_ConfigureSourceY1 (ViSession instrSession, ViInt32 source)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsrtm_SetAttributeViInt32 (instrSession, "", 
        RSRTM_ATTR_XY_Y_SOURCE, source), 2, "Source");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function defines an optional second source to be displayed in y direction in an XY-diagram. The command is only relevant for 4-channel R&S RTM instruments.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR source/Defines the source to be displayed.
ViStatus _VI_FUNC rsrtm_ConfigureSourceY2 (ViSession instrSession, ViInt32 source)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsrtm_SetAttributeViInt32 (instrSession, "", 
        RSRTM_ATTR_XY_Y2_SOURCE, source), 2, "Source");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function activates or deactivates the intensity control of the waveform via an additional signal source and sets the intensity mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR mode/Selects the mode of intensity control.
ViStatus _VI_FUNC rsrtm_ConfigureIntensityControl (ViSession instrSession,
                                            ViInt32 mode)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsrtm_SetAttributeViInt32 (instrSession, "", 
        RSRTM_ATTR_XY_INTENSITY_CONTROL, mode), 2, "Mode");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function defines the source to be used to determine the intensity of the xy-waveform.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR source/Defines the source.
ViStatus _VI_FUNC rsrtm_ConfigureSourceIntensity (ViSession instrSession,
                                           ViInt32 source)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsrtm_SetAttributeViInt32 (instrSession, "", 
        RSRTM_ATTR_XY_Z_SOURCE, source), 2, "Source");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function defines the threshold for intensity with a two-state modulation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR treshold/This control defines the threshold.
ViStatus _VI_FUNC rsrtm_ConfigureIntensityTreshold (ViSession instrSession,
                                             ViReal64 treshold)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsrtm_SetAttributeViReal64 (instrSession, "", 
        RSRTM_ATTR_XY_Z_TRESHOLD, treshold), 2, "Treshold");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function defines the position of the zoom reference point (the reference point of the zoom window) in relation to the reference point of original time base.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR position/Defines the zoom reference point.
ViStatus _VI_FUNC rsrtm_ZoomReferencePoint (ViSession instrSession,
                                              ViInt32 position)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsrtm_SetAttributeViInt32 (instrSession, "",
                                               RSRTM_ATTR_ZOOM_REFERENCE_POINT,
                                               position
                                               ), 2, "Position");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function switches the zoom window on or off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR zoomEnabled/Switches the zoom window on or off.
ViStatus _VI_FUNC rsrtm_ConfigureZoomWindowState (ViSession instrSession,
                                           ViBoolean zoomEnabled)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm(rsrtm_SetAttributeViBoolean(instrSession, NULL,
        RSRTM_ATTR_ZOOM_WINDOW_STATE, zoomEnabled), 2, "Zoom Enabled");
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN This function defines the position of the zoom reference point (the reference point of the zoom window) in relation to the reference point of original time base.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR position/Defines the zoom reference point.
ViStatus _VI_FUNC rsrtm_ConfigureZoomReferencePoint (ViSession instrSession,
                                                     ViReal64 position)
{
    ViStatus    error = VI_SUCCESS;
    viCheckParm(rsrtm_SetAttributeViReal64(instrSession, NULL,
        RSRTM_ATTR_ZOOM_REFERENCE_POINT, position), 2, "Position");
Error:
    return error;
}

/// HIFN  This function defines the time base in the zoom diagram in seconds per division.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR scale/Scaling of the zoom time base.
ViStatus _VI_FUNC rsrtm_ConfigureZoomDiagramTimebase (ViSession instrSession,
                                               ViReal64 scale)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm(rsrtm_SetAttributeViReal64(instrSession, NULL,
        RSRTM_ATTR_ZOOM_DIAGRAM_TIMEBASE, scale), 2, "Scale");    

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function defines the offset of the trigger point to the reference point of the zoom diagram.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR time/Defines the offset of the trigger.
ViStatus _VI_FUNC rsrtm_ConfigureZoomOffsetOfTriggerPoint (ViSession instrSession,
                                                    ViReal64 time)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm(rsrtm_SetAttributeViReal64(instrSession, "",
        RSRTM_ATTR_ZOOM_OFFSET_OF_TRIGGER_POINT, time), 2, "Time");    

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function configures the markers.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR markerAction/This Control selects the marker action.
ViStatus _VI_FUNC rsrtm_MarkerSearch (ViSession instrSession,
                                              ViInt32 markerAction)
{
    ViStatus    error = VI_SUCCESS;

    if (rsrtm_invalidViInt32Range (markerAction, 
        RSRTM_VAL_MARKER_ACTION_SET, RSRTM_VAL_MARKER_ACTION_ACLEAR) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Marker Action");
    }
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    switch (markerAction)
    {
        case RSRTM_VAL_MARKER_ACTION_SET:
            viCheckErr (rsrtm_SetAttributeViString (instrSession, "",
                RSRTM_ATTR_MARKER_SET, VI_NULL));
            break;
        case RSRTM_VAL_MARKER_ACTION_NEXT:
            viCheckErr (rsrtm_SetAttributeViString (instrSession, "",
                RSRTM_ATTR_MARKER_NEXT, VI_NULL));
            break;
        case RSRTM_VAL_MARKER_ACTION_PREV:
            viCheckErr (rsrtm_SetAttributeViString (instrSession, "",
                RSRTM_ATTR_MARKER_PREVIOUS, VI_NULL));
            break;
        case RSRTM_VAL_MARKER_ACTION_CLEAR:
            viCheckErr (rsrtm_SetAttributeViString (instrSession, "",
                RSRTM_ATTR_MARKER_CLEAR, VI_NULL));
            break;
        case RSRTM_VAL_MARKER_ACTION_ACLEAR:
            viCheckErr (rsrtm_SetAttributeViString (instrSession, "",
                RSRTM_ATTR_MARKER_CLEAR_ALL, VI_NULL));
            break;

        default:
            break;
    }
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function configures the indicated measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR measurement/This control selects the measurement.
/// HIPAR measurementState/Switches the indicated measurement on or off.
ViStatus _VI_FUNC rsrtm_ConfigureMeasurement(
    ViSession   instrSession,
    ViInt32     measurement,
    ViBoolean   measurementState
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar   rep_cap_measurement[25] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsrtm_GetAttributeRepCapName (instrSession,
                                                 RSRTM_ATTR_MEASUREMENT_ENABLED,
                                                 "Measurement",
                                                 measurement - 1,
                                                 25,
                                                 rep_cap_measurement), 2, "Measurement");

    viCheckParm(rsrtm_SetAttributeViBoolean(instrSession, rep_cap_measurement,
        RSRTM_ATTR_MEASUREMENT_ENABLED, measurementState), 3, "Measurement State");    
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function configures the state of the indicated measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR measurement/This control selects the measurement.
/// HIPAR signalSource/Selects the source of the measurement.
/// HIPAR referenceSource/Selects the source of the reference.
ViStatus _VI_FUNC rsrtm_ConfigureMeasurementSource(
    ViSession   instrSession,
    ViInt32     measurement,
    ViInt32     signalSource,
    ViInt32     referenceSource
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    if (rsrtm_invalidViInt32Range (measurement, RSRTM_VAL_MEASUREMENT_1, 
        RSRTM_VAL_MEASUREMENT_4) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Measurement");
    }
    if (rsrtm_invalidViInt32Range (signalSource, RSRTM_VAL_MEASUREMENT_SOURCE_NONE, 
        RSRTM_VAL_MEASUREMENT_SOURCE_TRIG) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Signal Source");
    }
    if (rsrtm_invalidViInt32Range (referenceSource, RSRTM_VAL_MEASUREMENT_SOURCE_NONE, 
        RSRTM_VAL_MEASUREMENT_SOURCE_R4) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Reference Source");
    }

    viCheckErr (viPrintf(instrSession, "MEAS%ld:SOUR %s,%s", 
        measurement, signalSourceArr[signalSource], signalSourceArr[referenceSource]));
    
    viCheckErr(rsrtm_CheckStatusTimed(instrSession));
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN Document your function here.
/// HIFN You may use multiple lines for documentation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR measurement/This control selects the measurement.
/// HIPAR quickMeasurementsEnabled/This function enables and disables all quick measurements of the indicated measurement
ViStatus _VI_FUNC rsrtm_ConfigureQuickMeasurements (ViSession instrSession,
                                                    ViInt32 measurement,
                                                    ViBoolean quickMeasurementsEnabled)
{
    ViStatus    error = VI_SUCCESS;
	ViChar   rep_cap_measurement[25] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsrtm_GetAttributeRepCapName (instrSession,
                                                 RSRTM_ATTR_MEASUREMENT_ENABLED,
                                                 "Measurement",
                                                 measurement - 1,
                                                 25,
                                                 rep_cap_measurement), 2, "Measurement");
	
    if (quickMeasurementsEnabled)
    {
        viCheckParm(rsrtm_SetAttributeViString(instrSession, rep_cap_measurement,
        RSRTM_ATTR_QUICK_MEASUREMENTS_ENABLE, VI_NULL), 3, "Quick Measurements Enable");
    }
    else
    {
        viCheckParm(rsrtm_SetAttributeViString(instrSession, rep_cap_measurement,
        RSRTM_ATTR_QUICK_MEASUREMENTS_DISABLE, VI_NULL), 3, "Quick Measurements Disable");
    }
Error:
	Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN This function starts or stops the quick measurement and sets the status bit
/// HIRET Document return value here.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR measurement/This control selects the measurement.
/// HIPAR quickMeasurementsEnabled/Enables and disables quick measurement.
ViStatus _VI_FUNC rsrtm_ConfigureQuickMeasurementsWithStatusBit (ViSession instrSession,
                                                                 ViInt32 measurement,
                                                                 ViBoolean quickMeasurementsEnabled)
{
    ViStatus    error = VI_SUCCESS;
	ViChar   rep_cap_measurement[25] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsrtm_GetAttributeRepCapName (instrSession,
                                                 RSRTM_ATTR_MEASUREMENT_ENABLED,
                                                 "Measurement",
                                                 measurement - 1,
                                                 25,
                                                 rep_cap_measurement), 2, "Measurement");
	
	viCheckParm(rsrtm_SetAttributeViBoolean(instrSession, rep_cap_measurement,
        RSRTM_ATTR_QUICK_MEASUREMENTS_ENABLE_WITH_STATUS_BIT, quickMeasurementsEnabled), 3, "Quick Measurements Enable");

Error:
	Rs_UnlockSession(instrSession, VI_NULL);
	return error;
}

/// HIFN  This function returns the results of the quick measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR measurement/This control selects the measurement.
/// HIPAR results/Returns the list of values.
ViStatus _VI_FUNC rsrtm_QueryQuickMeasurementResults (ViSession instrSession,
                                            ViInt32 measurement,
                                            ViInt32 arraySize,
                                            ViReal64 results[])
{
    ViStatus    error = VI_SUCCESS;
    ViChar      *buffer = VI_NULL;
    ViChar      *pbuff = VI_NULL;
    ViInt32     i = 0;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    if (rsrtm_invalidViInt32Range (measurement, RSRTM_VAL_MEASUREMENT_1, 
        RSRTM_VAL_MEASUREMENT_4) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Measurement");
    }

    checkErr (viPrintf (instrSession, "MEAS%ld:ARES?\n", measurement));
    viCheckErr(Rs_ReadDataUnknownLength (instrSession, &buffer, VI_NULL)); 
    
    pbuff = strtok (buffer, ",");
    
    while (pbuff != NULL && (i<arraySize))
    {
        if (i<arraySize)
        {
            results[i++] = atof (pbuff);
            pbuff = strtok (NULL, ",");
        }
        i++;
    }
    
    viCheckErr(rsrtm_CheckStatus(instrSession));
Error:
    if (buffer) free (buffer);
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN This function configures automatic measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function. 
/// HIPAR instrSession/The handle identifies a particular instrument session.
/// HIPAR measurement/This control selects the measurement.
/// HIPAR state/Activates or deactivates the statistical evaluation for the selected measurement.
/// HIPAR numberOfWaveforms/Sets the number of measured waveforms.
ViStatus _VI_FUNC rsrtm_ConfigureAutomaticMeasurement (ViSession instrSession,
                                                       ViInt32 measurement,
                                                       ViBoolean state,
                                                       ViInt32 numberOfWaveforms)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap_measurement[25] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsrtm_GetAttributeRepCapName (instrSession,
                                                 RSRTM_ATTR_AUTOMATIC_MEASUREMENT_STATE,
                                                 "Measurement",
                                                 measurement - 1,
                                                 25,
                                                 rep_cap_measurement), 2, "Measurement");

    viCheckParm(rsrtm_SetAttributeViBoolean(instrSession, rep_cap_measurement,
        RSRTM_ATTR_AUTOMATIC_MEASUREMENT_STATE, state), 3, "State");
	
	if (state){
		viCheckParm(rsrtm_SetAttributeViInt32(instrSession, rep_cap_measurement,
	        RSRTM_ATTR_AUTOMATIC_MEASUREMENT_NUMBER_OF_WAVEFORMS, numberOfWaveforms), 4, "Number Of Waveforms");
	}
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN This function resets the statistical results for the selected measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function. 
/// HIPAR instrSession/The handle identifies a particular instrument session.
/// HIPAR measurement/This control selects the measurement.
ViStatus _VI_FUNC rsrtm_ResetAutomaticMeasurement (ViSession instrSession,
                                                   ViInt32 measurement)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap_measurement[25] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsrtm_GetAttributeRepCapName (instrSession,
                                                 RSRTM_ATTR_RESET_AUTOMATIC_MEASUREMENT,
                                                 "Measurement",
                                                 measurement - 1,
                                                 25,
                                                 rep_cap_measurement), 2, "Measurement");

    viCheckErr (rsrtm_SetAttributeViString (instrSession, rep_cap_measurement,
        									RSRTM_ATTR_RESET_AUTOMATIC_MEASUREMENT, VI_NULL));
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN This function returns the result of the automatic measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function. 
/// HIPAR instrSession/The handle identifies a particular instrument session.
/// HIPAR measurement/This control selects the measurement.
/// HIPAR numberOfWaveforms/Returns the result of the automatic measurement.
ViStatus _VI_FUNC rsrtm_QueryAutomaticMeasurementNumberOfWaveforms (ViSession instrSession,
                                                                    ViInt32 measurement,
                                                                    ViInt32 *numberOfWaveforms)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap_measurement[25] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsrtm_GetAttributeRepCapName (instrSession,
                                                 RSRTM_ATTR_AUTOMATIC_MEASUREMENT_NUMBER_OF_MEASURED_WAVEFORMS,
                                                 "Measurement",
                                                 measurement - 1,
                                                 25,
                                                 rep_cap_measurement), 2, "Measurement");
	
	viCheckParm (rsrtm_GetAttributeViInt32 (instrSession, rep_cap_measurement,
		RSRTM_ATTR_AUTOMATIC_MEASUREMENT_NUMBER_OF_MEASURED_WAVEFORMS, numberOfWaveforms), 4, "Number Of Waveforms");

	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN This function returns the result of the automatic measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function. 
/// HIPAR instrSession/The handle identifies a particular instrument session.
/// HIPAR measurement/This control selects the measurement.
/// HIPAR resultType/This control selects the result type.
/// HIPAR result/Returns the automatic measuremnt result.
ViStatus _VI_FUNC rsrtm_QueryAutomaticMeasurementResult (ViSession instrSession,
                                                         ViInt32 measurement,
                                                         ViInt32 resultType,
                                                         ViReal64 *result)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap_measurement[25] = "";
	ViChar      rep_cap_type[25] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));
	
	if (rsrtm_invalidViInt32Range (resultType, 0, 3) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Result Type");
    }
	
    viCheckParm (rsrtm_GetAttributeRepCapName (instrSession,
                                                 RSRTM_ATTR_AUTOMATIC_MEASUREMENT_RESULT,
                                                 "Measurement",
                                                 measurement - 1,
                                                 25,
                                                 rep_cap_measurement), 2, "Measurement");

	sprintf(rep_cap_type, ",%s", autoMeasTypeArr[resultType]);
	strcat(rep_cap_measurement, rep_cap_type);
	
    viCheckParm(rsrtm_GetAttributeViReal64 (instrSession, rep_cap_measurement,
        						RSRTM_ATTR_AUTOMATIC_MEASUREMENT_RESULT, result), 4, "Result"); 
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  Sets the edges to be used for delay measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.
/// HIPAR instrSession/The handle identifies a particular instrument session.
/// HIPAR measurement/This control selects the measurement.
/// HIPAR source1Slope/Slope of source 1 (first waveform).
/// HIPAR source2Slope/Slope of source 2 (second waveform).
ViStatus _VI_FUNC rsrtm_ConfigureDelayMeasuremenSlope (ViSession instrSession,
                                                       ViInt32 measurement,
                                                       ViInt32 source1Slope,
                                                       ViInt32 source2Slope)
{
	ViStatus    error = VI_SUCCESS;
    ViString	slopeArr[] = {"Pos","Neg"};
	ViChar      rep_cap[25] = "";
	ViChar      rep_cap_slope[25] = "";
	
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    if (rsrtm_invalidViInt32Range (source1Slope, RSRTM_VAL_MEAS_SLOPE_POS, 
        RSRTM_VAL_MEAS_SLOPE_NEG) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Source 1 Slope");
    }
	
	if (rsrtm_invalidViInt32Range (source2Slope, RSRTM_VAL_MEAS_SLOPE_POS, 
        RSRTM_VAL_MEAS_SLOPE_NEG) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Source 2 Slope");
    }
	
	viCheckParm (rsrtm_GetAttributeRepCapName (instrSession,
                                                 RSRTM_ATTR_CLEAR_MEASUREMENT_STATISTIC_DATA,
                                                 "Measurement",
                                                 measurement - 1,
                                                 25,
                                                 rep_cap), 2, "Measurement");
	
	sprintf (rep_cap_slope, ",%s", slopeArr[source1Slope]);
	strcat (rep_cap, rep_cap_slope);
	sprintf (rep_cap_slope, ",%s", slopeArr[source2Slope]);
	strcat (rep_cap, rep_cap_slope);

    viCheckParm(rsrtm_SetAttributeViString(instrSession, rep_cap,
        RSRTM_ATTR_MEAS_DELAY_SLOPE, VI_NULL), 2, "Measurement"); 
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}


/// HIFN  This function returns the measurement category.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.
/// HIPAR instrSession/The handle identifies a particular instrument session.
/// HIPAR measurement/This control selects the measurement.
/// HIPAR category/Returns the measurement category.
ViStatus _VI_FUNC rsrtm_QueryMeasurementCategory (ViSession instrSession,
                                                  ViInt32 measurement,
                                                  ViInt32 *category)
{
	ViStatus    error = VI_SUCCESS;
    ViChar   rep_cap_measurement[25] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsrtm_GetAttributeRepCapName (instrSession,
                                                 RSRTM_ATTR_MEASUREMENT_ENABLED,
                                                 "Measurement",
                                                 measurement - 1,
                                                 25,
                                                 rep_cap_measurement), 2, "Measurement");

    viCheckParm(rsrtm_GetAttributeViInt32 (instrSession, rep_cap_measurement,
        RSRTM_ATTR_MEAS_CATEGORY, category), 3, "Category");    
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function sets the lower and upper reference levels.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.
/// HIPAR instrSession/The handle identifies a particular instrument session.
/// HIPAR relativeMode/Selects the reference level.
ViStatus _VI_FUNC rsrtm_ConfigureReferenceLevelMode (ViSession instrSession,
                                                     ViInt32 relativeMode)
{
	ViStatus    error = VI_SUCCESS;

    viCheckParm(rsrtm_SetAttributeViInt32(instrSession, NULL,
        RSRTM_ATTR_MEAS_RELATIVE_MODE, relativeMode), 3, "Relative Mode");  

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  Set the lower and upper reference levels for rise and fall time mesurements,
/// HIFN  and the middle reference level used for phase and delay measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.
/// HIPAR instrSession/The handle identifies a particular instrument session.
/// HIPAR lowerLevel/Set the lower reference level for rise and fall time mesurements.
/// HIPAR upperLevel/Set the upper reference level for rise and fall time mesurements.
/// HIPAR middleLevel/Set the middle reference level used for phase and delay measurements.
ViStatus _VI_FUNC rsrtm_ConfigureUserReferenceLevel (ViSession instrSession,
                                                     ViReal64 lowerLevel,
                                                     ViReal64 upperLevel,
                                                     ViReal64 middleLevel)
{
	ViStatus    error = VI_SUCCESS;

    viCheckParm(rsrtm_SetAttributeViReal64 (instrSession, NULL,
        RSRTM_ATTR_MEAS_RELATIVE_LOWER_LEVEL, lowerLevel), 2, "Lower Level");  
    
    viCheckParm(rsrtm_SetAttributeViReal64 (instrSession, NULL,
        RSRTM_ATTR_MEAS_RELATIVE_UPPER_LEVEL, upperLevel), 3, "Upper Level");
    
    viCheckParm(rsrtm_SetAttributeViReal64 (instrSession, NULL,
        RSRTM_ATTR_MEAS_RELATIVE_MIDDLE_LEVEL, middleLevel), 4, "Middle Level");    
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function configures the cursor settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR cursorState/Switches the indicated cursor on or off.
/// HIPAR source/Defines the source of the cursor measurement.
/// HIPAR type/Defines the cursor measurement type.
ViStatus _VI_FUNC rsrtm_ConfigureCursor(
    ViSession   instrSession,
    ViBoolean   cursorState,
    ViInt32     source,
    ViInt32     type
)
{
    ViStatus    error = VI_SUCCESS;

    viCheckParm(rsrtm_SetAttributeViBoolean(instrSession, NULL,
        RSRTM_ATTR_CURSOR_ENABLED, cursorState), 2, "Cursor State");  
    
    viCheckParm(rsrtm_SetAttributeViInt32(instrSession, NULL,
        RSRTM_ATTR_CURSOR_SOURCE, source), 3, "Cursor Source");
    
    viCheckParm(rsrtm_SetAttributeViInt32(instrSession, NULL,
        RSRTM_ATTR_CURSOR_MEASUREMENT_TYPE, type), 4, "Type");    
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function switches all cursors off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.  The handle identifies a particular instrument session.
ViStatus _VI_FUNC rsrtm_AllCursorsOff(
    ViSession   instrSession
)
{
    ViStatus    error = VI_SUCCESS;

    viCheckParm(rsrtm_SetAttributeViString(instrSession, "",
        RSRTM_ATTR_ALL_CURSORS_OFF, VI_NULL), 2, "All Cursors Off");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function configures the horizontal and vertical position of selected cursor or cursor line.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR cursor/This control selects the cursor.
/// HIPAR xPosition/Defines X position of selected cursor.
/// HIPAR xPosition/Defines Y position of selected cursor.
ViStatus _VI_FUNC rsrtm_ConfigureCursorPosition (ViSession instrSession,
                                                 ViInt32 cursor,
                                                 ViReal64 xPosition,
                                                 ViReal64 yPosition)
{
    ViStatus    error = VI_SUCCESS;
    ViChar  rep_cap[15] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsrtm_GetAttributeRepCapName (instrSession,
                                               RSRTM_ATTR_CURSOR_X_POSITION,
                                               "CPosition",
                                               cursor - 1,
                                               2,
                                               rep_cap), 2, "Cursor");

    viCheckParm(rsrtm_SetAttributeViReal64(instrSession, rep_cap,
        RSRTM_ATTR_CURSOR_X_POSITION, xPosition), 3, "X Position"); 
    
    viCheckParm(rsrtm_SetAttributeViReal64(instrSession, rep_cap,
        RSRTM_ATTR_CURSOR_Y_POSITION, yPosition), 4, "Y Position");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function configures the positioning mode of the horizontal and vertical cursors.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR xCoupling/Defines the positioning mode of the vertical cursor.
/// HIPAR yCoupling/Defines the positioning mode of the horizontal cursor.
ViStatus _VI_FUNC rsrtm_ConfigureCursorCoupling(
    ViSession   instrSession,
    ViBoolean   xCoupling,
    ViBoolean   yCoupling
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, NULL));

    viCheckParm(rsrtm_SetAttributeViBoolean(instrSession, NULL,
        RSRTM_ATTR_CURSOR_X_POSITION_COUPLING, xCoupling), 2, "X Coupling");  
    
    viCheckParm(rsrtm_SetAttributeViBoolean(instrSession, NULL,
        RSRTM_ATTR_CURSOR_Y_POSITION_COUPLING, yCoupling), 3, "Y Coupling");  
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN This function resets the cursor to the initial position or sets the cursor lines to typical points of the waveform.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function. 
/// HIPAR instrSession/The handle identifies a particular instrument session.
/// HIPAR cursor/This control selects the cursor.
/// HIPAR position/Selects if the cursor position resets or autosets.
ViStatus _VI_FUNC rsrtm_SetCursorPosition (ViSession instrSession,
                                           ViInt32 cursor, ViInt32 position)
{
	ViStatus    error = VI_SUCCESS;
	ViChar      rep_cap[15] = "";
	
	if (rsrtm_invalidViInt32Range (position, 0, 1) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Position");
    }
	
	viCheckParm (rsrtm_GetAttributeRepCapName (instrSession,
                                                 RSRTM_ATTR_CURSOR_AUTOSET,
                                                 "Cursor",
                                                 cursor,
                                                 15,
                                                 rep_cap), 2, "Cursor");

	switch(position) 
    {
        case RSRTM_VAL_CURSOR_POSITION_AUTOSET:
            viCheckErr(rsrtm_SetAttributeViString (instrSession, "", RSRTM_ATTR_CURSOR_AUTOSET, VI_NULL));   
            
        break;
            
        case RSRTM_VAL_CURSOR_POSITION_RESET:
            viCheckErr(rsrtm_SetAttributeViString(instrSession, "", RSRTM_ATTR_CURSOR_RESET, VI_NULL));   
        break;
            
        default: 
        break;
    }
   
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN This function sets the cursor to the next or the previous peak.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.
/// HIPAR instrSession/The handle identifies a particular instrument session.
/// HIPAR cursor/This control selects the cursor.
/// HIPAR peak/This control selects if the cursor is set to the next or the previous peak.
ViStatus _VI_FUNC rsrtm_SetFFTAnalysisCursorToPeak (ViSession instrSession,
                                                    ViInt32 cursor, ViInt32 peak)
{
	ViStatus    error = VI_SUCCESS;
	ViChar      rep_cap[15] = "";
	
	if (rsrtm_invalidViInt32Range (peak, 0, 1) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Position");
    }
	
	viCheckParm (rsrtm_GetAttributeRepCapName (instrSession,
                                                 RSRTM_ATTR_CURSOR_AUTOSET,
                                                 "Cursor",
                                                 cursor,
                                                 15,
                                                 rep_cap), 2, "Cursor");

	switch(peak) 
    {
        case RSRTM_VAL_CURSOR_PEAK_NEXT:
            viCheckErr (rsrtm_SetAttributeViString(instrSession, "", RSRTM_ATTR_CURSOR_PEAK_NEXT, VI_NULL));   
            
        break;
        case RSRTM_VAL_CURSOR_PEAK_PREVIOUS:
            viCheckErr (rsrtm_SetAttributeViString(instrSession, "", RSRTM_ATTR_CURSOR_PEAK_PREVIOUS, VI_NULL));   
        break;
        default: 
        break;
    }
   
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function configures the cursor waveform tracking.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR tracking/Switches the cursor waveform tracking.
ViStatus _VI_FUNC rsrtm_ConfigureCursorWaveformTracking(
    ViSession   instrSession,
    ViBoolean   tracking
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm(rsrtm_SetAttributeViBoolean(instrSession, NULL,
        RSRTM_ATTR_CURSOR_TRACKING, tracking), 3, "Cursor Tracking");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function configures the adjustment of cursor lines if the vertical or horizontal scales are changed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR adjustment/Switches the adjustment of cursor lines if the vertical or horizontal scales are changed.
ViStatus _VI_FUNC rsrtm_ConfigureCursorAdjustment (ViSession instrSession,
                                                     ViBoolean adjustment)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm(rsrtm_SetAttributeViBoolean (instrSession, NULL,
        RSRTM_ATTR_CURSOR_ADJUSTMENT, adjustment), 2, "Adjustment");   
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function sets the unit for horizontal or vertical ratio measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR direction/Selects the distance direction
/// HIPAR measurementUnit/Returns the delta value (distance) of two horizontal or vertical cursors (RTM) / cursor lines (RTO).
ViStatus _VI_FUNC rsrtm_ConfigureCursorRatioMeasurementUnit (ViSession instrSession,
                                                               ViInt32 direction,
                                                               ViInt32 measurementUnit)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    switch(direction) 
    {
        case RSRTM_VAL_CURSOR_HORIZONTAL:
            viCheckParm(rsrtm_SetAttributeViInt32 (instrSession, NULL,
            RSRTM_ATTR_CURSOR_X_RATIO_MEASUREMENT_UNIT, measurementUnit), 3, "Measurement Unit");   
            
        break;
            
        case RSRTM_VAL_CURSOR_VERTICAL:
            viCheckParm(rsrtm_SetAttributeViInt32 (instrSession, NULL,
            RSRTM_ATTR_CURSOR_Y_RATIO_MEASUREMENT_UNIT, measurementUnit), 3, "Measurement Unit");   
        break;
            
        default: 
        break;
    }
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}



/// HIFN  This function returns the ratio of the x-values (e.g. a duty cycle) between the first and second cursors and the first and third cursors
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR direction/Selects the distance direction
/// HIPAR ratioMeasurement/Returns the ratio.
ViStatus _VI_FUNC rsrtm_QueryCursorRatioMeasurement (ViSession instrSession,
                                                       ViInt32 direction,
                                                       ViReal64 *ratioMeasurement)
{
    ViStatus    error = VI_SUCCESS;
    ViChar  rep_cap[15] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    switch(direction) 
    {
        case RSRTM_VAL_CURSOR_HORIZONTAL:
            viCheckParm(rsrtm_GetAttributeViReal64(instrSession, rep_cap,
                RSRTM_ATTR_CURSOR_X_RATIO_MEASUREMENT, ratioMeasurement), 3, "Ratio Measurement");   
        break;
            
        case RSRTM_VAL_CURSOR_VERTICAL:
            viCheckParm(rsrtm_GetAttributeViReal64(instrSession, rep_cap,
                RSRTM_ATTR_CURSOR_Y_RATIO_MEASUREMENT, ratioMeasurement), 3, "Ratio Measurement");   
        break;
            
        default: 
        break;
    }
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function queries the delta value (distance) of two horizontal or vertical cursors (RTM) / cursor lines (RTO).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR cursorDistance/Returns the delta value (distance) of two horizontal or vertical cursors (RTM) / cursor lines (RTO).
/// HIPAR direction/Selects the distance direction
ViStatus _VI_FUNC rsrtm_QueryCursorDistance (ViSession instrSession,
                                             ViInt32 direction,
                                               ViReal64 *cursorDistance)
{
    ViStatus    error = VI_SUCCESS;
    ViChar  rep_cap[15] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsrtm_GetAttributeRepCapName (instrSession,
                                                 RSRTM_ATTR_CURSOR_DISTANCE,
                                                 "CursorDirection",
                                                 direction,
                                                 15,
                                                 rep_cap), 2, "Direction");
    
    viCheckParm(rsrtm_GetAttributeViReal64(instrSession, rep_cap,
        RSRTM_ATTR_CURSOR_DISTANCE, cursorDistance), 3, "Cursor Distance");   
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function queries the inverse value of the delta value (distance) of the two vertical or horizontal cursors (RTM) / cursor lines (RTO).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR cursorDistance/Returns the inverse value of the delta value (distance) of the two vertical or horizontal cursors (RTM) / cursor lines (RTO).
ViStatus _VI_FUNC rsrtm_QueryCursorInverseDistance(
    ViSession   instrSession,
    ViReal64*   cursorDistance
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm(rsrtm_GetAttributeViReal64(instrSession, NULL,
        RSRTM_ATTR_CURSOR_INVERSE_DISTANCE, cursorDistance), 2, "Cursor Distance");   
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}


/// HIFN  This function queries the inverse value of the voltage difference
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR cursorDistance/Returns the inverse value of the voltage difference.
ViStatus _VI_FUNC rsrtm_QueryCursorVoltageInverseDistance (ViSession instrSession,
                                                             ViReal64 *cursorDistance)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsrtm_GetAttributeViReal64 (instrSession, "Vert", 
        RSRTM_ATTR_CURSOR_VOLTAGE_INVERSE_DISTANCE, cursorDistance), 3, "Cursor Distance");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function queries the cursor positions.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR data/Returns the cursor positions.
ViStatus _VI_FUNC rsrtm_QueryCursorPosition (ViSession instrSession,
                                              ViReal64*  data)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckErr (rsrtm_GetAttributeViReal64 (instrSession, NULL,
        RSRTM_ATTR_CURSOR_POSITION, data));
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}


/// HIFN  This function configures the minimum level by which the waveform must rise or fall so that it will be identified as a maximum or a minimum by the search functions.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR peakExcursion/Defines the minimum level by which the waveform must rise or fall so that it will be identified as a maximum or a minimum by the search functions.
ViStatus _VI_FUNC rsrtm_ConfigureCursorPeakExcursion(
    ViSession   instrSession,
    ViInt32 peakExcursion
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm(rsrtm_SetAttributeViInt32(instrSession, NULL,
        RSRTM_ATTR_CURSOR_PEAK_EXCURSION, peakExcursion), 2, "Peak Excursion");   
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  Defines the main measurement. This measurement is the one referred to 
/// HIFN  if the measurement waveform is used as a source for math calculations. 
/// HIFN  The main measurement type must belong to the same category as the other 
/// HIFN  types assigned to the same measurement waveform, if there are any.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR measurementFunction/Defines or queries the main measurement. This measurement is the one 
/// HIPAR measurementFunction/referred to if the measurement waveform is used as a source for math 
/// HIPAR measurementFunction/calculations. The main measurement type must belong to the same 
/// HIPAR measurementFunction/category as the other types assigned to the same measurement waveform, 
/// HIPAR measurementFunction/if there are any.
ViStatus _VI_FUNC rsrtm_ConfigureAmplitudeTimeMainMeasurement(
    ViSession   instrSession,
    ViInt32 measurement,
    ViInt32 measurementFunction
)
{
    ViStatus error = VI_SUCCESS;
    ViChar   rep_cap[15] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsrtm_GetAttributeRepCapName (instrSession,
                                                 RSRTM_ATTR_AMPT_MAIN_MEASUREMENT,
                                                 "Measurement",
                                                 measurement - 1,
                                                 15,
                                                 rep_cap), 2, "Measurement");

    viCheckParm(rsrtm_SetAttributeViInt32(instrSession, rep_cap,
        RSRTM_ATTR_AMPT_MAIN_MEASUREMENT, measurementFunction), 3, "Amplitude Time Measurement Function");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  Defines the main measurement. This measurement is the one referred to 
/// HIFN  if the measurement waveform is used as a source for math calculations. 
/// HIFN  The main measurement type must belong to the same category as the other 
/// HIFN  types assigned to the same measurement waveform, if there are any.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR measurementFunction/Defines or queries the main measurement. This measurement is the one 
/// HIPAR measurementFunction/referred to if the measurement waveform is used as a source for math 
/// HIPAR measurementFunction/calculations. The main measurement type must belong to the same 
/// HIPAR measurementFunction/category as the other types assigned to the same measurement waveform, 
/// HIPAR measurementFunction/if there are any.
ViStatus _VI_FUNC rsrtm_ConfigureAmplitudeTimeMainMeasurementRTM(
    ViSession   instrSession,
    ViInt32 measurement,
    ViInt32 measurementFunction)
{
    ViStatus error = VI_SUCCESS;
    ViChar      model [RSRTM_IO_BUFFER_SIZE] = ""; 
    ViChar   rep_cap[15] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckErr (rsrtm_GetAttributeViString (instrSession, "",
        RS_ATTR_INSTRUMENT_MODEL, RSRTM_IO_BUFFER_SIZE, model));

    if (strstr (model, "RTO"))
        viCheckErr (RS_ERROR_INSTRUMENT_MODEL);
    
    if (rsrtm_invalidViInt32Range (measurement, RSRTM_VAL_MEASUREMENT_1, 
                                                  RSRTM_VAL_MEASUREMENT_8) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Measurement");
    }
    viCheckParm (rsrtm_GetAttributeRepCapName (instrSession,
                                                 RSRTM_ATTR_AMPT_MAIN_MEASUREMENT,
                                                 "Measurement",
                                                 measurement - 1,
                                                 15,
                                                 rep_cap), 2, "Measurement");

    viCheckParm(rsrtm_SetAttributeViInt32(instrSession, rep_cap,
        RSRTM_ATTR_AMPT_MAIN_MEASUREMENT, measurementFunction), 3, "Amplitude Time Measurement Function");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function turns the mask test mode on or off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR state/This Control Turns the mask test mode on or off.
ViStatus _VI_FUNC rsrtm_ConfigureMaskTestEnable (ViSession instrSession,
                                          ViBoolean state)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsrtm_SetAttributeViBoolean (instrSession, "",
        RSRTM_ATTR_MASK_ENABLED, state), 3, "State");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN Configures the actions to be taken when a violation against the
/// HIFN mask limits occurs. For each action, you can define when and how
/// HIFN often the action will be executed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR action/Selects configured action.
/// HIPAR execution/Sets when and how often the action will be executed.
/// HIPAR violationCount/Sets the number of mask violations after which the action is executed.
ViStatus _VI_FUNC rsrtm_ConfigureMaskTestAction (ViSession instrSession,
                                                 ViInt32 action,
                                                 ViInt32 execution,
                                                 ViInt32 violationCount)
{
    ViStatus    error = VI_SUCCESS;
    ViChar    rep_cap[15] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsrtm_GetAttributeRepCapName (instrSession,
                                                 RSRTM_ATTR_MASK_TEST_ACTION_EXECUTION,
                                                 "MaskAction",
                                                 action,
                                                 15,
                                                 rep_cap), 2, "Action");
	
	viCheckParm (rsrtm_SetAttributeViInt32 (instrSession, rep_cap,
        RSRTM_ATTR_MASK_TEST_ACTION_EXECUTION, execution), 3, "Execution");
	
	if ((execution == RSRTM_VAL_MASK_ACTION_EXECUTION_SINGLE) || (execution == RSRTM_VAL_MASK_ACTION_EXECUTION_CYCLIC))
	{
		viCheckParm (rsrtm_SetAttributeViInt32 (instrSession, rep_cap,
	        RSRTM_ATTR_MASK_TEST_VIOLATION_COUNT, violationCount), 4, "Violation Count");
	}
	
Error:
	Rs_UnlockSession(instrSession, VI_NULL);
	return error;
}

/// HIFN  Creates a mask from the envelope waveform of the test source set with MASK: SOURce.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsrtm_MaskCopyChannel (ViSession instrSession)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckErr (rsrtm_SetAttributeViString (instrSession, NULL,
        RSRTM_ATTR_MASK_COPY_CHANNEL, VI_NULL ));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  Creates a mask from the envelope waveform of the test source set with MASK: SOURce.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR totalCount/This Output shows the number of tested acquisitions.
ViStatus _VI_FUNC rsrtm_QueryMaskAcquisitions (ViSession instrSession,
                                        ViInt32 *totalCount)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsrtm_GetAttributeViInt32 (instrSession, NULL,
        RSRTM_ATTR_MASK_COUNT, totalCount), 2, "Total Count");
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}


/// HIFN  This function returns the number of acquistions that hit the mask.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR acquistions/This Control shows the number of acquistions.
ViStatus _VI_FUNC rsrtm_QueryMaskHitCount (ViSession instrSession,
                                           ViInt32 *acquistions)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsrtm_GetAttributeViInt32 (instrSession, NULL,
        RSRTM_ATTR_MASK_HIT_COUNT, acquistions), 2, "Acquistions");
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function loads a stored mask from the specified file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR name/Defines the file with stored mask.
ViStatus _VI_FUNC rsrtm_MaskLoad (ViSession instrSession, ViString name)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsrtm_SetAttributeViString (instrSession, NULL,
        RSRTM_ATTR_MASK_LOAD, name), 2, "Name");
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function saves the current mask in the specified file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR name/Defines the file.
ViStatus _VI_FUNC rsrtm_MaskStore (ViSession instrSession,
                                        ViString name)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsrtm_SetAttributeViString (instrSession, NULL,
        RSRTM_ATTR_MASK_STORE, name), 2, "Name");
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function changes the width of the mask in vertical direction.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR yPosition/This control defines the offset.
ViStatus _VI_FUNC rsrtm_ConfigureMaskVerticalOffset (ViSession instrSession,
                                          ViReal64 yPosition)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsrtm_SetAttributeViReal64 (instrSession, NULL,
        RSRTM_ATTR_MASK_VERTICAL_OFFSET, yPosition), 2, "yPosition ");
    
Error:
    return error;
}

/// HIFN  This function changes the width of the mask in vertical direction.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR yStretch/This control defines stretch or compress.
ViStatus _VI_FUNC rsrtm_ConfigureMaskVerticalScaling (ViSession instrSession,
                                          ViReal64 yStretch)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsrtm_SetAttributeViReal64 (instrSession, NULL,
        RSRTM_ATTR_MASK_VERTICAL_SCALING, yStretch), 2, "Y Stretch");
    
Error:
    return error;
}

/// HIFN  This function changes the width of the mask in vertical direction.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR yWidth/This control defines the shift.
ViStatus _VI_FUNC rsrtm_ConfigureMaskVerticalWidth (ViSession instrSession,
                                          ViReal64 yWidth)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsrtm_SetAttributeViReal64 (instrSession, NULL,
        RSRTM_ATTR_MASK_VERTICAL_WIDTH, yWidth), 3, "Y Width");

Error:
    return error;
}

/// HIFN  This function changes the width of the mask in vertical direction.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR xWidth/This control defines the width.
ViStatus _VI_FUNC rsrtm_ConfigureMaskHorizontalWidth (ViSession instrSession,
                                          ViReal64 xWidth)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsrtm_SetAttributeViReal64 (instrSession, NULL,
        RSRTM_ATTR_MASK_HORIZONTAL_WIDTH, xWidth), 2, "X Width");

Error:
    return error;
}

/// HIFN  Configures mask test parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR action/Selects the action.
ViStatus _VI_FUNC rsrtm_ConfigureMaskTest (ViSession instrSession,
                                         ViInt32 channel,
                                         ViInt32 action)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsrtm_SetAttributeViInt32 (instrSession, "",
        RSRTM_ATTR_MASK_SOURCE, channel), 2, "Channel");
    viCheckParm (rsrtm_SetAttributeViInt32 (instrSession, "",
        RSRTM_ATTR_MASK_TEST, action), 3, "Action");
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN Sets the counters of passed and failed acquisitions to Zero.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsrtm_MaskResetTestCounter (ViSession instrSession)
{
	ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckErr (rsrtm_SetAttributeViString (instrSession, "",
        RSRTM_ATTR_MASK_RESET_TEST_COUNTER, NULL));

Error:
	Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  This function enables and disables the search mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function. 
/// HIPAR state/enables and disables the search mode. 
ViStatus _VI_FUNC rsrtm_ConfigureSearchState (ViSession instrSession,
                                              ViBoolean state)
{
	ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsrtm_SetAttributeViBoolean (instrSession, "",
        RSRTM_ATTR_SEARCH_STATE, state), 2, "State");
   
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
} 

/// HIFN  Selects the symptom you want to search for.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function. 
/// HIPAR action/Selects the symptom.
ViStatus _VI_FUNC rsrtm_ConfigureSearchCondition (ViSession instrSession,
                                                  ViInt32 action)
{
	ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsrtm_SetAttributeViInt32 (instrSession, "",
        RSRTM_ATTR_SEARCH_CONDITION, action), 2, "Action");
   
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  Selects the waveform to be analyzed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.
/// HIPAR searchSource/Selects the waveform to be analyzed.
ViStatus _VI_FUNC rsrtm_ConfigureSearchSource (ViSession instrSession,
                                               ViInt32 searchSource)
{
	ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsrtm_SetAttributeViInt32 (instrSession, "",
        RSRTM_ATTR_SEARCH_SOURCE, searchSource), 2, "Search Source");
   
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
} 

/// HIFN  This function configures the conditions for the edge trigger search.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.
/// HIPAR triggerSlope/Sets the slope to be found.
/// HIPAR triggerLevel/Sets the voltage level for the edge search.
/// HIPAR triggerHystersis/Sets a hysteresis range above and below the search level.
ViStatus _VI_FUNC rsrtm_ConfigureEdgeTriggerSearch (ViSession instrSession,
                                                    ViInt32 triggerSlope,
                                                    ViReal64 triggerLevel,
                                                    ViReal64 triggerHystersis)
{
	ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsrtm_SetAttributeViInt32 (instrSession, "",
        RSRTM_ATTR_SEARCH_TRIGGER_EDGE_SLOPE, triggerSlope), 2, "Trigger Slope");
	
	viCheckParm (rsrtm_SetAttributeViReal64 (instrSession, "",
        RSRTM_ATTR_SEARCH_TRIGGER_EDGE_LEVEL, triggerLevel), 3, "Trigger Level");
	
	viCheckParm (rsrtm_SetAttributeViReal64 (instrSession, "",
        RSRTM_ATTR_SEARCH_TRIGGER_EDGE_HYSTERSIS, triggerHystersis), 4, "Trigger Hystersis");
   
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}


/// HIFN  This function configures the conditions for the width trigger search.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.
/// HIPAR level/Sets the voltage level on which the pulse width is measured.
/// HIPAR levelDelta/Defines a range around the width value.
/// HIPAR width/Sets the reference pulse width, the nominal value for comparisons.
/// HIPAR widthDelta/Sets a range delta t to the reference pulse width.
/// HIPAR polarity/Sets the polarity to be found. 
/// HIPAR range/Sets how the measured pulse width is compared with the given limit(s).
ViStatus _VI_FUNC rsrtm_ConfigureWidthTriggerSearch (ViSession instrSession,
                                                     ViReal64 level,
                                                     ViReal64 levelDelta,
                                                     ViReal64 width,
                                                     ViReal64 widthDelta,
                                                     ViInt32 polarity,
                                                     ViInt32 range)
{
	ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));
	
	viCheckParm (rsrtm_SetAttributeViReal64 (instrSession, "",
        RSRTM_ATTR_SEARCH_TRIGGER_WIDTH_LEVEL, level), 2, "Level");
	
	viCheckParm (rsrtm_SetAttributeViReal64 (instrSession, "",
        RSRTM_ATTR_SEARCH_TRIGGER_WIDTH_HYSTERSIS, levelDelta), 4, "Level Delta");
	
	viCheckParm (rsrtm_SetAttributeViReal64 (instrSession, "",
        RSRTM_ATTR_SEARCH_TRIGGER_WIDTH_WIDTH, width), 3, "Width");
	
	viCheckParm (rsrtm_SetAttributeViReal64 (instrSession, "",
        RSRTM_ATTR_SEARCH_TRIGGER_WIDTH_DELTA, widthDelta), 4, "Width Delta");
	
	viCheckParm (rsrtm_SetAttributeViInt32 (instrSession, "",
        RSRTM_ATTR_SEARCH_TRIGGER_WIDTH_POLARITY, polarity), 5, "Polarity");
	
	viCheckParm (rsrtm_SetAttributeViInt32 (instrSession, "",
        RSRTM_ATTR_SEARCH_TRIGGER_WIDTH_RANGE, range), 6, "Range");
   
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}


/// HIFN  This function configures the conditions for the peak trigger search.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.
/// HIPAR polarity/Sets the polarity to be found.
/// HIPAR limit/Sets the amplitude limit.
ViStatus _VI_FUNC rsrtm_ConfigurePeakTriggerSearch (ViSession instrSession,
                                                    ViInt32 polarity,
                                                    ViReal64 limit)
{
	ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));
	
	viCheckParm (rsrtm_SetAttributeViInt32 (instrSession, "",
        RSRTM_ATTR_SEARCH_TRIGGER_PEAK_POLARITY, polarity), 2, "Polarity");
	
	viCheckParm (rsrtm_SetAttributeViReal64 (instrSession, "",
        RSRTM_ATTR_SEARCH_TRIGGER_PEAK_LIMIT, limit), 3, "Limit");
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN This function configures the conditions for the rise time trigger search.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.
/// HIPAR polarity/Sets the slope to be found.
/// HIPAR upperLevel/Sets the upper voltage threshold.
/// HIPAR lowerLevel/Sets the lower voltage threshold.
/// HIPAR range/Sets how the measured rise or fall time is compared with the given limit(s).
/// HIPAR time/Sets the reference rise or fall time, the nominal value for comparisons.
/// HIPAR deltaTime/Sets a range delta t to the reference rise/fall time set with
/// HIPAR deltaTime/"Time" if "Range" is set to Within or Outside.
ViStatus _VI_FUNC rsrtm_ConfigureRiseTimeTriggerSearch (ViSession instrSession,
                                                        ViInt32 polarity,
                                                        ViReal64 upperLevel,
                                                        ViReal64 lowerLevel,
                                                        ViInt32 range,
                                                        ViReal64 time,
                                                        ViReal64 deltaTime)
{
	ViStatus    error = VI_SUCCESS;
	
	checkErr (Rs_LockSession (instrSession, VI_NULL));
	
	viCheckParm (rsrtm_SetAttributeViInt32 (instrSession, "",
        RSRTM_ATTR_SEARCH_TRIGGER_RISE_TIME_POLARITY, polarity), 5, "Polarity");
	
	viCheckParm (rsrtm_SetAttributeViReal64 (instrSession, "Upper",
        RSRTM_ATTR_SEARCH_TRIGGER_RISE_TIME_LEVEL, upperLevel), 2, "Upper Level");
	
	viCheckParm (rsrtm_SetAttributeViReal64 (instrSession, "Lower",
        RSRTM_ATTR_SEARCH_TRIGGER_RISE_TIME_LEVEL, lowerLevel), 4, "Lower Level");
	
	viCheckParm (rsrtm_SetAttributeViInt32 (instrSession, "",
        RSRTM_ATTR_SEARCH_TRIGGER_RISE_TIME_RANGE, range), 6, "Range");
	
	viCheckParm (rsrtm_SetAttributeViReal64 (instrSession, "",
        RSRTM_ATTR_SEARCH_TRIGGER_RISE_TIME_REFERENCE_TIME, time), 3, "Time");
	
	viCheckParm (rsrtm_SetAttributeViReal64 (instrSession, "",
        RSRTM_ATTR_SEARCH_TRIGGER_RISE_TIME_DELTA_TIME, deltaTime), 4, "Delta Time");
	
	
   
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN This function configures the conditions for the runt trigger search.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.
/// HIPAR polarity/Sets the slope to be found.
/// HIPAR upperLevel/Sets the upper voltage threshold.
/// HIPAR lowerLevel/Sets the lower voltage threshold.
/// HIPAR range/Sets how the measured pulse width is compared with the given limit(s).
/// HIPAR width/Sets the reference runt pulse width, the nominal value for comparisons.
/// HIPAR deltaWidth/Sets a range delta t to the reference rise/fall time set with 
/// HIPAR deltaWidth/"Width" if "Range" is set to Within or Outside.
ViStatus _VI_FUNC rsrtm_ConfigureRuntTriggerSearch (ViSession instrSession,
                                                    ViInt32 polarity,
                                                    ViReal64 upperLevel,
                                                    ViReal64 lowerLevel,
                                                    ViInt32 range,
                                                    ViReal64 width,
                                                    ViReal64 deltaWidth)
{
	ViStatus    error = VI_SUCCESS;
	
	checkErr (Rs_LockSession (instrSession, VI_NULL));
	
	viCheckParm (rsrtm_SetAttributeViInt32 (instrSession, "",
        RSRTM_ATTR_SEARCH_TRIGGER_RUNT_POLARITY, polarity), 5, "Polarity");
	
	viCheckParm (rsrtm_SetAttributeViReal64 (instrSession, "Upper",
        RSRTM_ATTR_SEARCH_TRIGGER_RUNT_LEVEL, upperLevel), 2, "Upper Level");
	
	viCheckParm (rsrtm_SetAttributeViReal64 (instrSession, "Lower",
        RSRTM_ATTR_SEARCH_TRIGGER_RUNT_LEVEL, lowerLevel), 4, "Lower Level");
	
	viCheckParm (rsrtm_SetAttributeViInt32 (instrSession, "",
        RSRTM_ATTR_SEARCH_TRIGGER_RUNT_RANGE, range), 6, "Range");
	
	viCheckParm (rsrtm_SetAttributeViReal64 (instrSession, "",
        RSRTM_ATTR_SEARCH_TRIGGER_RUNT_WIDTH, width), 3, "Width");
	
	viCheckParm (rsrtm_SetAttributeViReal64 (instrSession, "",
        RSRTM_ATTR_SEARCH_TRIGGER_RUNT_DELTA_WIDTH, deltaWidth), 4, "Delta Width");
	
	
   
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  Configures the data to clock trigger search.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.
/// HIPAR line/Select configured line.
/// HIPAR source/Selects the waveform to be analyzed. One of the active channel, math, and reference waveforms can be searched.
/// HIPAR level/Sets the voltage levels for clock or data signals.
/// HIPAR hysteresis/Sets a hysteresis range to the clock or data levels.
ViStatus _VI_FUNC rsrtm_ConfigureData2ClockTriggerSearchSource (ViSession instrSession,
                                                                ViInt32 line,
                                                                ViInt32 source,
                                                                ViReal64 level,
                                                                ViReal64 hysteresis)
{
    ViStatus    error = VI_SUCCESS;
    ViChar    	rep_cap[15] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsrtm_GetAttributeRepCapName (instrSession,
                                                 RSRTM_ATTR_SEARCH_TRIGGER_D2C_LEVEL,
                                                 "D2CLine",
                                                 line,
                                                 10,
                                                 rep_cap), 2, "Line");	
	
	if (line == RSRTM_VAL_SEARCH_D2C_DATA)
	{
		viCheckParm (rsrtm_SetAttributeViInt32 (instrSession, "",
	        RSRTM_ATTR_SEARCH_SOURCE, source), 3, "Source");
	}
	else
	{
		viCheckParm (rsrtm_SetAttributeViInt32 (instrSession, "",
	        RSRTM_ATTR_SEARCH_TRIGGER_D2C_CLOCK_SOURCE, source), 3, "Source");
	}
		
	viCheckParm (rsrtm_SetAttributeViReal64 (instrSession, rep_cap,
	        RSRTM_ATTR_SEARCH_TRIGGER_D2C_LEVEL, level), 4, "Level");
	
	viCheckParm (rsrtm_SetAttributeViReal64 (instrSession, rep_cap,
	        RSRTM_ATTR_SEARCH_TRIGGER_D2C_HYSTERESIS, hysteresis), 5, "Hysteresis");
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN Configures the data to clock trigger search.
/// HIFN You may use multiple lines for documentation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.
/// HIPAR polarity/Sets the slope to be found on clock signal. It define the time 
/// HIPAR polarity/reference point for the setup and hold time
/// HIPAR setupTime/Sets the minimum time before the clock edge while the data signal must stay steady 
/// HIPAR setupTime/above or below the data level.
/// HIPAR holdTime/Sets the minimum time after the clock edge while the data signal must stay 
/// HIPAR holdTime/steady above or below the data level.
ViStatus _VI_FUNC rsrtm_ConfigureData2ClockTriggerSearch (ViSession instrSession,
                                                          ViInt32 polarity,
                                                          ViReal64 setupTime,
                                                          ViReal64 holdTime)
{
	ViStatus    error = VI_SUCCESS;
    checkErr (Rs_LockSession (instrSession, VI_NULL));
	
	viCheckParm (rsrtm_SetAttributeViInt32 (instrSession, "",
	        RSRTM_ATTR_SEARCH_TRIGGER_D2C_POLARITY, polarity), 2, "Polarity");
		
	viCheckParm (rsrtm_SetAttributeViReal64 (instrSession, "",
	        RSRTM_ATTR_SEARCH_TRIGGER_D2C_SETUP_TIME, setupTime), 3, "Setup Time");
	
	viCheckParm (rsrtm_SetAttributeViReal64 (instrSession, "",
	        RSRTM_ATTR_SEARCH_TRIGGER_D2C_HOLD_TIME, holdTime), 4, "Hold Time");

Error:
	Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN Configures the pattern search source channel. Up to four channels can be used as source. 
/// HIFN For each channel, the threshold level and the hysteresis can be set individually.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.
/// HIPAR source/Selects the channel for which the level and the hysteresis are defined.
/// HIPAR level/Sets the threshold value for the selected source channel.
/// HIPAR hysteresis/Sets a hysteresis range to the level of the selected source channel in order to avoid unwanted search 
/// HIPAR hysteresis/results caused by noise oscillation around the level.
ViStatus _VI_FUNC rsrtm_ConfigurePatternTriggerSearchSource (ViSession instrSession,
                                                             ViInt32   source,
                                                             ViReal64  level,
                                                             ViReal64  hysteresis)
{
	ViStatus    error = VI_SUCCESS;
    ViChar    rep_cap[15] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsrtm_GetAttributeRepCapName (instrSession,
                                                 RSRTM_ATTR_SEARCH_TRIGGER_PATTERN_LEVEL,
                                                 "Channel",
                                                 source,
                                                 5,
                                                 rep_cap), 2, "Source");
	
	viCheckParm (rsrtm_SetAttributeViReal64 (instrSession, rep_cap,
	        RSRTM_ATTR_SEARCH_TRIGGER_PATTERN_LEVEL, level), 2, "Level");
		
	viCheckParm (rsrtm_SetAttributeViReal64 (instrSession, rep_cap,
	        RSRTM_ATTR_SEARCH_TRIGGER_PATTERN_HYSTERESIS, hysteresis), 3, "Hysteresis");

Error:
	Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN Configures the pattern trigger search.
/// HIRET Document return value here.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.
/// HIPAR channelPattern/Sets the state of the source channels used for pattern search.
/// HIPAR combiation/Sets the logical combination of the channel states.
/// HIPAR comparison/Sets the condition how the duration of a steady pattern is compared with the given reference time.
/// HIPAR width/Sets the reference time of a steady pattern, the nominal value for comparisons.
/// HIPAR variation/Sets a range delta t to the reference pattern duration, when COMPARISON  is set to "Within" or "Outside".
ViStatus _VI_FUNC rsrtm_ConfigurePatternTriggerSearch (ViSession instrSession,
                                                       ViString  channelPattern,
                                                       ViInt32   combiation,
                                                       ViInt32   comparison,
                                                       ViReal64  width,
                                                       ViReal64  variation)
{
	ViStatus    error = VI_SUCCESS;
    checkErr (Rs_LockSession (instrSession, VI_NULL));
	
	viCheckParm (rsrtm_SetAttributeViString (instrSession, "",
	        RSRTM_ATTR_SEARCH_TRIGGER_PATTERN, channelPattern), 2, "Channel Pattern");
	
	viCheckParm (rsrtm_SetAttributeViInt32 (instrSession, "",
	        RSRTM_ATTR_SEARCH_TRIGGER_PATTERN_COMBINATION, combiation), 3, "Combiation");
	
	viCheckParm (rsrtm_SetAttributeViInt32 (instrSession, "",
	        RSRTM_ATTR_SEARCH_TRIGGER_PATTERN_COMPARISON, comparison), 4, "Comparison");
		
	viCheckParm (rsrtm_SetAttributeViReal64 (instrSession, "",
	        RSRTM_ATTR_SEARCH_TRIGGER_PATTERN_WIDTH, width), 5, "Width");
	
	if ((comparison == RSRTM_VAL_WITHIN) || (comparison == RSRTM_VAL_WITHIN))
	{
		viCheckParm (rsrtm_SetAttributeViReal64 (instrSession, "",
		        RSRTM_ATTR_SEARCH_TRIGGER_PATTERN_WIDTH_DELTA, variation), 6, "Variation");
	}

Error:
	Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN This function configures the start and end values for the gate.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.
/// HIPAR mode/Defines the search area.
/// HIPAR start/Sets the start time of the search area in relation to the trigger point.
/// HIPAR stop/Sets the end time of the search area in relation to the trigger point.
ViStatus _VI_FUNC rsrtm_ConfigureSearchGate (ViSession instrSession,
                                             ViInt32 mode, ViReal64 start,
                                             ViReal64 stop)
{
	ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));
	
	viCheckParm (rsrtm_SetAttributeViInt32 (instrSession, "",
        			RSRTM_ATTR_SEARCH_GATE_MODE, mode), 2, "Mode");
	
	if (mode == RSRTM_VAL_SEARCH_GATE_USER)
	{
		viCheckParm (rsrtm_SetAttributeViReal64 (instrSession, "",
	        			RSRTM_ATTR_SEARCH_GATE_START, start), 3, "Start");
	
		viCheckParm (rsrtm_SetAttributeViReal64 (instrSession, "",
	        			RSRTM_ATTR_SEARCH_GATE_STOP, stop), 4, "Stop");
	}
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function shows or hides the table of search results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.
/// HIPAR showResult/Shows or hides the table of search results.
ViStatus _VI_FUNC rsrtm_ConfigureShowResultDiagram (ViSession instrSession,
                                                    ViBoolean showResult)
{
	ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));
	
	viCheckParm (rsrtm_SetAttributeViBoolean (instrSession, "",
        RSRTM_ATTR_SEARCH_SHOW_RESULT, showResult), 2, "Show Result"); 
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function returns the number of search results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.
/// HIPAR numberOfResults/Returns the number of search results.
ViStatus _VI_FUNC rsrtm_QueryNumberOfResults (ViSession instrSession,
                                              ViInt32 *numberOfResults)
{
	ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));
	
	viCheckParm (rsrtm_GetAttributeViInt32 (instrSession, "",
        RSRTM_ATTR_SEARCH_NUMBER_OF_RESULTS, numberOfResults), 2, "Number Of Results"); 
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}


/// HIFN  This function returns the result values of the specified search result.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.
/// HIPAR resultNumber/Number of result to be read. 
/// HIPAR xPosition/Returns the X-position (time) of the search result
/// HIPAR yPosition/Returns the Y-position of the search result, currently not relevant. .
/// HIPAR resultType/Returns the type of the search result (Edge, Peak, Width).
/// HIPAR polarity/Returns the slope or polarity of the search result.
/// HIPAR voltageWidth/For peak searches, the value contains the peak voltage.
ViStatus _VI_FUNC rsrtm_QuerySearchResult (ViSession instrSession,
                                           ViInt32 resultNumber,
                                           ViReal64 *xPosition,
                                           ViReal64 *yPosition,
                                           ViInt32 *resultType,
                                           ViInt32 *polarity,
                                           ViReal64 *voltageWidth)
{
	ViStatus error = VI_SUCCESS;
    ViChar   command[RSRTM_IO_BUFFER_SIZE];
    ViChar   *p2buf  = NULL;
    ViChar   *p2data  = NULL;
    ViUInt32 count = 0;
	ViString resultTypeArr[] = {"EDGE","WIDTH","PEAK",VI_NULL};
	ViString polarityArr[] = {"POSITIVE","NEGATIVE","EITHER",VI_NULL}; 

    checkErr (Rs_LockSession (instrSession, VI_NULL));    
    
    sprintf (command, "SEAR:RES%ld?\n", resultNumber);
    checkErr (viWrite (instrSession, (ViBuf) command, (ViUInt32) strlen (command), NULL));
    checkErr (Rs_ReadDataUnknownLength (instrSession, &p2data, &count));
    checkErr (rsrtm_CheckStatus (instrSession));
    
    p2buf = strtok (p2data, ",");
	p2buf = strtok (NULL, ",");
    if (p2buf == NULL)
        checkErr (VI_ERROR_INV_RESPONSE);
    if (xPosition != NULL) *xPosition = atof (p2buf);
    
    p2buf = strtok (NULL, ",");
    if (p2buf == NULL)
        checkErr (VI_ERROR_INV_RESPONSE);
    if (yPosition != NULL) *yPosition = atof (p2buf);

    p2buf = strtok (NULL, ",");
    if (p2buf == NULL)
        checkErr (VI_ERROR_INV_RESPONSE);
    if (resultType != NULL) *resultType = rsrtm_StringIndex (resultTypeArr, p2buf);;

    p2buf = strtok (NULL, ",");
    if (p2buf == NULL)
        checkErr (VI_ERROR_INV_RESPONSE);
    if (polarity != NULL) *polarity = rsrtm_StringIndex (polarityArr, p2buf);;
	
	p2buf = strtok (NULL, ",");
    if (p2buf == NULL)
        checkErr (VI_ERROR_INV_RESPONSE);
    if (voltageWidth != NULL) *voltageWidth = atof (p2buf);

Error:
    if (p2data) free (p2data);
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}



/// HIFN  This function returns all result values of the search.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.
/// HIPAR arraySize/Pass the number of elements in the array parameters.
/// HIPAR xPosition/Returns the X-position (time) of the search result
/// HIPAR yPosition/Returns the Y-position of the search result, currently not relevant. .
/// HIPAR resultType/Returns the type of the search result (Edge, Peak, Width).
/// HIPAR polarity/Returns the slope or polarity of the search result.
/// HIPAR voltageWidth/For peak searches, the value contains the peak voltage.
/// HIPAR numberOfResults/Number of returned results.
ViStatus _VI_FUNC rsrtm_QueryAllSearchResults (ViSession instrSession,
                                               ViInt32 arraySize,
                                               ViReal64 xPosition[],
                                               ViReal64 yPosition[],
                                               ViInt32 resultType[],
                                               ViInt32 polarity[],
                                               ViReal64 voltageWidth[],
                                               ViInt32 *numberOfResults)
{
	ViStatus error = VI_SUCCESS;
    ViChar   command[RSRTM_IO_BUFFER_SIZE];
    ViChar   *p2buf  = NULL;
    ViChar   *p2data  = NULL;
    ViUInt32 count = 0;
	ViInt32	 i = 0;
	ViString resultTypeArr[] = {"EDGE","WIDTH","PEAK",VI_NULL};
	ViString polarityArr[] = {"POSITIVE","NEGATIVE","EITHER",VI_NULL}; 

    checkErr (Rs_LockSession (instrSession, VI_NULL));    
    
    sprintf (command, "SEAR:RES:ALL?\n");
    checkErr (viWrite (instrSession, (ViBuf) command, (ViUInt32) strlen (command), NULL));
    checkErr (Rs_ReadDataUnknownLength (instrSession, &p2data, &count));
    checkErr (rsrtm_CheckStatus (instrSession));
    
	p2buf = strtok (p2data, ",");
	
    for (i=0; i<arraySize; i++)
	{
		p2buf = strtok (NULL, ",");
	    if (p2buf == NULL)
	        checkErr (VI_ERROR_INV_RESPONSE);
	    xPosition[i] = atof (p2buf);
    
	    p2buf = strtok (NULL, ",");
	    if (p2buf == NULL)
	        checkErr (VI_ERROR_INV_RESPONSE);
	    yPosition[i] = atof (p2buf);

	    p2buf = strtok (NULL, ",");
	    if (p2buf == NULL)
	        checkErr (VI_ERROR_INV_RESPONSE);
	    resultType[i] = rsrtm_StringIndex (resultTypeArr, p2buf);;

	    p2buf = strtok (NULL, ",");
	    if (p2buf == NULL)
	        checkErr (VI_ERROR_INV_RESPONSE);
	    polarity[i] = rsrtm_StringIndex (polarityArr, p2buf);;
	
		p2buf = strtok (NULL, ",");
	    if (p2buf == NULL)
	        checkErr (VI_ERROR_INV_RESPONSE);
	    voltageWidth[i] = atof (p2buf);
		*numberOfResults = i;
		p2buf = strtok (NULL, ",");
		if (p2buf == NULL) break;
	}

Error:
    if (p2data) free (p2data);
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function enables the decoding of the selected bus.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR state/Enables the decoding of the selected bus.
ViStatus _VI_FUNC rsrtm_ConfigureDecodeState(
    ViSession    instrSession,
    ViBoolean    state
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsrtm_SetAttributeViBoolean (instrSession, NULL,
                                                 RSRTM_ATTR_PROTOCOL_STATE, 
                                                 state
                                                 ), 2, "State");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN Selects protocol type.
/// HIFN You may use multiple lines for documentation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR protocol/This control selects protocol type.
ViStatus _VI_FUNC rsrtm_ConfigureBusProtocol (ViSession instrSession,
                                              ViInt32 protocol)
{
    ViStatus    error = VI_SUCCESS;
    
    viCheckParm(rsrtm_SetAttributeViInt32(instrSession, NULL,
        RSRTM_ATTR_PROTOCOL_TYPE, protocol), 2, "Protocol Type");
Error:
    return error;
}

/// HIFN Configuration of the protocol display
/// HIRET Document return value here.
/// HIRET You may use multiple lines for documentation.
/// HIPAR instrSession/Document parameter here.
/// HIPAR format/Sets the decoding format for the display on the screen.
/// HIPAR height/Sets the height of the display on the screen.
/// HIPAR displayLines/Displays the individual bit lines above the decoded bus line.
/// HIPAR verticalPosition/selects the vertical position of the decoded bus signal in divisions on the sreen
ViStatus _VI_FUNC rsrtm_ConfigureProtocolDisplay (ViSession instrSession,
                                                  ViInt32 format, ViInt32 height,
                                                  ViBoolean displayLines,
                                                  ViReal64 verticalPosition)
{
    ViStatus    error = VI_SUCCESS;
    
    viCheckParm(rsrtm_SetAttributeViInt32(instrSession, NULL,
        RSRTM_ATTR_PROTOCOL_DISPLAY_FORMAT, format), 2, "Format");
    viCheckParm(rsrtm_SetAttributeViInt32(instrSession, NULL,
        RSRTM_ATTR_PROTOCOL_DISPLAY_HEIGHT, height), 3, "Height");
    viCheckParm(rsrtm_SetAttributeViBoolean(instrSession, NULL,
        RSRTM_ATTR_PROTOCOL_DISPLAY_LINES, displayLines), 4, "Display Lines");    
    viCheckParm(rsrtm_SetAttributeViReal64(instrSession, NULL,
        RSRTM_ATTR_PROTOCOL_DISPLAY_VERTICAL, verticalPosition), 2, "Vertical Position");    
Error:
    return error;
}

/// HIFN Executes the analysis of all channels that are configured for the selected bus 
/// HIFN and sets the threshold for digitization of analog signals for each channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
ViStatus _VI_FUNC rsrtm_ProtocolFindLevel (ViSession instrSession,
                                           ViInt32 channel)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[15] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsrtm_GetAttributeRepCapName (instrSession,
                                                 RSRTM_ATTR_CHANNEL_OFFSET,
                                                 "Channel", channel,
                                                 15, rep_cap), 2, "Channel");

    viCheckErr (rsrtm_SetAttributeViString (instrSession, rep_cap,
        							RSRTM_ATTR_PROTOCOL_FIND_LEVEL, VI_NULL));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN This function queries the contents of the frame table in block data format.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/This control indicates the allocated memory in Decoded Data.
/// HIPAR decodedData/Returns the contents of the frame table in block data format.
ViStatus _VI_FUNC rsrtm_QueryBusDecodedData (ViSession instrSession,
                                             ViInt32 bufferSize,
                                             ViChar decodedData[])
{
	ViStatus    error = VI_SUCCESS;
	ViChar      buffer [RS_MAX_MESSAGE_BUF_SIZE] = "";
	ViUInt32    len = 0;
	ViChar*     extBuf = NULL;
	ViUInt32    ret_cnt = 0;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));
	
	strcpy (buffer, "BUS:LIST?\n");
	
	checkErr (viWrite (instrSession, (ViBuf) buffer, (ViUInt32) strlen (buffer), NULL));
    checkErr (viRead (instrSession, (ViBuf) buffer, 1, &ret_cnt)); 
    if (buffer[0]=='#')
    {
        /* Disable termination character */
        checkErr (viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_FALSE));
    
        /* Parse header */
        checkErr (viRead (instrSession, (ViBuf) buffer, 1, &ret_cnt));
        buffer[1] = '\0';
        sscanf (buffer, "%ld", &len);
        checkErr (viRead (instrSession, (ViBuf) buffer, len, &ret_cnt));
        buffer[len] = '\0';
        sscanf (buffer, "%ld", &len);
        
        if (len>0)
        {
            checkAlloc (extBuf = (ViChar*) malloc (len));
            /* Read useful data */
            checkErr (viRead (instrSession, (ViBuf) extBuf, len, &ret_cnt));
    
            if (len != ret_cnt)
                viCheckErr (RS_ERROR_UNEXPECTED_RESPONSE);

            /* Flush garbage */
            viScanf (instrSession,"%*[^\r\n]");
		}
	}
	else
		checkErr(VI_ERROR_INV_RESPONSE);
	
	len = (len <= (ViUInt32) bufferSize) ? len : (ViUInt32) bufferSize;
	strncpy (decodedData, extBuf, (size_t) len);
	
	error = rsrtm_CheckStatus (instrSession);


Error:
	if (extBuf) free (extBuf);
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN This function saves the decoded data (frame table) to the specified CSV file (comma-separated list).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR filePath/String containing the storage device, path, and file name.
ViStatus _VI_FUNC rsrtm_SaveBusDecodedData (ViSession instrSession,
                                            ViString filePath)
{
	ViStatus    error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsrtm_SetAttributeViString (instrSession, "",
                                                RSRTM_ATTR_BUS_SAVE_DECODED_DATA,
                                                filePath), 3, "File Path");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function configures source waveform of the data line (SDA) and 
/// HIFN  clock line (SCL).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR line/This control select I2C line.
/// HIPAR waveformSource/Selects the source of the waveform.
ViStatus _VI_FUNC rsrtm_ConfigureI2CSource(
    ViSession    instrSession,
    ViInt32      line,
    ViInt32      waveformSource
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar    rep_cap[15] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsrtm_GetAttributeRepCapName (instrSession,
                                                 RSRTM_ATTR_PROTOCOL_I2C_SOURCE,
                                                 "I2CLine",
                                                 line,
                                                 15,
                                                 rep_cap), 2, "Line");

    viCheckParm (rsrtm_SetAttributeViInt32 (instrSession, rep_cap,
                                               RSRTM_ATTR_PROTOCOL_I2C_SOURCE,
                                               waveformSource
                                               ), 2, "Waveform Source");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function sets the trigger for i2c bus.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR access/Selects access mode.
/// HIPAR addressLength/Sets the length of the slave address.
/// HIPAR deviceAddress/This control sets the address of the slave device.
/// HIPAR triggerMode/Sets the trigger mode for I2C.
/// HIPAR patternLength/This Control defines how many bytes are considered in the trigger condition.
/// HIPAR pattern/This control defines the bit pattern as trigger condition.
/// HIPAR patternOffset/Sets the number of bytes before the first byte of interest, relating to the end of the address bytes.
ViStatus _VI_FUNC rsrtm_ConfigureI2CTriggerSetting (ViSession instrSession,
                                               ViInt32 access,
                                               ViInt32 addressLength,
                                               ViString deviceAddress,
                                               ViInt32 triggerMode,
                                               ViInt32 patternLength,
                                               ViString pattern,
                                               ViInt32 patternOffset)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsrtm_SetAttributeViInt32(instrSession, NULL,
        RSRTM_ATTR_PROTOCOL_I2C_TRIGGER_ACCESS, access), 2, "Access");    
    
    viCheckParm (rsrtm_SetAttributeViInt32(instrSession, NULL,
        RSRTM_ATTR_PROTOCOL_I2C_TRIGGER_ADDRESS_TYPE, addressLength), 3, "Address length");    

    viCheckParm (rsrtm_SetAttributeViString (instrSession, NULL,
        RSRTM_ATTR_PROTOCOL_I2C_TRIGGER_ADDRESS, deviceAddress), 4, "Device Address");    

    viCheckParm (rsrtm_SetAttributeViInt32(instrSession, NULL,
        RSRTM_ATTR_PROTOCOL_I2C_TRIGGER_MODE, triggerMode), 5, "Trigger Mode");    
    
    viCheckParm (rsrtm_SetAttributeViInt32(instrSession, NULL,
        RSRTM_ATTR_PROTOCOL_I2C_TRIGGER_CONDITION_LENGTH, patternLength), 6, "Pattern Length");    
    
    viCheckParm (rsrtm_SetAttributeViString (instrSession, NULL,
        RSRTM_ATTR_PROTOCOL_I2C_TRIGGER_BIT_PATTERN, pattern), 7, "Pattern");    

    viCheckParm (rsrtm_SetAttributeViInt32(instrSession, NULL,
        RSRTM_ATTR_PROTOCOL_I2C_TRIGGER_FIRST_BYTE_OFFSET, patternOffset), 8, "Pattern Offset");    
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function sets a hysteresis range around the trigger level.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR hysteresis/Sets hysteresis range.
ViStatus _VI_FUNC rsrtm_ConfigureTriggerHysteresis (ViSession instrSession,
                                                          ViInt32 trigger,
                                                          ViInt32 hysteresis)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[15] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsrtm_GetAttributeRepCapName (instrSession,
                                                 RSRTM_ATTR_TRIGGER_HYSTERESIS,
                                                 "Trigger",
                                                 trigger,
                                                 15,
                                                 rep_cap), 2, "Trigger");

    viCheckParm (rsrtm_SetAttributeViInt32(instrSession, rep_cap,
        RSRTM_ATTR_TRIGGER_HYSTERESIS, hysteresis), 3, "Hysteresis");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  Select the state for each digital channel. Sets the logical combination of the trigger states of the channels. Sets the trigger point depending on the result of the logical combination of the channel states. 
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR patternSource/This control sets the state for each digital channel.
/// HIPAR patternFunction/Sets the logical combination of the trigger states of the channels.
/// HIPAR patternCondition/This Control sets the trigger point depending on the result of the logical combination of the channel states.
ViStatus _VI_FUNC rsrtm_ConfigurePatternTriggerSource (ViSession instrSession,
                                                         ViString patternSource,
                                                         ViInt32 patternFunction,
                                                         ViBoolean patternCondition)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    checkErr (rsrtm_ConfigureTrigger (instrSession, RSRTM_VAL_TRIGGER_A,
                                      RSRTM_VAL_PATTERN_TRIGGER));
    viCheckParm (rsrtm_SetAttributeViString (instrSession, NULL,
        RSRTM_ATTR_TRIGGER_PATTERN_SOURCE, patternSource), 3, "Pattern Source");    
    
    viCheckParm (rsrtm_SetAttributeViInt32 (instrSession, NULL,
        RSRTM_ATTR_TRIGGER_PATTERN_FUNCTION, patternFunction), 4, "Pattern Function");    

    viCheckParm (rsrtm_SetAttributeViBoolean (instrSession, NULL,
        RSRTM_ATTR_TRIGGER_PATTERN_POINT_STATE, patternCondition), 5, "Pattern Condition");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN Queries the number of decoded frames.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.
/// HIPAR count/Number of decoded frames.
ViStatus _VI_FUNC rsrtm_QueryI2CFrameCount (ViSession instrSession,
                                            ViInt32 *count)
{
	ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsrtm_GetAttributeViInt32 (instrSession, NULL,
				        						RSRTM_ATTR_PROTOCOL_I2C_FRAME_COUNT, 
												count), 2, "Count"); 
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN Queries the overall state of the frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.
/// HIPAR frame/This control selects the frame.
/// HIPAR status/The overall state of the frame.
/// HIPAR frameStart/Time of frame start
/// HIPAR frameStop/Time of frame end.
/// HIPAR addressType/This control show address length.
/// HIPAR address/Value of the frame address.
/// HIPAR RWBit/Value of the R/W bit.
/// HIPAR ackBit/Value of the address acknowledge bit.
ViStatus _VI_FUNC rsrtm_QueryI2CFrameStatus (ViSession instrSession,
                                             ViInt32 frame, ViInt32 *status,
                                             ViReal64 *frameStart,
                                             ViReal64 *frameStop,
                                             ViInt32 *addressType,
                                             ViInt32 *address, ViInt32 *RWBit,
                                             ViInt32 *ackBit)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSRTM_IO_BUFFER_SIZE];

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    checkErr (rsrtm_GetAttributeViString (instrSession, "", RS_ATTR_OPTIONS_LIST, RSRTM_IO_BUFFER_SIZE, buffer));
    
    if (!(rsrtm_CheckOption (instrSession,"K1")))
    {
        viCheckErr (RS_ERROR_INSTRUMENT_OPTION);
    }
        
    viCheckErr (viQueryf (instrSession, "BUS:I2C:FRAM%ld:STAT?", "%s", frame, buffer));
    *status = rsrtm_StringIndex (protocolI2CFrameStatus_Arr, buffer);
    viCheckErr (viQueryf (instrSession, "BUS:I2C:FRAM%ld:STAR?", "%le", frame, frameStart));
    viCheckErr (viQueryf (instrSession, "BUS:I2C:FRAM%ld:STOP?", "%le", frame, frameStop));
    viCheckErr (viQueryf (instrSession, "BUS:I2C:FRAM%ld:AMOD?", "%s", frame, buffer));
    *addressType = rsrtm_StringIndex (protocolI2CAddressType_Arr, buffer);
    viCheckErr (viQueryf (instrSession, "BUS:I2C:FRAM%ld:ADDR?", "%ld", frame, address));
    viCheckErr (viQueryf (instrSession, "BUS:I2C:FRAM%ld:ACC?", "%s", frame, buffer));
    *RWBit = rsrtm_StringIndex (protocolI2CRWBit_Arr, buffer);
    viCheckErr (viQueryf (instrSession, "BUS:I2C:FRAM%ld:AACC?", "%s", frame, buffer));
    *ackBit = rsrtm_StringIndex (protocolI2CAckBit_Arr, buffer);
    
    checkErr (rsrtm_CheckStatus (instrSession));
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN Queries the slave address of the indicated frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.
/// HIPAR frame/This control selects the frame.
/// HIPAR address/Value of the frame slave address.
ViStatus _VI_FUNC rsrtm_QueryI2CFrameSlaveAddress (ViSession instrSession,
                                                   ViInt32 frame,
                                                   ViInt32 *address)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSRTM_IO_BUFFER_SIZE];

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    checkErr (rsrtm_GetAttributeViString (instrSession, "", RS_ATTR_OPTIONS_LIST, RSRTM_IO_BUFFER_SIZE, buffer));
    
    if (!(rsrtm_CheckOption (instrSession,"K1")))
    {
        viCheckErr (RS_ERROR_INSTRUMENT_OPTION);
    }
    
    viCheckErr (viQueryf (instrSession, "BUS:I2C:FRAM%ld:ADEV?", "%ld", frame, address));

    checkErr (rsrtm_CheckStatus (instrSession));
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN Queries the start time of the address field for the indicated frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.
/// HIPAR frame/This control selects the frame.
/// HIPAR addressStart/Time of address start.
ViStatus _VI_FUNC rsrtm_QueryI2CFrameAddressStartTime (ViSession instrSession,
                                                             ViInt32 frame,
                                                             ViReal64 *addressStart)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSRTM_IO_BUFFER_SIZE];

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    checkErr (rsrtm_GetAttributeViString (instrSession, "", RS_ATTR_OPTIONS_LIST, RSRTM_IO_BUFFER_SIZE, buffer));
    
    if (!(rsrtm_CheckOption (instrSession,"K1")))
    {
        viCheckErr (RS_ERROR_INSTRUMENT_OPTION);
    }
    
    viCheckErr (viQueryf (instrSession, "BUS:I2C:FRAM%ld:AST?", "%le", frame, addressStart));

    checkErr (rsrtm_CheckStatus (instrSession));
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN Queries the start time of the address acknowledge bit.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.
/// HIPAR frame/This control selects the frame.
/// HIPAR ackBitStart/Time of ack bit start.
ViStatus _VI_FUNC rsrtm_QueryI2CFrameAddressAckBitStart (ViSession instrSession,
                                                         ViInt32 frame,
                                                         ViReal64 *ackBitStart)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSRTM_IO_BUFFER_SIZE];

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    checkErr (rsrtm_GetAttributeViString (instrSession, "", RS_ATTR_OPTIONS_LIST, RSRTM_IO_BUFFER_SIZE, buffer));
    
    if (!(rsrtm_CheckOption (instrSession,"K1")))
    {
        viCheckErr (RS_ERROR_INSTRUMENT_OPTION);
    }
    
    viCheckErr (viQueryf (instrSession, "BUS:I2C:FRAM%ld:ADBS?", "%le", frame, ackBitStart));

    checkErr (rsrtm_CheckStatus (instrSession));
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN This function checks if address is complete.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.
/// HIPAR frame/This control selects the frame.
/// HIPAR complete/Status of address.
ViStatus _VI_FUNC rsrtm_QueryI2CFrameAddressComplete (ViSession instrSession,
                                                      ViInt32 frame,
                                                      ViBoolean *complete)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSRTM_IO_BUFFER_SIZE];

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    checkErr (rsrtm_GetAttributeViString (instrSession, "", RS_ATTR_OPTIONS_LIST, RSRTM_IO_BUFFER_SIZE, buffer));
    
    if (!(rsrtm_CheckOption (instrSession,"K1")))
    {
        viCheckErr (RS_ERROR_INSTRUMENT_OPTION);
    }
    
	viCheckErr (viQueryf (instrSession, "BUS:I2C:FRAM%ld:ACOM?", "%s", frame, buffer));
    if (strcmp (buffer, "ON") == 0) 
        *complete = VI_TRUE;
    else
        *complete = VI_FALSE;	

    checkErr (rsrtm_CheckStatus (instrSession));
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN This function returns the number of bytes in the specified frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.
/// HIPAR frame/This control selects the frame.
/// HIPAR length/Returns the number of bytes in the specified frame
ViStatus _VI_FUNC rsrtm_QueryI2CFrameLength (ViSession instrSession,
                                             ViInt32 frame, ViInt32 *length)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSRTM_IO_BUFFER_SIZE];

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    checkErr (rsrtm_GetAttributeViString (instrSession, "", RS_ATTR_OPTIONS_LIST, RSRTM_IO_BUFFER_SIZE, buffer));
    
    if (!(rsrtm_CheckOption (instrSession,"K1")))
    {
        viCheckErr (RS_ERROR_INSTRUMENT_OPTION);
    }
    
    viCheckErr (viQueryf (instrSession, "BUS:I2C:FRAM%ld:BCO?", "%ld", frame, length));

    checkErr (rsrtm_CheckStatus (instrSession));
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN Queries the data value of the specified byte.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.
/// HIPAR frame/This control selects the frame.
/// HIPAR byteNumber/This control selects the Byte.
/// HIPAR data/Value of the specified byte
/// HIPAR ackBit/Value of the data acknowledge bit.
ViStatus _VI_FUNC rsrtm_QueryI2CFrameData (ViSession instrSession,
                                           ViInt32 frame, ViInt32 byteNumber,
                                           ViInt32 *data, ViInt32 *ackBit)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSRTM_IO_BUFFER_SIZE];

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    checkErr (rsrtm_GetAttributeViString (instrSession, "", RS_ATTR_OPTIONS_LIST, RSRTM_IO_BUFFER_SIZE, buffer));
    
    if (!(rsrtm_CheckOption (instrSession,"K1")))
    {
        viCheckErr (RS_ERROR_INSTRUMENT_OPTION);
    }
    
    viCheckErr (viQueryf (instrSession, "BUS:I2C:FRAM%ld:BYTE%ld:VAL?", "%ld", frame, byteNumber, data));
    viCheckErr (viQueryf (instrSession, "BUS:I2C:FRAM%ld:BYTE%ld:ACC?", "%s", frame, byteNumber, buffer));
    *ackBit = rsrtm_StringIndex(protocolI2CAckBit_Arr, buffer);
    
    checkErr (rsrtm_CheckStatus (instrSession));
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN Queries the start time of the acknowledge bit of the specified byte.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.
/// HIPAR frame/This control selects the frame.
/// HIPAR byteNumber/This control selects the Byte.
/// HIPAR startTime/Start time of the acknowledge bit of the specified byte
ViStatus _VI_FUNC rsrtm_QueryI2CFrameAcknowledgeByteStart (ViSession instrSession,
                                                           ViInt32 frame,
                                                           ViInt32 byteNumber,
                                                           ViReal64 *startTime)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSRTM_IO_BUFFER_SIZE];

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    checkErr (rsrtm_GetAttributeViString (instrSession, "", RS_ATTR_OPTIONS_LIST, RSRTM_IO_BUFFER_SIZE, buffer));
    
    if (!(rsrtm_CheckOption (instrSession,"K1")))
    {
        viCheckErr (RS_ERROR_INSTRUMENT_OPTION);
    }
    
    viCheckErr (viQueryf (instrSession, "BUS:I2C:FRAM%ld:BYTE%ld:ACKS?", "%le", frame, byteNumber, startTime));
	
    checkErr (rsrtm_CheckStatus (instrSession));
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN Queries the start time of the specified data byte.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.
/// HIPAR frame/This control selects the frame.
/// HIPAR byteNumber/Document parameter here.
/// HIPAR startTime/Start time of the byte.
ViStatus _VI_FUNC rsrtm_QueryI2CFrameDataByteStart (ViSession instrSession,
                                                    ViInt32 frame,
                                                    ViInt32 byteNumber,
                                                    ViReal64 *startTime)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSRTM_IO_BUFFER_SIZE];

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    checkErr (rsrtm_GetAttributeViString (instrSession, "", RS_ATTR_OPTIONS_LIST, RSRTM_IO_BUFFER_SIZE, buffer));
    
    if (!(rsrtm_CheckOption (instrSession,"K1")))
    {
        viCheckErr (RS_ERROR_INSTRUMENT_OPTION);
    }
    
    viCheckErr (viQueryf (instrSession, "BUS:I2C:FRAM%ld:BYTE%ld:STAR?", "%le", frame, byteNumber, startTime));
	
    checkErr (rsrtm_CheckStatus (instrSession));
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN This function checks if data byte is complete.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.
/// HIPAR frame/This control selects the frame.
/// HIPAR byteNumber/This control selects the Byte.
/// HIPAR complete/Indicates if specified data byte is complete.
ViStatus _VI_FUNC rsrtm_QueryI2CFrameDataComplete (ViSession instrSession,
                                                   ViInt32 frame,
                                                   ViInt32 byteNumber,
                                                   ViBoolean *complete)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSRTM_IO_BUFFER_SIZE];

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    checkErr (rsrtm_GetAttributeViString (instrSession, "", RS_ATTR_OPTIONS_LIST, RSRTM_IO_BUFFER_SIZE, buffer));
    
    if (!(rsrtm_CheckOption (instrSession,"K1")))
    {
        viCheckErr (RS_ERROR_INSTRUMENT_OPTION);
    }
    
    viCheckErr (viQueryf (instrSession, "BUS:I2C:FRAM%ld:BYTE%ld:COMP?", "%s", frame, byteNumber, buffer));
    
    if (strcmp(buffer, "ON") == 0) 
        *complete = VI_TRUE;
    else
        *complete = VI_FALSE;
	
    checkErr (rsrtm_CheckStatus (instrSession));
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN Queries the all data words of the specified frame in comma-separated list of decimal values of the data bytes.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.
/// HIPAR frame/This control selects the frame.
/// HIPAR arraySize/Pass the number of values allocated for Data result array.
/// HIPAR data/Data words of the specified frame.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rsrtm_QueryI2CFrameAllData (ViSession instrSession,
                                              ViInt32 frame, ViInt32 arraySize,
                                              ViInt32 data[],
                                              ViInt32 *returnedValues)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSRTM_IO_BUFFER_SIZE];
	ViChar      *p2buf  = NULL;
    ViChar      *extBuf=VI_NULL; 
    ViUInt32    ret_cnt;
    ViInt32     i=0;
    ViInt32     value_count = 0;
	
    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    checkErr (rsrtm_GetAttributeViString (instrSession, "", RS_ATTR_OPTIONS_LIST, RSRTM_IO_BUFFER_SIZE, buffer));
    
    if (!(rsrtm_CheckOption (instrSession,"K1")))
    {
        viCheckErr (RS_ERROR_INSTRUMENT_OPTION);
    }
    
    viCheckErr( viPrintf(instrSession, "BUS:I2C:FRAM%d:DATA?", frame));
    viCheckErr (Rs_ReadDataUnknownLength(instrSession, &extBuf, &ret_cnt)); 
    
    p2buf = strtok(extBuf, ",");
    
    value_count = atoi (p2buf);
    
    arraySize = (arraySize < value_count) ? arraySize : value_count;
    
    p2buf = strtok(NULL, ",");
    
    while (p2buf && (i < arraySize))
    {                  
        data[i] = atol(p2buf);
        p2buf = strtok(NULL, ",");
        i++;
    }
    *returnedValues = i;
	
    checkErr (rsrtm_CheckStatus (instrSession));
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function configures input channel of the selected line.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR line/This control select configured SPI line.
/// HIPAR waveformSource/Selects the source of the waveform.
ViStatus _VI_FUNC rsrtm_ConfigureSPISource(
    ViSession    instrSession,
    ViInt32        line,
    ViInt32        waveformSource
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar        rep_cap[20] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    if (rsrtm_invalidViInt32Range (line, RSRTM_VAL_SPI_CLOCK, RSRTM_VAL_SPI_MOSI) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Line");
    }
    
    viCheckParm (rsrtm_GetAttributeRepCapName (instrSession,
                                                 RSRTM_ATTR_PROTOCOL_SPI_SOURCE,
                                                 "SPILine",
                                                 line,
                                                 20,
                                                 rep_cap), 2, "Line");
    
    viCheckParm (rsrtm_SetAttributeViInt32 (instrSession, rep_cap,
                                               RSRTM_ATTR_PROTOCOL_SPI_SOURCE,
                                               waveformSource
                                               ), 3, "Waveform Source");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function configures whether transmitted data or the slave select 
/// HIFN  signal is high active (high = 1) or low active (low = 1).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR line/This control select configured SPI line.
/// HIPAR polarity/Selects line polarity.
ViStatus _VI_FUNC rsrtm_ConfigureSPIPolarity(
    ViSession    instrSession,
    ViInt32        line,
    ViInt32        polarity
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar        rep_cap[20] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    if (rsrtm_invalidViInt32Range (line, RSRTM_VAL_SPI_CLOCK, RSRTM_VAL_SPI_MOSI) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Line");
    }
    
    viCheckParm (rsrtm_GetAttributeRepCapName (instrSession,
                                                 RSRTM_ATTR_PROTOCOL_SPI_POLARITY,
                                                 "SPILine",
                                                 line,
                                                 20,
                                                 rep_cap), 2, "Line");
    
    if ((line == RSRTM_VAL_SPI_MISO) || (line == RSRTM_VAL_SPI_MOSI))
	{
		viCheckParm (rsrtm_SetAttributeViInt32 (instrSession, rep_cap,
                                               RSRTM_ATTR_PROTOCOL_SPI_MISO_POLARITY,
                                               polarity
                                               ), 3, "Polarity");
	}
	else
	{
		viCheckParm (rsrtm_SetAttributeViInt32 (instrSession, rep_cap,
                                               RSRTM_ATTR_PROTOCOL_SPI_POLARITY,
                                               polarity
                                               ), 3, "Polarity");
	}
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function configures bit count and it's order in message. The 
/// HIFN  display of the decoded signal considers this setting.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR bitOrder/Select if the data of the messages starts with msb or lsb.
/// HIPAR wordLength/Sets the number of bits in a word.
ViStatus _VI_FUNC rsrtm_ConfigureSPIWord(
    ViSession    instrSession,
    ViInt32        bitOrder,
    ViInt32        wordLength
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsrtm_SetAttributeViInt32 (instrSession, NULL,
                                               RSRTM_ATTR_PROTOCOL_SPI_BIT_ORDER,
                                               bitOrder
                                               ), 3, "Bit Order");

    viCheckParm (rsrtm_SetAttributeViInt32 (instrSession, NULL,
                                               RSRTM_ATTR_PROTOCOL_SPI_WORD_LENGTH,
                                               wordLength
                                               ), 4, "Word Length");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN This function sets one of the Rx and Tx lines as trigger source.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.
/// HIPAR line/Select one of the lines as trigger source.
ViStatus _VI_FUNC rsrtm_ConfigureSPITriggerSource (ViSession instrSession,
                                                   ViInt32 line)
{
	ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsrtm_SetAttributeViInt32 (instrSession, NULL,
        		RSRTM_ATTR_PROTOCOL_SPI_TRIGGER_SOURCE, line), 2, "Line");    

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  Specifies the trigger mode for SPI/SSPI protocols.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR mode/This control specifies the trigger mode.
ViStatus _VI_FUNC rsrtm_ConfigureSPITriggerType (ViSession instrSession,
                                                 ViInt32 mode)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsrtm_SetAttributeViInt32 (instrSession, NULL,
        RSRTM_ATTR_PROTOCOL_SPI_TRIGGER_MODE, mode), 2, "Mode");    

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function configure MOSI and MISO trigger conditions.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR offset/Sets the number of bits or words before the first word of interest.
/// HIPAR bitPatternLength/Defines how many bits build up the serial pattern.
/// HIPAR bitPattern/Specifies the trigger bit pattern.
ViStatus _VI_FUNC rsrtm_ConfigureSPITriggerData(
    ViSession    instrSession,
    ViInt32        offset,
    ViInt32    bitPatternLength,
    ViString    bitPattern
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsrtm_SetAttributeViInt32 (instrSession, "",
                                               RSRTM_ATTR_PROTOCOL_SPI_TRIGGER_OFFSET,
                                               offset
                                               ), 3, "Offset");    
    
    viCheckParm (rsrtm_SetAttributeViString (instrSession, "Bit Pattern",
                                                RSRTM_ATTR_PROTOCOL_SPI_BIT_PATTERN,
                                                bitPattern
                                                ), 5, "Bit Pattern");
    
    viCheckParm (rsrtm_SetAttributeViInt32 (instrSession, "Bit Pattern Length",
                                                RSRTM_ATTR_PROTOCOL_SPI_BIT_PATTERN_LENGTH,
                                                bitPatternLength
                                                ), 4, "Bit Pattern Length"); 

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN This function configures input channel of the selected line.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR line/Selects configured SSPI line.
/// HIPAR waveformSource/Sets the waveform of the selected line.
ViStatus _VI_FUNC rsrtm_ConfigureSSPISource (ViSession instrSession,
                                             ViInt32 line,
                                             ViInt32 waveformSource)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap [15] = "";
    
	if (rsrtm_invalidViInt32Range (line, RSRTM_VAL_SPI_CLOCK, RSRTM_VAL_SPI_MOSI) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Line");
    } 
	if (line == RSRTM_VAL_SPI_CSEL)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Line");
    }
	
	viCheckParm (rsrtm_GetAttributeRepCapName (instrSession,
                                                 RSRTM_ATTR_PROTOCOL_SSPI_SOURCE,
                                                 "SPILine",
                                                 line,
                                                 15,
                                                 rep_cap), 2, "Line");
	
    viCheckParm(rsrtm_SetAttributeViInt32(instrSession, rep_cap,
        RSRTM_ATTR_PROTOCOL_SSPI_SOURCE, waveformSource), 3, "Waveform Source");
Error:
    return error;
}

/// HIFN Configures whether transmitted data or the slave select signal is high active (high = 1) or low active (low = 1)..
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR line/Selects configured SSPI line.
/// HIPAR polarity/Selects line polarity.
ViStatus _VI_FUNC rsrtm_ConfigureSSPIPolarity (ViSession instrSession,
                                               ViInt32 line, ViInt32 polarity)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap [15] = "";
	
	if (rsrtm_invalidViInt32Range (line, RSRTM_VAL_SPI_CLOCK, RSRTM_VAL_SPI_MOSI) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Line");
    }
	if (line == RSRTM_VAL_SPI_CSEL)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Line");
    }
	
    viCheckParm (rsrtm_GetAttributeRepCapName (instrSession,
                                                 RSRTM_ATTR_PROTOCOL_SSPI_POLARITY,
                                                 "SPILine",
                                                 line,
                                                 15,
                                                 rep_cap), 2, "Line");
	
    if ((line == RSRTM_VAL_SPI_MISO) || (line == RSRTM_VAL_SPI_MOSI))
	{
		viCheckParm(rsrtm_SetAttributeViInt32(instrSession, rep_cap,
        RSRTM_ATTR_PROTOCOL_SSPI_MISO_POLARITY, polarity), 3, "Polarity");
	}
	else
	{
		viCheckParm(rsrtm_SetAttributeViInt32(instrSession, rep_cap,
        RSRTM_ATTR_PROTOCOL_SSPI_POLARITY, polarity), 3, "Polarity");
	}
Error:
    return error;
}

/// HIFN This function configures the idle time the data and clock lines are low
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR interframeTime/This control configures the interframe time
/// HIPAR bitOrder/Select if the data of the messages starts with msb or lsb
/// HIPAR wordLength/Sets the number of bits in a word.
ViStatus _VI_FUNC rsrtm_ConfigureSSPIPackets (ViSession instrSession,
                                              ViReal64 interframeTime,
                                              ViInt32 bitOrder,
                                              ViInt32 wordLength)
{
    ViStatus    error = VI_SUCCESS;
	
    viCheckParm(rsrtm_SetAttributeViReal64(instrSession, NULL,
        RSRTM_ATTR_PROTOCOL_SSPI_INTERFRAME_TIME, interframeTime), 2, "Interframe Time");
	
    viCheckParm(rsrtm_SetAttributeViInt32(instrSession, NULL,
        RSRTM_ATTR_PROTOCOL_SSPI_BIT_ORDER, bitOrder), 3, "Bit Order");
	
    viCheckParm(rsrtm_SetAttributeViInt32(instrSession, NULL,
        RSRTM_ATTR_PROTOCOL_SSPI_WORD_LENGTH, wordLength), 4, "Word Length");
    
Error:
    return error;
}

/// HIFN Queries the number of decoded frames.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.
/// HIPAR count/Number of decoded frames.
ViStatus _VI_FUNC rsrtm_QuerySPIFrameCount (ViSession instrSession,
                                            ViInt32 *count)
{
	ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsrtm_GetAttributeViInt32 (instrSession, NULL,
                                               RSRTM_ATTR_PROTOCOL_SPI_FRAME_COUNT,
                                               count), 2, "count"); 

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN Queries the number of words in frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.
/// HIPAR frame/This control selects the frame.
/// HIPAR count/Number of words.
ViStatus _VI_FUNC rsrtm_QuerySPIWordCount (ViSession instrSession,
                                           ViInt32 frame, ViInt32 *count)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSRTM_IO_BUFFER_SIZE];

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    checkErr (rsrtm_GetAttributeViString (instrSession, "", RS_ATTR_OPTIONS_LIST, RSRTM_IO_BUFFER_SIZE, buffer));
    
    if (!(rsrtm_CheckOption (instrSession,"K1")))
    {
        viCheckErr (RS_ERROR_INSTRUMENT_OPTION);
    }
        
    viCheckErr (viQueryf (instrSession, "BUS:SPI:FRAM%ld:WCO?", "%ld", frame, count));
    
    checkErr (rsrtm_CheckStatus (instrSession));
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN Queries the overall state of the frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.
/// HIPAR frame/This control selects the frame.
/// HIPAR frameStart/Returns the start time of the specified frame.
/// HIPAR frameStop/Returns the end time of the specified frame.
/// HIPAR status/The overall state of the frame.
ViStatus _VI_FUNC rsrtm_QuerySPIFrameStatus (ViSession instrSession,
                                             ViInt32 frame, ViReal64 *frameStart,
                                             ViReal64 *frameStop,
                                             ViInt32 *status)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSRTM_IO_BUFFER_SIZE];

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    checkErr (rsrtm_GetAttributeViString (instrSession, "", RS_ATTR_OPTIONS_LIST, RSRTM_IO_BUFFER_SIZE, buffer));
    
    if (!(rsrtm_CheckOption (instrSession,"K1")))
    {
        viCheckErr (RS_ERROR_INSTRUMENT_OPTION);
    }
        
    viCheckErr (viQueryf (instrSession, "BUS:SPI:FRAM%ld:STAR?", "%le", frame, frameStart));
    viCheckErr (viQueryf (instrSession, "BUS:SPI:FRAM%ld:STOP?", "%le", frame, frameStop));
    viCheckErr (viQueryf (instrSession, "BUS:SPI:FRAM%ld:STAT?", "%s", frame, buffer));
    *status = rsrtm_StringIndex (protocolSPIFrameStatus_Arr, buffer);
    
    checkErr (rsrtm_CheckStatus (instrSession));
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN Queries the data value of the specified word.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.
/// HIPAR frame/This control selects the frame.
/// HIPAR word/This control selects the word.
/// HIPAR valueStart/Start time of the data word.
/// HIPAR valueStop/Stop time of the data word.
/// HIPAR MOSIValue/Returns the data value of the specified word on the MOSI line.
/// HIPAR MISOValue/Returns the data value of the specified word on the MISO line.
ViStatus _VI_FUNC rsrtm_QuerySPIFrameData (ViSession instrSession,
                                           ViInt32 frame, ViInt32 word,
                                           ViReal64 *valueStart,
                                           ViReal64 *valueStop,
                                           ViInt32 *MOSIValue,
                                           ViInt32 *MISOValue)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSRTM_IO_BUFFER_SIZE];

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    checkErr (rsrtm_GetAttributeViString (instrSession, "", RS_ATTR_OPTIONS_LIST, RSRTM_IO_BUFFER_SIZE, buffer));
    
    if (!(rsrtm_CheckOption (instrSession,"K1")))
    {
        viCheckErr (RS_ERROR_INSTRUMENT_OPTION);
    }
        
    viCheckErr (viQueryf (instrSession, "BUS:SPI:FRAM%ld:WORD%ld:STAR?", "%le", frame, word, valueStart));
    viCheckErr (viQueryf (instrSession, "BUS:SPI:FRAM%ld:WORD%ld:STOP?", "%le", frame, word, valueStop));
    viCheckErr (viQueryf (instrSession, "BUS:SPI:FRAM%ld:WORD%ld:MOSI?", "%ld", frame, word, MOSIValue));
    viCheckErr (viQueryf (instrSession, "BUS:SPI:FRAM%ld:WORD%ld:MISO?", "%ld", frame, word, MISOValue));
    
    checkErr (rsrtm_CheckStatus (instrSession));
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN Queries the data words of the specified frame of the MISO or MOSI line.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.
/// HIPAR line/This control selects line.
/// HIPAR frame/This control selects the frame.
/// HIPAR arraySize/Pass the number of values allocated for Data result array.
/// HIPAR data/Data words of the specified frame.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rsrtm_QuerySPIFrameAllData (ViSession instrSession,
                                              ViInt32 line, ViInt32 frame,
                                              ViInt32 arraySize, ViInt32 data[],
                                              ViInt32 *returnedValues)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSRTM_IO_BUFFER_SIZE];
	ViChar      *p2buf  = NULL; 
    ViChar      *extBuf=VI_NULL; 
    ViUInt32    ret_cnt;
    ViInt32     i=0;
    ViInt32     value_count = 0;
	
    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    checkErr (rsrtm_GetAttributeViString (instrSession, "", RS_ATTR_OPTIONS_LIST, RSRTM_IO_BUFFER_SIZE, buffer));
    
    if (!(rsrtm_CheckOption (instrSession,"K1")))
    {
        viCheckErr (RS_ERROR_INSTRUMENT_OPTION);
    }
	if (rsrtm_invalidViInt32Range (line, RSRTM_VAL_SPI_MISO, RSRTM_VAL_SPI_MOSI) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Line");
    }
        
    if (line == RSRTM_VAL_SPI_MISO)
	{
		viCheckErr( viPrintf(instrSession, "BUS:SPI:FRAM%d:DATA:MISO?", frame));
	}
	else
	{
		viCheckErr( viPrintf(instrSession, "BUS:SPI:FRAM%d:DATA:MOSI?", frame));
	}
	
    viCheckErr (Rs_ReadDataUnknownLength(instrSession, &extBuf, &ret_cnt)); 
    
    p2buf = strtok(extBuf, ",");
    
    value_count = atoi (p2buf);
    
    arraySize = (arraySize < value_count) ? arraySize : value_count;
    
    p2buf = strtok(NULL, ",");
    
    while (p2buf && (i < arraySize))
    {                  
        data[i] = atol(p2buf);
        p2buf = strtok(NULL, ",");
        i++;
    }
    *returnedValues = i;
    
    checkErr (rsrtm_CheckStatus (instrSession));
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function configures the input channels for the transmitter and 
/// HIFN  receiver signals. 
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR waveformSource/Selects the source of the waveform.
ViStatus _VI_FUNC rsrtm_ConfigureUARTSource(
    ViSession    instrSession,
    ViInt32        waveformSource
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsrtm_SetAttributeViInt32 (instrSession, NULL,
                                               RSRTM_ATTR_PROTOCOL_UART_SOURCE,
                                               waveformSource
                                               ), 2, "Waveform Source");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}


/// HIFN  This function configures the input channels for the transmitter and 
/// HIFN  receiver signals. 
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR line/Selects line.
/// HIPAR waveformSource/Selects the source of the waveform.
ViStatus _VI_FUNC rsrtm_ConfigureUARTSourceLine (ViSession instrSession,
                                                 ViInt32 line,
                                                 ViInt32 waveformSource)
{
	ViStatus    error = VI_SUCCESS;
	ViChar      rep_cap [15] = "";

	checkErr (Rs_LockSession (instrSession, VI_NULL));
	
    viCheckParm (rsrtm_GetAttributeRepCapName (instrSession,
                                                 RSRTM_ATTR_PROTOCOL_UART_SOURCE_LINE,
                                                 "UARTLine", line, 15,
                                                 rep_cap), 2, "Line");

    viCheckParm (rsrtm_SetAttributeViInt32 (instrSession, rep_cap,
                                               RSRTM_ATTR_PROTOCOL_UART_SOURCE_LINE,
                                               waveformSource), 2, "Waveform Source");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN This function selects whether transmitted data is high active (high = 1, Positive) or low active (low = 1, Negative).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR polarity/Selects whether transmitted data is high active (high = 1, Positive) or low active (low = 1, Negative).
ViStatus _VI_FUNC rsrtm_ConfigureUARTPolarity (ViSession instrSession,
                                               ViInt32 polarity)
{
    ViStatus    error = VI_SUCCESS;
    viCheckParm(rsrtm_SetAttributeViInt32(instrSession, NULL,
        RSRTM_ATTR_PROTOCOL_UART_POLARITY, polarity), 2, "Polarity");
Error:
    return error;
}

/// HIFN This function configures the UART port.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR interframeTime/This control configures the interframe time.
/// HIPAR wordLength/Sets the number of bits in a word.
/// HIPAR bitRate/Sets the number of transmitted bits per second.
/// HIPAR parity/Selects the optional parity bit that is used for error detection.
/// HIPAR stopBits/Selects the number of stop bits.
ViStatus _VI_FUNC rsrtm_ConfigureUART (ViSession instrSession,
                                       ViReal64 interframeTime,
                                       ViInt32 wordLength, ViInt32 bitRate,
                                       ViInt32 parity, ViInt32 stopBits)
{
    ViStatus    error = VI_SUCCESS;
	
    viCheckParm(rsrtm_SetAttributeViReal64(instrSession, NULL,
        RSRTM_ATTR_PROTOCOL_UART_INTERFRAME_TIME, interframeTime), 2, "Interframe Time");
	
    viCheckParm(rsrtm_SetAttributeViInt32(instrSession, NULL,
        RSRTM_ATTR_PROTOCOL_UART_WORD_LENGTH, wordLength), 2, "Word Length");
	
    viCheckParm(rsrtm_SetAttributeViInt32(instrSession, NULL,
        RSRTM_ATTR_PROTOCOL_UART_BIT_RATE, bitRate), 3, "Bit Rate"); 
	
    viCheckParm(rsrtm_SetAttributeViInt32(instrSession, NULL,
        RSRTM_ATTR_PROTOCOL_UART_PARITY, parity), 4, "Parity"); 
	
    viCheckParm(rsrtm_SetAttributeViInt32(instrSession, NULL,
        RSRTM_ATTR_PROTOCOL_UART_STOP_BITS, stopBits), 5, "Stop Bits"); 
    
Error:
    return error;
}

/// HIFN This function sets one of the Rx and Tx lines as trigger source.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR line/Select one of the lines as trigger source.
ViStatus _VI_FUNC rsrtm_ConfigureUARTTriggerSource (ViSession instrSession,
                                                    ViInt32 line)
{
	ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsrtm_SetAttributeViInt32 (instrSession, NULL,
        				RSRTM_ATTR_PROTOCOL_UART_TRIGGER_SOURCE, line), 2, "Line");    
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function sets the trigger for UART/RS-232 interfaces.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR mode/This control select trigger type.
ViStatus _VI_FUNC rsrtm_ConfigureUARTTriggerType (ViSession instrSession,
                                              ViInt32 mode)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsrtm_SetAttributeViInt32 (instrSession, NULL,
        RSRTM_ATTR_PROTOCOL_UART_TRIGGER_MODE, mode), 2, "Mode");    
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN This function sets trigger pattern for UART/RS-232 interfaces
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR pattern/This control defines the bit pattern as trigger condition.
/// HIPAR patternLength/Defines how many symbols build up the serial pattern.
/// HIPAR patternOffset/Sets the number of symbols before the first symbol of the pattern.
ViStatus _VI_FUNC rsrtm_ConfigureUARTTriggerPattern (ViSession instrSession,
                                                     ViString pattern,
                                                     ViInt32 patternLength,
                                                     ViInt32 patternOffset)
{
    ViStatus    error = VI_SUCCESS;
	
    viCheckParm(rsrtm_SetAttributeViString(instrSession, NULL,
        RSRTM_ATTR_PROTOCOL_UART_TRIGGER_DATA_PATTERN, pattern), 2, "Pattern");
	
    viCheckParm(rsrtm_SetAttributeViInt32(instrSession, NULL,
        RSRTM_ATTR_PROTOCOL_UART_TRIGGER_CONDITION_LENGTH, patternLength), 3, "Pattern Length");
	
    viCheckParm(rsrtm_SetAttributeViInt32(instrSession, NULL,
        RSRTM_ATTR_PROTOCOL_UART_TRIGGER_DATA_POSITION_INDEX_MIN, patternOffset), 4, "Pattern Offset");    
    
Error:
    return error;
}

/// HIFN Queries the number of decoded frames on the RX and TX lines, respectivley.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR line/Select one of the lines as trigger source.
/// HIPAR count/Number of decoded frames on the RX and TX lines, respectively.
ViStatus _VI_FUNC rsrtm_QueryUARTFrameCount (ViSession instrSession,
                                             ViInt32 line, ViInt32 *count)
{
	ViStatus    error = VI_SUCCESS;
	ViChar      rep_cap [15] = "";

	checkErr (Rs_LockSession (instrSession, VI_NULL));
	
    viCheckParm (rsrtm_GetAttributeRepCapName (instrSession,
                                                 RSRTM_ATTR_PROTOCOL_UART_SOURCE_LINE,
                                                 "UARTLine", line, 15,
                                                 rep_cap), 2, "Line");

    viCheckParm (rsrtm_GetAttributeViInt32 (instrSession, rep_cap,
                                               RSRTM_ATTR_PROTOCOL_UART_FRAME_COUNT,
                                               count), 2, "count"); 
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN Queries the number of words in frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR line/Select one of the lines as trigger source.
/// HIPAR frame/This control selects the frame.
/// HIPAR count/Number of words. 
ViStatus _VI_FUNC rsrtm_QueryUARTWordCount (ViSession instrSession,
                                            ViInt32 line, ViInt32 frame,
                                            ViInt32 *count)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSRTM_IO_BUFFER_SIZE];

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    checkErr (rsrtm_GetAttributeViString (instrSession, "", RS_ATTR_OPTIONS_LIST, RSRTM_IO_BUFFER_SIZE, buffer));
    
    if (!(rsrtm_CheckOption (instrSession,"K2")))
    {
        viCheckErr (RS_ERROR_INSTRUMENT_OPTION);
    }
	if (rsrtm_invalidViInt32Range (line, RSRTM_VAL_UART_RX, RSRTM_VAL_UART_TX) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Line");
    }
        
    viCheckErr (viQueryf (instrSession, "BUS:UART:%s:FRAM%ld:WCO?", "%ld", protocolUARTLine_Arr[line], frame, count));
    
    checkErr (rsrtm_CheckStatus (instrSession));
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN Queries the detailed decoded data of the specified word.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR line/Select one of the lines as trigger source.
/// HIPAR frame/This control selects the frame.
/// HIPAR start/Returns the start time of the specified symbol (word).
/// HIPAR status/Returns the status of the specified symbol (word).
/// HIPAR stop/Returns the end time of the specified symbol (word).
/// HIPAR word/This control selects the word.
/// HIPAR value/Decimal value of the specified symbol (word) on the Rx line and Tx line.
ViStatus _VI_FUNC rsrtm_QueryUARTFrameData (ViSession instrSession,
                                            ViInt32 line, ViInt32 frame, ViInt32 word,
                                            ViInt32 *status, ViReal64 *start, 
                                            ViReal64 *stop, ViInt32 *value)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSRTM_IO_BUFFER_SIZE];

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    checkErr (rsrtm_GetAttributeViString (instrSession, "", RS_ATTR_OPTIONS_LIST, RSRTM_IO_BUFFER_SIZE, buffer));
    
    if (!(rsrtm_CheckOption (instrSession,"K2")))
    {
        viCheckErr (RS_ERROR_INSTRUMENT_OPTION);
    }
	if (rsrtm_invalidViInt32Range (line, RSRTM_VAL_UART_RX, RSRTM_VAL_UART_TX) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Line");
    }
        
	viCheckErr (viQueryf (instrSession, "BUS:UART:%s:FRAM%ld:WORD%ld:STAT?", "%s", 
										protocolUARTLine_Arr[line], frame, word, buffer));
    *status = rsrtm_StringIndex (protocolUARTStatus_Arr, buffer);
	
	viCheckErr (viQueryf (instrSession, "BUS:UART:%s:FRAM%ld:WORD%ld:STAR?", "%le", 
										protocolUARTLine_Arr[line], frame, word, start));
	
    viCheckErr (viQueryf (instrSession, "BUS:UART:%s:FRAM%ld:WORD%ld:STOP?", "%le", 
										protocolUARTLine_Arr[line], frame, word, stop));
	
	viCheckErr (viQueryf (instrSession, "BUS:UART:%s:FRAM%ld:WORD%ld:VAL?", "%ld", 
										protocolUARTLine_Arr[line], frame, word, value));
    
	
    checkErr (rsrtm_CheckStatus (instrSession));
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN This function configures source waveform of the data line.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR waveformSource/Sets the waveform of the data line.
ViStatus _VI_FUNC rsrtm_ConfigureLINSource (ViSession instrSession,
                                            ViInt32 waveformSource)
{
	ViStatus    error = VI_SUCCESS;
	
	checkErr (Rs_LockSession (instrSession, VI_NULL));
	
    viCheckParm (rsrtm_SetAttributeViInt32 (instrSession, "",
                                               RSRTM_ATTR_PROTOCOL_LIN_SOURCE,
                                               waveformSource), 2, "Waveform Source"); 
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN This function configures the idle state of the bus. 
/// HIFN The idle state is the rezessive state and corresponds to a logic 1.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR polarity/Selects line polarity.
ViStatus _VI_FUNC rsrtm_ConfigureLINPolarity (ViSession instrSession,
                                              ViInt32 polarity)
{
	ViStatus    error = VI_SUCCESS;
	
	checkErr (Rs_LockSession (instrSession, VI_NULL));
	
    viCheckParm (rsrtm_SetAttributeViInt32 (instrSession, "",
                                               RSRTM_ATTR_PROTOCOL_LIN_POLARITY,
                                               polarity), 2, "Polarity"); 
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN This function configures the number of transmitted bits per second.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR standard/Selects the version of the LIN standard that is used in the DUT.
/// HIPAR bitRate/Sets the number of transmitted bits per second.
ViStatus _VI_FUNC rsrtm_ConfigureLINStandard (ViSession instrSession,
                                              ViInt32 standard, ViInt32 bitRate)
{
	ViStatus    error = VI_SUCCESS;
	
	checkErr (Rs_LockSession (instrSession, VI_NULL));
	
	viCheckParm (rsrtm_SetAttributeViInt32 (instrSession, "",
                                               RSRTM_ATTR_PROTOCOL_LIN_STANDARD,
                                               standard), 2, "Standard");
	
    viCheckParm (rsrtm_SetAttributeViReal64 (instrSession, "",
                                               RSRTM_ATTR_PROTOCOL_LIN_BIT_RATE,
                                               (ViReal64) bitRate), 3, "Bit Rate");
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN This function configures the trigger type for LIN analysis.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR type/Selects the trigger type for LIN analysis.
ViStatus _VI_FUNC rsrtm_ConfigureLINTriggerType (ViSession instrSession,
                                                 ViInt32 type)
{
	ViStatus    error = VI_SUCCESS;
	
	checkErr (Rs_LockSession (instrSession, VI_NULL));
	
	viCheckParm (rsrtm_SetAttributeViInt32 (instrSession, "",
                                               RSRTM_ATTR_PROTOCOL_LIN_TRIGGER_MODE,
                                               type), 2, "Type");
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN This function configures conditions for ID trigger.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR condition/Sets the operator to set a specific identifier or an identifier range.
/// HIPAR identifierPattern/Defines the identifier pattern.
ViStatus _VI_FUNC rsrtm_ConfigureLINTriggerID (ViSession instrSession,
                                               ViInt32 condition,
                                               ViString identifierPattern)
{
	ViStatus    error = VI_SUCCESS;
	
	checkErr (Rs_LockSession (instrSession, VI_NULL));
	
	viCheckParm (rsrtm_SetAttributeViInt32 (instrSession, "ID",
                                               RSRTM_ATTR_PROTOCOL_LIN_TRIGGER_CONDITION,
                                               condition), 2, "Condition");
	
	viCheckParm (rsrtm_SetAttributeViString (instrSession, "",
                                               RSRTM_ATTR_PROTOCOL_LIN_TRIGGER_ID_PATTERN,
                                               identifierPattern), 3, "Identifier Pattern");
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN This function configures conditions for data trigger.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR dataLength/Defines the length of the data pattern - the number of bytes in the pattern.
/// HIPAR dataPattern/Defines the data pattern. The number of bytes in the data pattern is defined with Data Length.
/// HIPAR condition/Sets the comparison condition for data
ViStatus _VI_FUNC rsrtm_ConfigureLINTriggerData (ViSession instrSession,
												 ViInt32 condition,
                                                 ViInt32 dataLength,
                                                 ViString dataPattern)
{
	ViStatus    error = VI_SUCCESS;
	
	checkErr (Rs_LockSession (instrSession, VI_NULL));
	
	viCheckParm (rsrtm_SetAttributeViInt32 (instrSession, "Data",
                                               RSRTM_ATTR_PROTOCOL_LIN_TRIGGER_CONDITION,
                                               condition), 2, "Condition");
	
	viCheckParm (rsrtm_SetAttributeViInt32 (instrSession, "",
                                               RSRTM_ATTR_PROTOCOL_LIN_TRIGGER_DATA_LENGTH,
                                               dataLength), 3, "Data Length");
	
	viCheckParm (rsrtm_SetAttributeViString (instrSession, "",
                                               RSRTM_ATTR_PROTOCOL_LIN_TRIGGER_DATA_PATTERN,
                                               dataPattern), 4, "Data Pattern");
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN This function configures conditions for Error Condition trigger.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR checksum/Triggers on a checksum error. 
/// HIPAR parityError/Triggers on a parity error.
/// HIPAR syncError/Triggers if synchronization caused an error.
ViStatus _VI_FUNC rsrtm_ConfigureLINTriggerErrorConditions (ViSession instrSession,
                                                            ViBoolean checksum,
                                                            ViBoolean parityError,
                                                            ViBoolean syncError)
{
	ViStatus    error = VI_SUCCESS;
	
	checkErr (Rs_LockSession (instrSession, VI_NULL));
	
	viCheckParm (rsrtm_SetAttributeViBoolean (instrSession, "Checksum",
                                               RSRTM_ATTR_PROTOCOL_LIN_TRIGGER_ERROR,
                                               checksum), 2, "Checksum");
	
	viCheckParm (rsrtm_SetAttributeViBoolean (instrSession, "Parity",
                                               RSRTM_ATTR_PROTOCOL_LIN_TRIGGER_ERROR,
                                               parityError), 3, "Parity Error");
	
	viCheckParm (rsrtm_SetAttributeViBoolean (instrSession, "Sync",
                                               RSRTM_ATTR_PROTOCOL_LIN_TRIGGER_ERROR,
                                               syncError), 4, "Sync Error");
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN Queries the number of decoded frames.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR count/Returns number of decoded frames.
ViStatus _VI_FUNC rsrtm_QueryLINFrameCount (ViSession instrSession,
                                            ViInt32 *count)
{
	ViStatus    error = VI_SUCCESS;
	
	checkErr (Rs_LockSession (instrSession, VI_NULL));
	
	viCheckParm (rsrtm_GetAttributeViInt32 (instrSession, "",
                                               RSRTM_ATTR_PROTOCOL_LIN_FRAME_COUNT,
                                               count), 2, "Count"); 
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN Queries the overall state of the frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR frame/This control selects the frame.
/// HIPAR frameStatus/For future use.
/// HIPAR frameStart/Time of frame start.
/// HIPAR frameStop/Time of frame stop
/// HIPAR version/Returns the version of the LIN standard for the specified frame.
ViStatus _VI_FUNC rsrtm_QueryLINFrameStatus (ViSession instrSession,
                                             ViInt32 frame, ViInt32 *frameStatus,
                                             ViReal64 *frameStart,
                                             ViReal64 *frameStop,
                                             ViInt32 *version)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSRTM_IO_BUFFER_SIZE];

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    checkErr (rsrtm_GetAttributeViString (instrSession, "", RS_ATTR_OPTIONS_LIST, RSRTM_IO_BUFFER_SIZE, buffer));
    
    if (!(rsrtm_CheckOption (instrSession,"K3")))
    {
        viCheckErr (RS_ERROR_INSTRUMENT_OPTION);
    }
	
	//for future use
    //viCheckErr (viQueryf (instrSession, "BUS:LIN:FRAM%ld:STAT?", "%s", frame, buffer));
    //*frameStatus = rsrtm_StringIndex (protocolLINFrameStatus_Arr, buffer); 
	frameStatus;
    viCheckErr (viQueryf (instrSession, "BUS:LIN:FRAM%ld:STAR?", "%le", frame, frameStart));
    viCheckErr (viQueryf (instrSession, "BUS:LIN:FRAM%ld:STOP?", "%le", frame, frameStop));
    viCheckErr (viQueryf (instrSession, "BUS:LIN:FRAM%ld:VERS?", "%s", frame, buffer));
    *version = rsrtm_StringIndex (protocolLINVersion_Arr, buffer); 
	
    checkErr (rsrtm_CheckStatus (instrSession));
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN Document your function here.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR frame/This control selects the frame.
/// HIPAR value/Document parameter here.
/// HIPAR parity/Document parameter here.
/// HIPAR status/Document parameter here.
ViStatus _VI_FUNC rsrtm_QueryLINFrameID (ViSession instrSession,
                                         ViInt32 frame, ViInt32 *value,
                                         ViInt32 *parity, ViInt32 *status)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSRTM_IO_BUFFER_SIZE];

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    checkErr (rsrtm_GetAttributeViString (instrSession, "", RS_ATTR_OPTIONS_LIST, RSRTM_IO_BUFFER_SIZE, buffer));
    
    if (!(rsrtm_CheckOption (instrSession,"K3")))
    {
        viCheckErr (RS_ERROR_INSTRUMENT_OPTION);
    }
	
	viCheckErr (viQueryf (instrSession, "BUS:LIN:FRAM%ld:IDV?", "%ld", frame, value));
    viCheckErr (viQueryf (instrSession, "BUS:LIN:FRAM%ld:IDPV?", "%ld", frame, parity));
    viCheckErr (viQueryf (instrSession, "BUS:LIN:FRAM%ld:IDST?", "%s", frame, buffer));
    *status = rsrtm_StringIndex(protocolLINIDStatus_Arr, buffer);
	
    checkErr (rsrtm_CheckStatus (instrSession));
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN Queries the identifier parameters of the frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR frame/This control selects the frame.
/// HIPAR field/This control selects the field which will be queried.
/// HIPAR fieldValue/Returns the value of selected field for the specified frame.
/// HIPAR fieldStatus/Returns the selected field state for the specified frame.
ViStatus _VI_FUNC rsrtm_QueryLINFrameField (ViSession instrSession,
                                            ViInt32 frame, ViInt32 field,
                                            ViInt32 *fieldValue,
                                            ViInt32 *fieldStatus)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSRTM_IO_BUFFER_SIZE];
	ViString    fieldArr[] = {"SY", "CS"};

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    checkErr (rsrtm_GetAttributeViString (instrSession, "", RS_ATTR_OPTIONS_LIST, RSRTM_IO_BUFFER_SIZE, buffer));
    
    if (!(rsrtm_CheckOption (instrSession,"K3")))
    {
        viCheckErr (RS_ERROR_INSTRUMENT_OPTION);
    }
	if (rsrtm_invalidViInt32Range (field, RSRTM_VAL_LIN_FIELD_SYNC, RSRTM_VAL_LIN_FIELD_CHKSUM) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Field");
    }
	
	viCheckErr (viQueryf (instrSession, "BUS:LIN:FRAM%ld:%sV?\n", "%ld", frame, fieldArr[field], fieldValue));
    viCheckErr (viQueryf (instrSession, "BUS:LIN:FRAM%ld:%sST?\n", "%s", frame, fieldArr[field], buffer));
	 *fieldStatus = rsrtm_StringIndex (protocolLINFieldStatus_Arr, buffer); 
	 
    checkErr (rsrtm_CheckStatus (instrSession));
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN Queries the number of data bytes in the specified frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR frame/This control selects the frame.
/// HIPAR byteCount/Returns the number of data bytes in the specified frame.
ViStatus _VI_FUNC rsrtm_QueryLINFrameByteCount (ViSession instrSession,
                                                ViInt32 frame,
                                                ViInt32 *byteCount)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSRTM_IO_BUFFER_SIZE];
	
    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    checkErr (rsrtm_GetAttributeViString (instrSession, "", RS_ATTR_OPTIONS_LIST, RSRTM_IO_BUFFER_SIZE, buffer));
    
    if (!(rsrtm_CheckOption (instrSession,"K3")))
    {
        viCheckErr (RS_ERROR_INSTRUMENT_OPTION);
    }
	
	viCheckErr (viQueryf (instrSession, "BUS:LIN:FRAM%ld:BCO?\n", "%ld", frame, byteCount));
    
    checkErr (rsrtm_CheckStatus (instrSession));
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN Queries the data value of the specified byte.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR frame/This control selects the frame.
/// HIPAR byteNumber/This control selects the Byte.
/// HIPAR data/Value of the specified byte
/// HIPAR statusCode/Returns the data state for the specified frame.
ViStatus _VI_FUNC rsrtm_QueryLINFrameData (ViSession instrSession,
                                           ViInt32 frame, ViInt32 byteNumber,
                                           ViInt32 *data, ViInt32 *statusCode)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSRTM_IO_BUFFER_SIZE];
	
    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    checkErr (rsrtm_GetAttributeViString (instrSession, "", RS_ATTR_OPTIONS_LIST, RSRTM_IO_BUFFER_SIZE, buffer));
    
    if (!(rsrtm_CheckOption (instrSession,"K3")))
    {
        viCheckErr (RS_ERROR_INSTRUMENT_OPTION);
    }
	
	viCheckErr (viQueryf (instrSession, "BUS:LIN:FRAM%ld:BYTE%ld:VAL?", "%ld", frame, byteNumber, data));
    viCheckErr (viQueryf (instrSession, "BUS:LIN:FRAM%ld:BYTE%ld:STAT?", "%s", frame, byteNumber, buffer));
    *statusCode = rsrtm_StringIndex(protocolLINDataStatus_Arr, buffer); 
	 
    checkErr (rsrtm_CheckStatus (instrSession));
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN Queries the data bytes of the specified frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR frame/This control selects the frame.
/// HIPAR arraySize/Pass the number of values allocated for Data result array.
/// HIPAR data/Array of decimal values of the data bytes.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rsrtm_QueryLINFrameAllData (ViSession instrSession,
                                              ViInt32 frame, ViInt32 arraySize,
                                              ViInt32 data[],
                                              ViInt32 *returnedValues)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSRTM_IO_BUFFER_SIZE];
	ViChar      *p2buf  = NULL; 
    ViChar      *extBuf=VI_NULL; 
    ViUInt32    ret_cnt; 
    ViInt32     i=0;
    ViInt32     value_count = 0;
	
    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    checkErr (rsrtm_GetAttributeViString (instrSession, "", RS_ATTR_OPTIONS_LIST, RSRTM_IO_BUFFER_SIZE, buffer));
    
    if (!(rsrtm_CheckOption (instrSession,"K3")))
    {
        viCheckErr (RS_ERROR_INSTRUMENT_OPTION);
    }
	
	viCheckErr( viPrintf(instrSession, "BUS:LIN:FRAM%d:DATA?", frame));
    viCheckErr (Rs_ReadDataUnknownLength (instrSession, &extBuf, &ret_cnt)); 
    
    p2buf = strtok (extBuf, ",");
    
    value_count = atoi (p2buf);
    
    arraySize = (arraySize < value_count) ? arraySize : value_count;
    
    p2buf = strtok(NULL, ",");
    
    while (p2buf && (i < arraySize))
    {                  
        data[i] = atol(p2buf);
        p2buf = strtok(NULL, ",");
        i++;
    }
    *returnedValues = i;
	 
    checkErr (rsrtm_CheckStatus (instrSession));
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN This function sets the event or combination of events to be searched for.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR event/Sets the event or combination of events to be searched for.
ViStatus _VI_FUNC rsrtm_ConfigureLINSearchEvent (ViSession instrSession,
                                                 ViInt32 event)
{
	ViStatus    error = VI_SUCCESS;
	
	checkErr (Rs_LockSession (instrSession, VI_NULL));
	
	viCheckParm (rsrtm_SetAttributeViInt32 (instrSession, "",
                                               RSRTM_ATTR_PROTOCOL_LIN_SEARCH_EVENT,
                                               event), 2, "Event");  
Error:
    Rs_UnlockSession (instrSession, VI_NULL);    
    return error;
}

/// HIFN This function selects the frame type to be searched for.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR frameSetup/Selects the frame type to be searched for.
ViStatus _VI_FUNC rsrtm_ConfigureLINSearchFrameSetup (ViSession instrSession,
                                                      ViInt32 frameSetup)
{
	ViStatus    error = VI_SUCCESS;
	
	checkErr (Rs_LockSession (instrSession, VI_NULL));
	
	viCheckParm (rsrtm_SetAttributeViInt32 (instrSession, "",
                                               RSRTM_ATTR_PROTOCOL_LIN_SEARCH_FRAME_SETUP,
                                               frameSetup), 2, "Frame Setup");  
Error:
    Rs_UnlockSession (instrSession, VI_NULL);    
    return error;
}

/// HIFN This function configures conditions for ID search.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR condition/Sets the operator to set a specific identifier or an identifier range
/// HIPAR identifierPattern/Defines the identifier pattern.
ViStatus _VI_FUNC rsrtm_ConfigureLINSearchID (ViSession instrSession,
                                              ViInt32 condition,
                                              ViString identifierPattern)
{
	ViStatus    error = VI_SUCCESS;
	
	checkErr (Rs_LockSession (instrSession, VI_NULL));
	
	viCheckParm (rsrtm_SetAttributeViInt32 (instrSession, "ID",
                                               RSRTM_ATTR_PROTOCOL_LIN_SEARCH_CONDITION,
                                               condition), 2, "Condition");
	
	viCheckParm (rsrtm_SetAttributeViString (instrSession, "",
                                               RSRTM_ATTR_PROTOCOL_LIN_SEARCH_ID_PATTERN,
                                               identifierPattern), 3, "identifierPattern");
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN This function configures conditions for data search.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR dataLength/Defines the length of the data pattern - the number of bytes in the pattern.
/// HIPAR dataPattern/Defines the data pattern.
/// HIPAR condition/Sets the comparison condition for data.
ViStatus _VI_FUNC rsrtm_ConfigureLINSearchData (ViSession instrSession,
                                                ViInt32 condition,
												ViInt32 dataLength,
                                                ViString dataPattern)
{
	ViStatus    error = VI_SUCCESS;
	
	checkErr (Rs_LockSession (instrSession, VI_NULL));
	
	viCheckParm (rsrtm_SetAttributeViInt32 (instrSession, "Data",
                                               RSRTM_ATTR_PROTOCOL_LIN_SEARCH_CONDITION,
                                               condition), 2, "Condition");
	
	viCheckParm (rsrtm_SetAttributeViInt32 (instrSession, "",
                                               RSRTM_ATTR_PROTOCOL_LIN_SEARCH_DATA_LENGTH,
                                               dataLength), 3, "Data Length");
	
	viCheckParm (rsrtm_SetAttributeViString (instrSession, "",
                                               RSRTM_ATTR_PROTOCOL_LIN_SEARCH_DATA_PATTERN,
                                               dataPattern), 4, "Data Pattern");
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN This function selects the error type to be searched for.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR checksum/Searches for checksum errors.
/// HIPAR parityError/Searches for parity errors.
/// HIPAR syncError/Searches for synchronization errors.
ViStatus _VI_FUNC rsrtm_ConfigureLINSearchErrorConditions (ViSession instrSession,
                                                           ViBoolean checksum,
                                                           ViBoolean parityError,
                                                           ViBoolean syncError)
{
	ViStatus    error = VI_SUCCESS;
	
	checkErr (Rs_LockSession (instrSession, VI_NULL));
	
	viCheckParm (rsrtm_SetAttributeViBoolean (instrSession, "Checksum",
                                               RSRTM_ATTR_PROTOCOL_LIN_SEARCH_ERROR,
                                               checksum), 2, "Checksum");
	
	viCheckParm (rsrtm_SetAttributeViBoolean (instrSession, "Parity",
                                               RSRTM_ATTR_PROTOCOL_LIN_SEARCH_ERROR,
                                               parityError), 3, "Parity Error");
	
	viCheckParm (rsrtm_SetAttributeViBoolean (instrSession, "Sync",
                                               RSRTM_ATTR_PROTOCOL_LIN_SEARCH_ERROR,
                                               syncError), 4, "Sync Error");
Error:
    Rs_UnlockSession (instrSession, VI_NULL);    
    return error;
}

/// HIFN This function configures input channel of the CAN line.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session..
/// HIPAR line/Selects the CAN-High or CAN-Low line. 
/// HIPAR waveformSource/Sets the source of the data line.
ViStatus _VI_FUNC rsrtm_ConfigureCANSource (ViSession instrSession,
                                            ViInt32 line, ViInt32 waveformSource)
{
	ViStatus    error = VI_SUCCESS;
	
	checkErr (Rs_LockSession (instrSession, VI_NULL));
	
	viCheckParm (rsrtm_SetAttributeViInt32 (instrSession, "",
                                               RSRTM_ATTR_PROTOCOL_CAN_LINE_TYPE,
                                               line), 2, "line");
	
	viCheckParm (rsrtm_SetAttributeViInt32 (instrSession, "",
                                               RSRTM_ATTR_PROTOCOL_CAN_SOURCE,
                                               waveformSource), 3, "Waveform Source"); 
Error:
    Rs_UnlockSession (instrSession, VI_NULL);    
    return error;
}

/// HIFN This function configures the number of transmitted bits per second.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR bitRate/Sets the number of transmitted bits per second.
ViStatus _VI_FUNC rsrtm_ConfigureCANBitRate (ViSession instrSession,
                                             ViInt32 bitRate)
{
	ViStatus    error = VI_SUCCESS;
	
	checkErr (Rs_LockSession (instrSession, VI_NULL));
	
	viCheckParm (rsrtm_SetAttributeViReal64 (instrSession, "",
                                               RSRTM_ATTR_PROTOCOL_CAN_BIT_RATE,
                                               (ViReal64)bitRate), 2, "Bit Rate");
Error:
    Rs_UnlockSession (instrSession, VI_NULL);    
    return error;
}

/// HIFN This function configures the position of the sample point within the bit 
/// HIFN in percent of the nominal bit time.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR samplePoint/Sets the position of the sample point within the bit in percent
/// HIPAR samplePoint/of the nominal bit time.
ViStatus _VI_FUNC rsrtm_ConfigureCANSamplePoint (ViSession instrSession,
                                                 ViReal64 samplePoint)
{
	ViStatus    error = VI_SUCCESS;
	
	checkErr (Rs_LockSession (instrSession, VI_NULL));
	
	viCheckParm (rsrtm_SetAttributeViReal64 (instrSession, "",
                                               RSRTM_ATTR_PROTOCOL_CAN_SAMPLE_POINT,
                                               samplePoint), 2, "Sample Point");
Error:
    Rs_UnlockSession (instrSession, VI_NULL);    
    return error;
}

/// HIFN This function configures the trigger type for CAN analysis.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR type/Selects the trigger type for CAN analysis.
/// HIPAR frameType/Selects the frame type which will be used as trigger condition.
ViStatus _VI_FUNC rsrtm_ConfigureCANTriggerType (ViSession instrSession,
                                                 ViInt32 type, ViInt32 frameType)
{
	ViStatus    error = VI_SUCCESS;
	
	checkErr (Rs_LockSession (instrSession, VI_NULL));
	
	viCheckParm (rsrtm_SetAttributeViInt32 (instrSession, "",
                                               RSRTM_ATTR_PROTOCOL_CAN_TRIGGER_MODE,
                                               type), 2, "Type");
	if (type == RSRTM_VAL_CAN_FRAME_TYPE)
	{
		viCheckParm (rsrtm_SetAttributeViInt32 (instrSession, "",
                                               RSRTM_ATTR_PROTOCOL_CAN_TRIGGER_FRAME_TYPE,
	                                           frameType), 3, "Frame Type");
	}
Error:
    Rs_UnlockSession (instrSession, VI_NULL);    
    return error;
}

/// HIFN This function configures conditions for ID trigger.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR IDType/Selects the format of data and remote frames.
/// HIPAR condition/Sets the comparison condition
/// HIPAR identifierPattern/Defines the identifier pattern.
ViStatus _VI_FUNC rsrtm_ConfigureCANTriggerID (ViSession instrSession,
                                               ViInt32 IDType, ViInt32 condition,
                                               ViString identifierPattern)
{
	ViStatus    error = VI_SUCCESS;
	
	checkErr (Rs_LockSession (instrSession, VI_NULL));
	
	viCheckParm (rsrtm_SetAttributeViInt32 (instrSession, "",
                                               RSRTM_ATTR_PROTOCOL_CAN_TRIGGER_ID_TYPE,
                                               IDType), 2, "ID Type");
	
	viCheckParm (rsrtm_SetAttributeViInt32 (instrSession, "ID",
                                               RSRTM_ATTR_PROTOCOL_CAN_TRIGGER_CONDITION,
                                               condition), 3, "Condition");
	
	viCheckParm (rsrtm_SetAttributeViString (instrSession, "",
                                               RSRTM_ATTR_PROTOCOL_CAN_TRIGGER_ID_PATTERN,
                                               identifierPattern), 4, "Identifier Pattern");
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN Queries the number of decoded frames.
/// HIFN You may use multiple lines for documentation.
/// HIRET Document return value here.
/// HIRET You may use multiple lines for documentation.
/// HIPAR instrSession/Document parameter here.
/// HIPAR count/Number of decoded frames.
ViStatus _VI_FUNC rsrtm_QueryCANFrameCount (ViSession instrSession,
                                            ViInt32 *count)
{
	ViStatus    error = VI_SUCCESS;
	
	checkErr (Rs_LockSession (instrSession, VI_NULL));
	
	viCheckParm (rsrtm_GetAttributeViInt32 (instrSession, "",
                                               RSRTM_ATTR_PROTOCOL_CAN_FRAME_COUNT,
                                               count), 2, "Count"); 
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN Queries the overall state of the frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR frame/This control selects the frame.
/// HIPAR frameStatus/The overall state of the frame.
/// HIPAR frameStart/Time of frame start.
/// HIPAR frameStop/Time of frame stop.
/// HIPAR frameType/Returns the frame type of the selected frame.
ViStatus _VI_FUNC rsrtm_QueryCANFrameStatus (ViSession instrSession,
                                             ViInt32 frame, ViInt32 *frameStatus,
                                             ViReal64 *frameStart,
                                             ViReal64 *frameStop,
                                             ViInt32 *frameType)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSRTM_IO_BUFFER_SIZE];

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    checkErr (rsrtm_GetAttributeViString (instrSession, "", RS_ATTR_OPTIONS_LIST, RSRTM_IO_BUFFER_SIZE, buffer));
    
    if (!(rsrtm_CheckOption (instrSession,"K3")))
    {
        viCheckErr (RS_ERROR_INSTRUMENT_OPTION);
    }
	
	viCheckErr (viQueryf (instrSession, "BUS:CAN:FRAM%ld:STAT?","%s", frame, buffer));
    *frameStatus = rsrtm_StringIndex (protocolCANFrameStatus_Arr, buffer);
    
    viCheckErr (viQueryf (instrSession, "BUS:CAN:FRAM%ld:STAR?","%le", frame, frameStart));
    
    viCheckErr (viQueryf (instrSession, "BUS:CAN:FRAM%ld:STOP?","%le", frame, frameStop));
    
    viCheckErr (viQueryf (instrSession, "BUS:CAN:FRAM%ld:TYPE?","%s", frame, buffer));
    *frameType = rsrtm_StringIndex (protocolCANFrameType_Arr, buffer); 
	
    checkErr (rsrtm_CheckStatus (instrSession));
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN Queries the identifier parameters of the frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR frame/This control selects the frame.
/// HIPAR IDType/Returns the identifier type of the selected frame,
/// HIPAR IDValue/Returns the identifier value of the selected frame.
/// HIPAR statusCode/Return the identifier state.
ViStatus _VI_FUNC rsrtm_QueryCANFrameID (ViSession instrSession,
                                         ViInt32 frame, ViInt32 *IDType,
                                         ViInt32 *IDValue, ViInt32 *statusCode)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSRTM_IO_BUFFER_SIZE];

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    checkErr (rsrtm_GetAttributeViString (instrSession, "", RS_ATTR_OPTIONS_LIST, RSRTM_IO_BUFFER_SIZE, buffer));
    
    if (!(rsrtm_CheckOption (instrSession,"K3")))
    {
        viCheckErr (RS_ERROR_INSTRUMENT_OPTION);
    }
	
	viCheckErr (viQueryf (instrSession, "BUS:CAN:FRAM%ld:IDTY?", "%s", frame, buffer));
    *IDType = rsrtm_StringIndex(protocolCANIDType_Arr, buffer);

    viCheckErr (viQueryf (instrSession, "BUS:CAN:FRAM%ld:IDV?", "%ld", frame, IDValue));
    
    viCheckErr (viQueryf (instrSession, "BUS:CAN:FRAM%ld:IDST?", "%s", frame, buffer));
    *statusCode = rsrtm_StringIndex (protocolCANFieldStatus_Arr, buffer);
	
    checkErr (rsrtm_CheckStatus (instrSession));
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN Queries the parameters of selected field of the frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR frame/This control selects the frame.
/// HIPAR field/This control selects the field which will be queried.
/// HIPAR fieldValue/Returns the value of selected field for the specified frame.
/// HIPAR fieldStatus/Returns the field state of the selected frame.
ViStatus _VI_FUNC rsrtm_QueryCANFrameField (ViSession instrSession,
                                            ViInt32 frame, ViInt32 field,
                                            ViInt32 *fieldValue,
                                            ViInt32 *fieldStatus)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSRTM_IO_BUFFER_SIZE];
	ViString    fieldArr[] = {"DLC", "CS", "ACK"};
	
    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    checkErr (rsrtm_GetAttributeViString (instrSession, "", RS_ATTR_OPTIONS_LIST, RSRTM_IO_BUFFER_SIZE, buffer));
    
    if (!(rsrtm_CheckOption (instrSession,"K3")))
    {
        viCheckErr (RS_ERROR_INSTRUMENT_OPTION);
    }
	
	viCheckErr (viQueryf (instrSession, "BUS:CAN:FRAM%ld:%sV?", "%ld", frame, fieldArr[field], fieldValue));
    
    viCheckErr (viQueryf (instrSession, "BUS:CAN:FRAM%ld:%sST?", "%s", frame, fieldArr[field], buffer));
    *fieldStatus = rsrtm_StringIndex (protocolCANFieldStatus_Arr, buffer);
	
    checkErr (rsrtm_CheckStatus (instrSession));
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN Queries the number of data bytes in the specified frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR frame/This control selects the frame.
/// HIPAR byteCount/Returns the number of data bytes in the specified frame.
ViStatus _VI_FUNC rsrtm_QueryCANFrameByteCount (ViSession instrSession,
                                                ViInt32 frame,
                                                ViInt32 *byteCount)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSRTM_IO_BUFFER_SIZE];
	
    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    checkErr (rsrtm_GetAttributeViString (instrSession, "", RS_ATTR_OPTIONS_LIST, RSRTM_IO_BUFFER_SIZE, buffer));
    
    if (!(rsrtm_CheckOption (instrSession,"K3")))
    {
        viCheckErr (RS_ERROR_INSTRUMENT_OPTION);
    }
	
	viCheckErr (viQueryf (instrSession, "BUS:CAN:FRAM%ld:BCO?", "%ld", frame, byteCount));
    
    checkErr (rsrtm_CheckStatus (instrSession));
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN Queries the location of a bit stuffing error of the frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR frame/This control selects the frame.
/// HIPAR errorPosition/Returns the location of a bit stuffing error.
ViStatus _VI_FUNC rsrtm_QueryCANFrameBitStuffingError (ViSession instrSession,
                                                       ViInt32 frame,
                                                       ViReal64 *errorPosition)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSRTM_IO_BUFFER_SIZE];
	
    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    checkErr (rsrtm_GetAttributeViString (instrSession, "", RS_ATTR_OPTIONS_LIST, RSRTM_IO_BUFFER_SIZE, buffer));
    
    if (!(rsrtm_CheckOption (instrSession,"K3")))
    {
        viCheckErr (RS_ERROR_INSTRUMENT_OPTION);
    }
	
	viCheckErr (viQueryf (instrSession, "BUS:CAN:FRAM%ld:BSEP?", "%le", frame, errorPosition));
    
    checkErr (rsrtm_CheckStatus (instrSession));
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN Queries the data value of the specified byte.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR frame/This control selects the frame.
/// HIPAR byteNumber/This control selects the Byte.
/// HIPAR data/Returns the state of the specified byte.
/// HIPAR statusCode/Returns the state of the specified byte.
ViStatus _VI_FUNC rsrtm_QueryCANFrameData (ViSession instrSession,
                                           ViInt32 frame, ViInt32 byteNumber,
                                           ViInt32 *data, ViInt32 *statusCode)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSRTM_IO_BUFFER_SIZE];
	
    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    checkErr (rsrtm_GetAttributeViString (instrSession, "", RS_ATTR_OPTIONS_LIST, RSRTM_IO_BUFFER_SIZE, buffer));
    
    if (!(rsrtm_CheckOption (instrSession,"K3")))
    {
        viCheckErr (RS_ERROR_INSTRUMENT_OPTION);
    }
	
	viCheckErr (viQueryf (instrSession, "BUS:CAN:FRAM%ld:BYTE%ld:VAL?", "%ld", frame, byteNumber, data));
    
    viCheckErr (viQueryf (instrSession, "BUS:CAN:FRAM%ld:BYTE%ld:STAT?", "%s", frame, byteNumber, buffer));
    *statusCode = rsrtm_StringIndex(protocolCANFieldStatus_Arr, buffer);
    
    checkErr (rsrtm_CheckStatus (instrSession));
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN Queries the data bytes of the specified frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR frame/This control selects the frame.
/// HIPAR arraySize/Pass the number of values allocated for Data result array.
/// HIPAR data/Array of decimal values of the data bytes.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rsrtm_QueryCANFrameAllData (ViSession instrSession,
                                              ViInt32 frame, ViInt32 arraySize,
                                              ViInt32 data[],
                                              ViInt32 *returnedValues)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      buffer [RSRTM_IO_BUFFER_SIZE];
	ViChar      *p2buf  = NULL; 
    ViChar      *extBuf=VI_NULL; 
    ViUInt32    ret_cnt; 
    ViInt32     i=0;
    ViInt32     value_count = 0;
	
    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    checkErr (rsrtm_GetAttributeViString (instrSession, "", RS_ATTR_OPTIONS_LIST, RSRTM_IO_BUFFER_SIZE, buffer));
    
    if (!(rsrtm_CheckOption (instrSession,"K3")))
    {
        viCheckErr (RS_ERROR_INSTRUMENT_OPTION);
    }
	
	viCheckErr (viPrintf (instrSession, "BUS:CAN:FRAM%ld:DATA?", frame));
    viCheckErr (Rs_ReadDataUnknownLength (instrSession, &extBuf, &ret_cnt)); 
    
    p2buf = strtok(extBuf, ",");
    
    value_count = atoi (p2buf);
    
    arraySize = (arraySize < value_count) ? arraySize : value_count;
    
    p2buf = strtok(NULL, ",");
    
    while (p2buf && (i < arraySize))
    {                  
        data[i] = atol (p2buf);
        p2buf = strtok (NULL, ",");
        i++;
    }
    *returnedValues = i;
    
    checkErr (rsrtm_CheckStatus (instrSession));
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN This function sets the event or combination of events to be searched for.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR event/Sets the event or combination of events to be searched for.
ViStatus _VI_FUNC rsrtm_ConfigureCANSearchEvent (ViSession instrSession,
                                                 ViInt32 event)
{
	ViStatus    error = VI_SUCCESS;
	
	checkErr (Rs_LockSession (instrSession, VI_NULL));
	
	viCheckParm (rsrtm_SetAttributeViInt32 (instrSession, "",
                                            RSRTM_ATTR_PROTOCOL_CAN_SEARCH_EVENT,
                                            event), 2, "Event");
Error:
    Rs_UnlockSession (instrSession, VI_NULL);    
    return error;
}

/// HIFN This function selects the frame type to be searched for 
/// HIFN if rsrtm_ConfigureCANSearchEvent is set to "Frame".
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR frameSetup/Selects the frame type to be searched for.
ViStatus _VI_FUNC rsrtm_ConfigureCANSearchFrameSetup (ViSession instrSession,
                                                      ViInt32 frameSetup)
{
	ViStatus    error = VI_SUCCESS;
	
	checkErr (Rs_LockSession (instrSession, VI_NULL));
	
	viCheckParm (rsrtm_SetAttributeViInt32 (instrSession, "",
                                               RSRTM_ATTR_PROTOCOL_CAN_SEARCH_FRAME_SETUP,
                                               frameSetup), 2, "Frame Setup");  
Error:
    Rs_UnlockSession (instrSession, VI_NULL);    
    return error;
}

/// HIFN This function specifies the frame type to be searched for 
/// HIFN if rsrtm_ConfigureCANSearchEvent is set to "Identifier".
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR frameType/Specifies the frame type to be searched for.
ViStatus _VI_FUNC rsrtm_ConfigureCANSearchFrameType (ViSession instrSession,
                                                     ViInt32 frameType)
{
	ViStatus    error = VI_SUCCESS;
	
	checkErr (Rs_LockSession (instrSession, VI_NULL));
	
	viCheckParm (rsrtm_SetAttributeViInt32 (instrSession, "",
                                               RSRTM_ATTR_PROTOCOL_CAN_SEARCH_FRAME_TYPE,
                                               frameType), 2, "Frame Type");  
Error:
    Rs_UnlockSession (instrSession, VI_NULL);    
    return error;
}

/// HIFN This function configures conditions for ID search.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR IDType/Selects the length of the identifier
/// HIPAR condition/Sets the comparison condition for the identifier
/// HIPAR identifierPattern/Defines the identifier pattern.
ViStatus _VI_FUNC rsrtm_ConfigureCANSearchID (ViSession instrSession,
                                              ViInt32 IDType, ViInt32 condition,
                                              ViString identifierPattern)
{
	ViStatus    error = VI_SUCCESS;
	
	checkErr (Rs_LockSession (instrSession, VI_NULL));
	
	viCheckParm (rsrtm_SetAttributeViInt32 (instrSession, "",
                                               RSRTM_ATTR_PROTOCOL_CAN_SEARCH_ID_TYPE,
                                               IDType), 2, "ID Type");
	
	viCheckParm (rsrtm_SetAttributeViInt32 (instrSession, "ID",
                                               RSRTM_ATTR_PROTOCOL_CAN_SEARCH_CONDITION,
                                               condition), 3, "Condition");
	
	viCheckParm (rsrtm_SetAttributeViString (instrSession, "",
                                               RSRTM_ATTR_PROTOCOL_CAN_SEARCH_ID_PATTERN,
                                               identifierPattern), 4, "Identifier Pattern");
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN Document your function here.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR condition/Document parameter here.
/// HIPAR dataLength/Document parameter here.
/// HIPAR dataPattern/Document parameter here.
ViStatus _VI_FUNC rsrtm_ConfigureCANSearchData (ViSession instrSession,
                                                ViInt32 condition,
                                                ViInt32 dataLength,
                                                ViString dataPattern)
{
	ViStatus    error = VI_SUCCESS;
	
	checkErr (Rs_LockSession (instrSession, VI_NULL));
	
	viCheckParm (rsrtm_SetAttributeViInt32 (instrSession, "Data",
                                               RSRTM_ATTR_PROTOCOL_CAN_SEARCH_CONDITION,
                                               condition), 2, "Condition");
	
	viCheckParm (rsrtm_SetAttributeViInt32 (instrSession, "",
                                               RSRTM_ATTR_PROTOCOL_CAN_SEARCH_DATA_LENGTH,
                                               dataLength), 3, "Data Length");
	
	viCheckParm (rsrtm_SetAttributeViString (instrSession, "",
                                               RSRTM_ATTR_PROTOCOL_CAN_SEARCH_DATA_PATTERN,
                                               dataPattern), 4, "Data Pattern");
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN This function selects the error type to be searched for. 
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR CRC/Searches for errors in the Cyclic Redundancy Check.
/// HIPAR stuffing/Searches for bit stuffing errors.
/// HIPAR form/Searches for form errors.
/// HIPAR ack/Searches for acknowledgement errors.
ViStatus _VI_FUNC rsrtm_ConfigureCANSearchErrorConditions (ViSession instrSession,
                                                           ViBoolean CRC,
                                                           ViBoolean stuffing,
                                                           ViBoolean form,
                                                           ViBoolean ack)
{
	ViStatus    error = VI_SUCCESS;
	
	checkErr (Rs_LockSession (instrSession, VI_NULL));
	
	viCheckParm (rsrtm_SetAttributeViBoolean (instrSession, "CRC",
                                               RSRTM_ATTR_PROTOCOL_CAN_SEARCH_ERROR,
                                               CRC), 2, "CRC");
	
	viCheckParm (rsrtm_SetAttributeViBoolean (instrSession, "Stuffing",
                                               RSRTM_ATTR_PROTOCOL_CAN_SEARCH_ERROR,
                                               stuffing), 3, "Stuffing");
	
	viCheckParm (rsrtm_SetAttributeViBoolean (instrSession, "Form",
                                               RSRTM_ATTR_PROTOCOL_CAN_SEARCH_ERROR,
                                               form), 4, "Form");
	
	viCheckParm (rsrtm_SetAttributeViBoolean (instrSession, "Ack",
                                               RSRTM_ATTR_PROTOCOL_CAN_SEARCH_ERROR,
                                               ack), 5, "Ack");
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN This function configures conditions for data trigger.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR dataPattern/Defines the data pattern.
/// HIPAR dataLength/Defines the length of the data pattern - the number of bytes in the pattern.
/// HIPAR condition/Sets the comparison condition for data
ViStatus _VI_FUNC rsrtm_ConfigureCANTriggerData (ViSession instrSession,
												 ViInt32 condition,
                                                 ViInt32 dataLength,
												 ViString dataPattern)
{
	ViStatus    error = VI_SUCCESS;
	
	checkErr (Rs_LockSession (instrSession, VI_NULL));
	
	viCheckParm (rsrtm_SetAttributeViInt32 (instrSession, "Data",
                                               RSRTM_ATTR_PROTOCOL_CAN_TRIGGER_CONDITION,
                                               condition), 2, "Condition");
	
	viCheckParm (rsrtm_SetAttributeViInt32 (instrSession, "",
                                               RSRTM_ATTR_PROTOCOL_CAN_TRIGGER_DATA_LENGTH,
                                               dataLength), 3, "Data Length");
	
	viCheckParm (rsrtm_SetAttributeViString (instrSession, "",
                                               RSRTM_ATTR_PROTOCOL_CAN_TRIGGER_DATA_PATTERN,
                                               dataPattern), 4, "Data Pattern");
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN This function configures conditions for Error Condition trigger.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR CRC/Sets state of triggering on the CRC error.
/// HIPAR stuffing/Sets state of triggering on the bit stuffing error.
/// HIPAR form/Sets state of triggering on the form error.
/// HIPAR ack/Sets state of triggering on the acknowledge error.
ViStatus _VI_FUNC rsrtm_ConfigureCANTriggerErrorConditions (ViSession instrSession,
                                                            ViBoolean CRC,
                                                            ViBoolean stuffing,
                                                            ViBoolean form,
                                                            ViBoolean ack)
{
	ViStatus    error = VI_SUCCESS;
	
	checkErr (Rs_LockSession (instrSession, VI_NULL));
	
	viCheckParm (rsrtm_SetAttributeViBoolean (instrSession, "CRC",
                                               RSRTM_ATTR_PROTOCOL_CAN_TRIGGER_ERROR,
                                               CRC), 2, "CRC");
	
	viCheckParm (rsrtm_SetAttributeViBoolean (instrSession, "Stuffing",
                                               RSRTM_ATTR_PROTOCOL_CAN_TRIGGER_ERROR,
                                               stuffing), 3, "Stuffing");
	
	viCheckParm (rsrtm_SetAttributeViBoolean (instrSession, "Form",
                                               RSRTM_ATTR_PROTOCOL_CAN_TRIGGER_ERROR,
                                               form), 4, "Form");
	
	viCheckParm (rsrtm_SetAttributeViBoolean (instrSession, "Ack",
                                               RSRTM_ATTR_PROTOCOL_CAN_TRIGGER_ERROR,
                                               ack), 5, "Ack");
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN This function sets the number of lines to be analyzed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or rsrtm_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR line/This control sets the number of lines to be analyzed.
ViStatus _VI_FUNC rsrtm_ConfigureParallelAnalyzedLines (ViSession instrSession,
                                                        ViInt32 line)
{
    ViStatus    error = VI_SUCCESS;
    viCheckParm(rsrtm_SetAttributeViInt32(instrSession, NULL,
        RSRTM_ATTR_PARALLEL_BUS_ANALYZED_LINES, line), 2, "Line");
Error:
    return error;
}

/// HIFN  This function defines a destination file for the hadcopy output.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR scheme/Defines the color mode for printing.
/// HIPAR pageOrientation/Defines the page orientation.
/// HIPAR printerName/Name of the printer.
/// HIPAR size/Paper size.
ViStatus _VI_FUNC rsrtm_ConfigureHardcopyPrinter(
    ViSession    instrSession,
    ViString     printerName,
    ViInt32      pageOrientation,
    ViInt32      scheme,
    ViInt32      size

)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsrtm_SetAttributeViInt32 (instrSession, NULL,
                                            RSRTM_ATTR_HARDCOPY_DEVICE_DESTINATION,
                                            RSRTM_VAL_HARDCOPY_DEVICE_PRINT), 1, "Device Destination");

    viCheckParm (rsrtm_SetAttributeViString (instrSession, NULL,
                                                RSRTM_ATTR_HARDCOPY_SELECT_PRINTER,
                                                printerName
                                                ), 2, "Printer Name");

    viCheckParm (rsrtm_SetAttributeViInt32 (instrSession, NULL,
                                       RSRTM_ATTR_HARDCOPY_DEVICE_ORIENTATION,
                                       pageOrientation
                                       ), 3, "Page Orientation");
    viCheckParm (rsrtm_SetAttributeViInt32 (instrSession, NULL,
                                                 RSRTM_ATTR_HARDCOPY_COLOR_SCHEME,
                                                 scheme
                                                 ), 4, "Scheme");
    viCheckParm (rsrtm_SetAttributeViInt32 (instrSession, NULL,
        RSRTM_ATTR_HARDCOPY_PAGE_SIZE, size), 5, "Size");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function defines a destination file for the hadcopy output.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR name/File name.
/// HIPAR scheme/Defines the color mode for printing.
/// HIPAR format/Defines the output associated with device configuration.
ViStatus _VI_FUNC rsrtm_ConfigureHardcopyFile(
    ViSession instrSession,
    ViString  name,
    ViInt32   format,
    ViInt32   scheme

)
{
    ViStatus    error = VI_SUCCESS;
    

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsrtm_SetAttributeViInt32 (instrSession, NULL,
                                       RSRTM_ATTR_HARDCOPY_DEVICE_DESTINATION,
                                       RSRTM_VAL_HARDCOPY_DEVICE_MEM
                                       ), 1, "Device Destination");
    viCheckParm (rsrtm_SetAttributeViString (instrSession, "",
                                                RSRTM_ATTR_HARDCOPY_FILE_NAME,
                                                name
                                                ), 2, "Name");

            
    viCheckParm (rsrtm_SetAttributeViInt32 (instrSession, NULL,
                                       RSRTM_ATTR_HARDCOPY_DEVICE_LANGUAGE_OUTPUT_FORMAT,
                                       format
                                       ), 3, "Format");
    
    viCheckParm (rsrtm_SetAttributeViInt32 (instrSession, NULL,
                                                 RSRTM_ATTR_HARDCOPY_COLOR_SCHEME,
                                                 scheme
                                                 ), 4, "Scheme");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function prints an image of the display to the printer or saves 
/// HIFN  an image to a file or the clipboard.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsrtm_HardcopyPrint(
    ViSession    instrSession
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsrtm_SetAttributeViString (instrSession, NULL,
                                                RSRTM_ATTR_HARDCOPY_PRINT,
                                                VI_NULL
                                                ), 1, "Session");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function queries the list of printers under Windows.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/This control indicates the allocated memory in Printer List.
/// HIPAR printerList/This control returns list of printers available under  Windows NT 
/// HIPAR printerList/system. Names are separated using commas (,).
ViStatus _VI_FUNC rsrtm_HardcopyGetPrinterList(
    ViSession    instrSession,
    ViInt32        bufferSize,
    ViChar        printerList[]
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSRTM_IO_BUFFER_SIZE] = "";
    ViChar      *p2buf;
    ViInt32     count;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    p2buf=printerList;
    count=0;     
    
    viCheckErr (rsrtm_GetAttributeViString (instrSession, "", RSRTM_ATTR_HARDCOPY_GET_FIRST_PRINTER,
                RSRTM_IO_BUFFER_SIZE, buffer));
    if (strlen (buffer) != 0)
	{
	    while (strlen (buffer) > 0)
	    {
	        count+=(ViUInt32) strlen(buffer);  
	        count++; //add one position for comma
	        if (count<=bufferSize)
	            p2buf += sprintf (p2buf, "%s,", buffer);
	        viCheckErr (rsrtm_GetAttributeViString (instrSession, "", RSRTM_ATTR_HARDCOPY_GET_NEXT_PRINTER,
	                RSRTM_IO_BUFFER_SIZE, buffer));      
	    }
	    *--p2buf='\0';
    
	    if (count>bufferSize)
	        error = count;     
	}

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function defines an instrument name.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR name/Defines the name.
ViStatus _VI_FUNC rsrtm_ConfigureInstrumentName (ViSession instrSession,
                                          ViString name)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsrtm_SetAttributeViString (instrSession, "",
        RSRTM_ATTR_INSTRUMENT_NAME, name), 2, "Name");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function sets the value of a ViInt32 attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to set the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value to which you want to set the attribute.
ViStatus _VI_FUNC rsrtm_SetAttributeViInt32 (ViSession instrSession, ViString channelName,
                                                ViUInt32 attributeId, ViInt32 value)
{
    return Rs_SetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/// HIFN  This function sets the value of a ViReal64 attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to set the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value to which you want to set the attribute.
ViStatus _VI_FUNC rsrtm_SetAttributeViReal64 (ViSession instrSession, ViString channelName,
                                                 ViUInt32 attributeId, ViReal64 value)
{
     return Rs_SetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/// HIFN  This function sets the value of a ViString attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to set the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value to which you want to set the attribute.
ViStatus _VI_FUNC rsrtm_SetAttributeViString (ViSession instrSession, ViString channelName,
                                                 ViUInt32 attributeId, ViString value)
{
    return Rs_SetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);
}

/// HIFN  This function sets the value of a ViBoolean attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to set the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value to which you want to set the attribute.
ViStatus _VI_FUNC rsrtm_SetAttributeViBoolean (ViSession instrSession, ViString channelName,
                                                  ViUInt32 attributeId, ViBoolean value)
{
    return Rs_SetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/// HIFN  This function sets the value of a ViSession attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to set the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value to which you want to set the attribute.
ViStatus _VI_FUNC rsrtm_SetAttributeViSession (ViSession instrSession, ViString channelName,
                                                  ViUInt32 attributeId, ViSession value)
{
    return Rs_SetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/// HIFN  This function queries the value of a ViInt32 attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to obtain the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Returns the current value of the attribute.  Pass the address of a 
/// HIPAR value/ViInt32 variable.
ViStatus _VI_FUNC rsrtm_GetAttributeViInt32 (ViSession instrSession, ViString channelName,
                                                ViUInt32 attributeId, ViInt32 *value)
{
    return Rs_GetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL,
                            (ViInt32) sizeof (ViInt32), (ViInt32 *)value);
}

/// HIFN  This function queries the value of a ViReal64 attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to obtain the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Returns the current value of the attribute.  Pass the address of a 
/// HIPAR value/ViReal64 variable.
ViStatus _VI_FUNC rsrtm_GetAttributeViReal64 (ViSession instrSession, ViString channelName,
                                                 ViUInt32 attributeId, ViReal64 *value)
{
    return Rs_GetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL,
                            (ViInt32) sizeof (ViReal64), (ViReal64 *)value);
}

/// HIFN  This function queries the value of a ViString attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to obtain the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR bufSize/Pass the number of bytes in the ViChar array you specify for the 
/// HIPAR bufSize/Attribute Value parameter.  
/// HIPAR value/The buffer in which the function returns the current value of the 
/// HIPAR value/attribute.  The buffer must be of type ViChar and have at least as many 
/// HIPAR value/bytes as indicated in the Buffer Size parameter.
ViStatus _VI_FUNC rsrtm_GetAttributeViString (ViSession instrSession, ViString channelName,
                                                 ViUInt32 attributeId, ViInt32 bufSize,
                                                 ViChar value[])
{
    return Rs_GetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL,
                            bufSize, value);
}

/// HIFN  This function queries the value of a ViBoolean attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to obtain the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Returns the current value of the attribute.  Pass the address of a 
/// HIPAR value/ViBoolean variable.
ViStatus _VI_FUNC rsrtm_GetAttributeViBoolean (ViSession instrSession, ViString channelName,
                                                  ViUInt32 attributeId, ViBoolean *value)
{
    return Rs_GetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL,
                            (ViInt32) sizeof (ViBoolean), (ViBoolean *)value);
}

/// HIFN  This function queries the value of a ViSession attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to obtain the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Returns the current value of the attribute.  Pass the address of a 
/// HIPAR value/ViSession variable.
ViStatus _VI_FUNC rsrtm_GetAttributeViSession (ViSession instrSession, ViString channelName,
                                                  ViUInt32 attributeId, ViSession *value)
{
    return Rs_GetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL,
                            (ViInt32) sizeof (ViSession), (ViSession *)value);
}

/// HIFN  This function checks the validity of a value you specify for a ViInt32 
/// HIFN  attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to check the attribute value. If the attribute 
/// HIPAR channelName/is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value which you want to verify as a valid value for the 
/// HIPAR value/attribute.
ViStatus _VI_FUNC rsrtm_CheckAttributeViInt32 (ViSession instrSession, ViString channelName,
                                                  ViUInt32 attributeId, ViInt32 value)
{
    return Rs_CheckAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/// HIFN  This function checks the validity of a value you specify for a 
/// HIFN  ViReal64 attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to check the attribute value. If the attribute 
/// HIPAR channelName/is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value which you want to verify as a valid value for the 
/// HIPAR value/attribute.
ViStatus _VI_FUNC rsrtm_CheckAttributeViReal64 (ViSession instrSession, ViString channelName,
                                                   ViUInt32 attributeId, ViReal64 value)
{
    return Rs_CheckAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/// HIFN  This function checks the validity of a value you specify for a 
/// HIFN  ViString attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to check the attribute value. If the attribute 
/// HIPAR channelName/is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value which you want to verify as a valid value for the 
/// HIPAR value/attribute.
ViStatus _VI_FUNC rsrtm_CheckAttributeViString (ViSession instrSession, ViString channelName,
                                                   ViUInt32 attributeId, ViString value)
{
    return Rs_CheckAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);
}

/// HIFN  This function checks the validity of a value you specify for a 
/// HIFN  ViBoolean attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to check the attribute value. If the attribute 
/// HIPAR channelName/is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value which you want to verify as a valid value for the 
/// HIPAR value/attribute.
ViStatus _VI_FUNC rsrtm_CheckAttributeViBoolean (ViSession instrSession, ViString channelName,
                                                    ViUInt32 attributeId, ViBoolean value)
{
    return Rs_CheckAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/// HIFN  This function checks the validity of a value you specify for a 
/// HIFN  ViSession attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to check the attribute value. If the attribute 
/// HIPAR channelName/is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value which you want to verify as a valid value for the 
/// HIPAR value/attribute.
ViStatus _VI_FUNC rsrtm_CheckAttributeViSession (ViSession instrSession, ViString channelName,
                                                    ViUInt32 attributeId, ViSession value)
{
    return Rs_CheckAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/// HIFN This function returns the repeated capability id(s) that belongs
/// HIFN to the attribute you specify. More then one id is listed in
/// HIFN comma separated string.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.
/// HIPAR attributeID/Pass the ID of an attribute.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the Repeated Capability Id(s) parameter.
/// HIPAR repeatedCapabilityIds/The buffer in which the function returns the repeated capability id(s) belonging to the attribute.
ViStatus _VI_FUNC rsrtm_GetAttributeRepeatedCapabilityIds (ViSession instrSession,
                                                            ViUInt32 attributeID,
                                                            ViInt32 bufferSize,
                                                            ViChar _VI_FAR repeatedCapabilityIds[])
{
    ViStatus    error = VI_SUCCESS;
    ViChar      repCapNameId[RS_MAX_MESSAGE_BUF_SIZE] = "";

    if (bufferSize <= 0)
        viCheckParm(RS_ERROR_INVALID_PARAMETER, 3, "Buffer Size");

    viCheckErr (Rs_GetAttributeRepCapNameId (instrSession, attributeID, repCapNameId));

    strncpy (repeatedCapabilityIds, repCapNameId, RS_MAX_MESSAGE_BUF_SIZE);
    repeatedCapabilityIds[(bufferSize > RS_MAX_MESSAGE_BUF_SIZE) ?
        RS_MAX_MESSAGE_BUF_SIZE : bufferSize] = '\0';

Error:
    return error;
}

/// HIFN This function returns the repeated capability name(s) that
/// HIFN belongs to the attribute and single repeated capability id you
/// HIFN may specify. More then one name is listed in comma separated
/// HIFN string.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.
/// HIPAR attributeID/Pass the ID of an attribute.
/// HIPAR repeatedCapabilityId/If the attribute applies to a repeated capability such as a trace, this parameter defines the Id of the repeated capability on which to query repeated capability id name(s).
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the Repeated Capability Id Name(s) parameter.
/// HIPAR repeatedCapabilityIdNames/The buffer in which the function returns the repeated capability id name(s) belonging to the attribute.
ViStatus _VI_FUNC rsrtm_GetAttributeRepeatedCapabilityIdNames (ViSession instrSession,
                                                                ViUInt32 attributeID,
                                                                ViString repeatedCapabilityId,
                                                                ViInt32 bufferSize,
                                                                ViChar _VI_FAR repeatedCapabilityIdNames[])
{
    ViStatus    error = VI_SUCCESS;
    ViChar      repCapName[RS_MAX_MESSAGE_BUF_SIZE] = "";

    if (bufferSize <= 0)
        viCheckParm(RS_ERROR_INVALID_PARAMETER, 4, "Buffer Size");

    viCheckErr (Rs_GetAttributeRepCapName (instrSession,
                                           attributeID,
                                           repeatedCapabilityId,
                                           repCapName));

    strncpy (repeatedCapabilityIdNames, repCapName, RS_MAX_MESSAGE_BUF_SIZE);
    repeatedCapabilityIdNames[(bufferSize >= RS_MAX_MESSAGE_BUF_SIZE) ?
        RS_MAX_MESSAGE_BUF_SIZE - 1 : bufferSize - 1] = '\0';

Error:
    return error;
}

/// HIFN  Enables the math waveform display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR mathNumber/Selects the math waveform.
/// HIPAR mathEnabled/Enables the math waveform display.
ViStatus _VI_FUNC rsrtm_ConfigureMathEnabled(
    ViSession   instrSession,
    ViInt32 mathNumber,
    ViBoolean   mathEnabled
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap [5];

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    if (rsrtm_invalidViInt32Range (mathNumber, 1, 4) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Math Number");
    }

    sprintf (rep_cap, "M%ld", mathNumber);  

    viCheckParm (rsrtm_SetAttributeViBoolean (instrSession, rep_cap,
                                                 RSRTM_ATTR_MATH_WAVEFORM_ENABLED,
                                                 mathEnabled), 3, "Math Enabled");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}


/// HIFN  This function sets the vertical position of the specified math waveform in the window.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR mathNumber/Selects the math waveform.
/// HIPAR waveformPosition/Sets the vertical position of the specified math waveform in the window.
ViStatus _VI_FUNC rsrtm_ConfigureMathWaveformPosition (ViSession instrSession,
                                                       ViInt32 mathNumber,
                                                       ViReal64 waveformPosition)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap [5];

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    if (rsrtm_invalidViInt32Range (mathNumber, 1, 4) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Math Number");
    }

    sprintf (rep_cap, "M%ld", mathNumber);  

    viCheckParm (rsrtm_SetAttributeViReal64 (instrSession, rep_cap,
                                                 RSRTM_ATTR_MATH_WAVEFORM_POSITION,
                                                 waveformPosition), 3, "Waveform Position");
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}


/// HIFN  This function sets the vertical scale for the specified math waveform.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR mathNumber/Selects the math waveform.
/// HIPAR verticalScale/Sets the vertical scale for the specified math waveform.
ViStatus _VI_FUNC rsrtm_ConfigureMathScaling (ViSession instrSession,
                                              ViInt32 mathNumber,
                                              ViReal64 verticalScale)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap [5];

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    if (rsrtm_invalidViInt32Range (mathNumber, 1, 4) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Math Number");
    }

    sprintf (rep_cap, "M%ld", mathNumber);  

    viCheckParm (rsrtm_SetAttributeViReal64 (instrSession, rep_cap,
                                                 RSRTM_ATTR_MATH_VERTICAL_SCALE,
                                                 verticalScale), 3, "Vertical Scale");
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  Configures general mathematics parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR mathNumber/Selects the math waveform.
/// HIPAR calculationMethod/Selects the method to build the resulting math waveform from 
/// HIPAR calculationMethod/consecutive acquisitions. The processing is similar to the waveform 
/// HIPAR calculationMethod/arithmetics - instead of the acquired waveforms the math waveforms are 
/// HIPAR calculationMethod/used to create envelope and average.
/// HIPAR expression/Defines the math expression to be calculated for the specified math 
/// HIPAR expression/channel.
ViStatus _VI_FUNC rsrtm_ConfigureMathSetup(
    ViSession   instrSession,
    ViInt32 mathNumber,
    ViInt32 calculationMethod,
    ViString    expression
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar  rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    if (rsrtm_invalidViInt32Range (mathNumber, 1, 4) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Math Number");
    }

    sprintf (rep_cap, "M%ld", mathNumber);  

    viCheckParm (rsrtm_SetAttributeViInt32 (instrSession, rep_cap,
                                               RSRTM_ATTR_MATH_ACQUISITION_TYPE,
                                               calculationMethod), 3, "Calculation Method");
    
    viCheckParm (rsrtm_SetAttributeViString (instrSession, rep_cap,
                                                RSRTM_ATTR_MATH_EXPRESSION,
                                                expression), 4, "Expression");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  Defines the frequency range by means of center frequency and frequency 
/// HIFN  span.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR mathNumber/Selects the math waveform.
/// HIPAR centerFrequency/Defines the position of the displayed frequency range, which is 
/// HIPAR centerFrequency/(Center - Span/2) to (Center + Span/2).
/// HIPAR frequencySpan/Defines the stop frequency of the displayed frequency span.
ViStatus _VI_FUNC rsrtm_ConfigureFFTFrequencyCenterSpan(
    ViSession   instrSession,
    ViInt32 mathNumber,
    ViReal64    centerFrequency,
    ViReal64    frequencySpan
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar  rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    if (rsrtm_invalidViInt32Range (mathNumber, 1, 4) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Math Number");
    }

    sprintf (rep_cap, "M%ld", mathNumber);  

    viCheckParm (rsrtm_SetAttributeViReal64 (instrSession, rep_cap,
                                                RSRTM_ATTR_FFT_CENTER_FREQUENCY,
                                                centerFrequency), 3, "Center Frequency");
    
    viCheckParm (rsrtm_SetAttributeViReal64 (instrSession, rep_cap,
                                                RSRTM_ATTR_FFT_FREQUENCY_SPAN,
                                                frequencySpan), 4, "Frequency Span");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  Performs FFT calculation for the full frequency span.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR mathNumber/Selects the math waveform.
ViStatus _VI_FUNC rsrtm_ConfigureFFTFrequencySpanFull(
    ViSession   instrSession,
    ViInt32 mathNumber
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar  rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    if (rsrtm_invalidViInt32Range (mathNumber, 1, 4) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Math Number");
    }

    sprintf (rep_cap, "M%ld", mathNumber);  

    viCheckErr (rsrtm_SetAttributeViString (instrSession, rep_cap,
                                               RSRTM_ATTR_FFT_FULL_SPAN, NULL));
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  Defines the frequency range by means of start and stop frequency.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR mathNumber/Selects the math waveform.
/// HIPAR startFrequency/Defines the start frequency of the displayed frequency span.
/// HIPAR stopFrequency/Defines the stop frequency of the displayed frequency span.
ViStatus _VI_FUNC rsrtm_ConfigureFFTFrequencyStartStop(
    ViSession   instrSession,
    ViInt32 mathNumber,
    ViReal64    startFrequency,
    ViReal64    stopFrequency
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar  rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    if (rsrtm_invalidViInt32Range (mathNumber, 1, 4) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Math Number");
    }

    sprintf (rep_cap, "M%ld", mathNumber);  

    viCheckParm (rsrtm_SetAttributeViReal64 (instrSession, rep_cap,
                                                RSRTM_ATTR_FFT_START_FREQUENCY,
                                                startFrequency), 3, "Start Frequency");
    
    viCheckParm (rsrtm_SetAttributeViReal64 (instrSession, rep_cap,
                                                RSRTM_ATTR_FFT_STOP_FREQUENCY,
                                                stopFrequency), 4, "Stop Frequency");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  Defines the FFT frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR mathNumber/Selects the math waveform.
/// HIPAR rbwandSpanCouplingEnabled/Couples the frequency span to the RBW.
/// HIPAR rbwRatio/Defines the ratio resolution bandwidth (Hz) / span (Hz).
/// HIPAR resolutionBandwidth/Defines the resolution bandwidth.
ViStatus _VI_FUNC rsrtm_ConfigureFFTResolutionBandwidth(
    ViSession   instrSession,
    ViInt32 mathNumber,
    ViBoolean   rbwandSpanCouplingEnabled,
    ViInt32 rbwRatio,
    ViReal64    resolutionBandwidth
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar  rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    if (rsrtm_invalidViInt32Range (mathNumber, 1, 4) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Math Number");
    }

    sprintf (rep_cap, "M%ld", mathNumber);  

    viCheckParm (rsrtm_SetAttributeViBoolean (instrSession, rep_cap,
                                                 RSRTM_ATTR_FFT_RESOLUTION_BANDWIDTH_COUPLING_ENABLED,
                                                 rbwandSpanCouplingEnabled), 3, "RBW and Span Coupling Enabled");

    if (rbwandSpanCouplingEnabled == VI_TRUE)
    {
        viCheckParm (rsrtm_SetAttributeViInt32 (instrSession, rep_cap,
                                                   RSRTM_ATTR_FFT_RESOLUTION_BANDWIDTH_RATIO,
                                                   rbwRatio), 4, "RBW Ratio");
    }
    else
    {
        viCheckParm (rsrtm_SetAttributeViReal64 (instrSession, rep_cap,
                                                    RSRTM_ATTR_FFT_RESOLUTION_BANDWIDTH,
                                                    resolutionBandwidth), 5, "Resolution Bandwidth");
    }
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  Queries the effective resolution bandwidth.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR mathNumber/Selects the math waveform.
/// HIPAR adjustedRBW/Queries the effective resolution bandwidth.
ViStatus _VI_FUNC rsrtm_QueryFFTAdjustedResolutionBandwidth(
    ViSession   instrSession,
    ViInt32 mathNumber,
    ViReal64*   adjustedRBW
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar  rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    if (rsrtm_invalidViInt32Range (mathNumber, 1, 4) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Math Number");
    }

    sprintf (rep_cap, "M%ld", mathNumber);  

    viCheckParm (rsrtm_GetAttributeViReal64 (instrSession, rep_cap,
                                                RSRTM_ATTR_FFT_EFFECTIVE_RESOLUTION_BANDWIDTH,
                                                adjustedRBW), 3, "Adjusted RBW");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  Defines the FFT window.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR mathNumber/Selects the math waveform.
/// HIPAR windowType/Windowing helps minimize the discontinuities at the end of the 
/// HIPAR windowType/measured signal interval and thus reduces the effect of spectral 
/// HIPAR windowType/leakage, increasing the frequency resolution.
ViStatus _VI_FUNC rsrtm_ConfigureFFTWindowType(
    ViSession   instrSession,
    ViInt32 mathNumber,
    ViInt32 windowType
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar  rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    if (rsrtm_invalidViInt32Range (mathNumber, 1, 4) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Math Number");
    }

    sprintf (rep_cap, "M%ld", mathNumber);  

    viCheckParm (rsrtm_SetAttributeViInt32 (instrSession, rep_cap,
                                               RSRTM_ATTR_FFT_WINDOW_TYPE,
                                               windowType), 3, "Window Type");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  Defines the number of spectrums used for averaging.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR mathNumber/Selects the math waveform.
/// HIPAR numberOfSpectrums/Defines the number of spectrums used for averaging.
ViStatus _VI_FUNC rsrtm_ConfigureFFTAveraging (ViSession instrSession,
                                                 ViInt32 mathNumber,
                                                 ViInt32 numberOfSpectrums)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      model [RSRTM_IO_BUFFER_SIZE] = ""; 
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckErr (rsrtm_GetAttributeViString (instrSession, "",
                                               RSRTM_ATTR_INSTRUMENT_MODEL,
                                               RSRTM_IO_BUFFER_SIZE, model));
    if (strstr(model,"RTM")!=NULL)
    {
        if (rsrtm_invalidViInt32Range (mathNumber, 1, 4) == VI_TRUE)
        {
            viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Math Number");
        }
        sprintf (rep_cap, "M%ld", mathNumber);  

        viCheckParm (rsrtm_SetAttributeViInt32 (instrSession, rep_cap,
                                                   RSRTM_ATTR_FFT_AVERAGING,
                                                   numberOfSpectrums), 3, "Number Of Spectrums");
    }
    else
    {
        viCheckErr (RS_ERROR_INSTRUMENT_MODEL);
    }

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}


/// HIFN This function defines the scaling unit and sets the vertical scale for the specified math waveform.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR mathNumber/Selects the math waveform.
/// HIPAR scalingUnit/Defines the scaling unit of the y-axis.
/// HIPAR verticalScale/Sets the vertical scale for the specified math waveform.
ViStatus _VI_FUNC rsrtm_ConfigureFFTScaling (ViSession instrSession,
                                             ViInt32 mathNumber,
                                             ViInt32 scalingUnit,
                                             ViReal64 verticalScale)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap [5];

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    if (rsrtm_invalidViInt32Range (mathNumber, 1, 4) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Math Number");
    }

    sprintf (rep_cap, "M%ld", mathNumber);  

	viCheckParm (rsrtm_SetAttributeViInt32 (instrSession, rep_cap,
                                                 RSRTM_ATTR_FFT_SCALING_UNIT,
                                                 scalingUnit), 3, "Scaling Unit");
	
    viCheckParm (rsrtm_SetAttributeViReal64 (instrSession, rep_cap,
                                                 RSRTM_ATTR_MATH_VERTICAL_SCALE,
                                                 verticalScale), 4, "Vertical Scale");
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN Configures the position and width of the time base extract in the Y(t)-window for which the FFT is calculated.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR mathNumber/Selects the math waveform.
/// HIPAR position/Sets the position of the time base extract in the Y(t)-window for which the FFT is calculated.
/// HIPAR width/Sets the width of the time base extract from the Y(t)-window for which the FFT is calculated.
ViStatus _VI_FUNC rsrtm_ConfigureFFTYtWindowExtract (ViSession instrSession,
                                                     ViInt32 mathNumber,
                                                     ViReal64 position,
                                                     ViReal64 width)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap [5];

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    if (rsrtm_invalidViInt32Range (mathNumber, 1, 4) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Math Number");
    }

    sprintf (rep_cap, "M%ld", mathNumber);  

	viCheckParm (rsrtm_SetAttributeViReal64 (instrSession, rep_cap,
                                                 RSRTM_ATTR_FFT_YT_WINDOW_POSITION,
                                                 position), 3, "Position");
	
    viCheckParm (rsrtm_SetAttributeViReal64 (instrSession, rep_cap,
                                                 RSRTM_ATTR_FFT_YT_WINDOW_WIDTH,
                                                 width), 4, "Width");
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN This function queries the sample rate of data used in an FFT analysis.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR mathNumber/Selects the math waveform.
/// HIPAR sampleRate/Displays the sample rate of data used in an FFT analysis.
ViStatus _VI_FUNC rsrtm_QueryFFTSampleRate (ViSession instrSession,
                                            ViInt32 mathNumber,
                                            ViInt32 *sampleRate)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap [5];

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    if (rsrtm_invalidViInt32Range (mathNumber, 1, 4) == VI_TRUE)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Math Number"); 

    sprintf (rep_cap, "M%ld", mathNumber);  

	viCheckParm (rsrtm_GetAttributeViInt32 (instrSession, rep_cap,
                                                 RSRTM_ATTR_FFT_SAMPLE_RATE,
                                                 sampleRate), 3, "Sample Rate");
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  Selects the source waveform to be used as a reference.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR referenceNumber/Selects the reference waveform.
/// HIPAR source/Selects the source waveform to be used as a reference.
ViStatus _VI_FUNC rsrtm_ConfigureReferenceWaveformSource(
    ViSession    instrSession,
    ViInt32    referenceNumber,
    ViInt32    source
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar    rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    if (rsrtm_invalidViInt32Range (referenceNumber, 1, 4) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Reference Number");
    }

    viCheckParm (rsrtm_GetAttributeRepCapName (instrSession,
                                                 RSRTM_ATTR_REFERENCE_WAVEFORM_SOURCE,
                                                 "ReferenceWfm",
                                                 referenceNumber - 1,
                                                 5,
                                                 rep_cap), 2, "Reference Number");       

    viCheckParm (rsrtm_SetAttributeViInt32 (instrSession, rep_cap,
                                               RSRTM_ATTR_REFERENCE_WAVEFORM_SOURCE, source
                                               ), 3, "Source");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  Copies the selected source waveform with all its settings to the 
/// HIFN  memory of the reference waveform
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR referenceNumber/Selects the reference waveform.
ViStatus _VI_FUNC rsrtm_UpdateReferenceWaveformWithSelectedSource(
    ViSession    instrSession,
    ViInt32    referenceNumber
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar    rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsrtm_GetAttributeRepCapName (instrSession,
                                                 RSRTM_ATTR_REFERENCE_WAVEFORM_UPDATE,
                                                 "ReferenceWfm",
                                                 referenceNumber - 1,
                                                 5,
                                                 rep_cap), 2, "Reference Number");       

    viCheckErr (rsrtm_SetAttributeViString (instrSession, rep_cap,
                                               RSRTM_ATTR_REFERENCE_WAVEFORM_UPDATE, NULL));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  Enables the reference waveform display in the diagram.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR displayEnabled/If enabled, the reference waveform is displayed in the diagram.
/// HIPAR referenceNumber/Selects the reference waveform.
ViStatus _VI_FUNC rsrtm_ConfigureReferenceWaveformDisplayState(
    ViSession    instrSession,
    ViInt32        referenceNumber,
    ViBoolean    displayEnabled
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar    rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsrtm_GetAttributeRepCapName (instrSession,
                                                 RSRTM_ATTR_REFERENCE_WAVEFORM_DISPLAY_ENABLED,
                                                 "ReferenceWfm",
                                                 referenceNumber - 1,
                                                 5,
                                                 rep_cap), 2, "Reference Number");       

    viCheckParm (rsrtm_SetAttributeViBoolean (instrSession, rep_cap,
                                                 RSRTM_ATTR_REFERENCE_WAVEFORM_DISPLAY_ENABLED, displayEnabled
                                                 ), 3, "Display Enabled");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  Loads or saves the reference waveform.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR referenceWfm/Selects the reference waveform.
/// HIPAR fileName/DDefines the file name.
/// HIPAR operation/This control selects the type of waveform operation to be performed.
ViStatus _VI_FUNC rsrtm_ReferenceWaveformFileOperations (ViSession instrSession,
                                                         ViInt32 referenceWfm,
                                                         ViInt32 operation,
                                                         ViString fileName)
{
    ViStatus    error = VI_SUCCESS;
    ViChar    rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsrtm_GetAttributeRepCapName (instrSession,
                                                 RSRTM_ATTR_REFERENCE_WAVEFORM_LOAD_FROM_FILE,
                                                 "ReferenceWfm",
                                                 referenceWfm - 1,
                                                 5,
                                                 rep_cap), 2, "Reference Number");       


    switch (operation)
    {
        case RSRTM_VAL_WAV_LOAD_CFG:
            viCheckParm (rsrtm_SetAttributeViString (instrSession, rep_cap,
                                                RSRTM_ATTR_REFERENCE_WAVEFORM_LOAD_CONFIGURATION, NULL
                                                ), 1, "Reference Waveform");
        break;
        
        case RSRTM_VAL_WAV_LOAD:
            viCheckParm (rsrtm_SetAttributeViString (instrSession, rep_cap,
                RSRTM_ATTR_REFERENCE_WAVEFORM_LOAD_FROM_FILE, fileName), 3, "File Name");
        break;
        
        case RSRTM_VAL_WAV_SAVE:
            viCheckParm (rsrtm_SetAttributeViString (instrSession, rep_cap,
                RSRTM_ATTR_REFERENCE_WAVEFORM_SAVE_TO_FILE, fileName), 3, "File Name");
        break;
        default: 
            viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Operation");
    }
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function configures the vertical scaling of the reference 
/// HIFN  waveform.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR referenceNumber/Selects the reference waveform.
/// HIPAR verticalScale/Sets the scale factor for the reference waveform.
/// HIPAR verticalPosition/Moves the reference waveform and its horizontal axis up or down in the 
/// HIPAR verticalPosition/diagram, if vertical scaling is set to "Independent".
ViStatus _VI_FUNC rsrtm_ConfigureReferenceWaveformVerticalScale(
    ViSession    instrSession,
    ViInt32        referenceNumber,
    ViReal64    verticalScale,
    ViReal64    verticalPosition
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar    rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsrtm_GetAttributeRepCapName (instrSession,
                                                 RSRTM_ATTR_REFERENCE_WAVEFORM_VERTICAL_SCALE,
                                                 "ReferenceWfm",
                                                 referenceNumber - 1,
                                                 5,
                                                 rep_cap), 2, "Reference Number");       

    viCheckParm (rsrtm_SetAttributeViReal64 (instrSession, rep_cap,
                                                RSRTM_ATTR_REFERENCE_WAVEFORM_VERTICAL_SCALE, verticalScale
                                                ), 3, "Vertical Scale");
    
    viCheckParm (rsrtm_SetAttributeViReal64 (instrSession, rep_cap,
                                                RSRTM_ATTR_REFERENCE_WAVEFORM_VERTICAL_POSITION, verticalPosition
                                                ), 4, "Vertical Position");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function changes the horizontal scale (timebase) of the reference waveform independent of the channel waveform settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR referenceWfm/This control selects the reference waveform.
/// HIPAR scale/This control sets horizontal scale.
ViStatus _VI_FUNC rsrtm_ConfigureReferenceWaveformHorizontalScale (ViSession instrSession,
                                                            ViInt32 referenceWfm,
                                                            ViReal64 scale,
                                                            ViReal64 horizontalPosition)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsrtm_GetAttributeRepCapName (instrSession,
                                                 RSRTM_ATTR_REFERENCE_WAVEFORM_HORIZONTAL_SCALE,
                                                 "ReferenceWfm",
                                                 referenceWfm - 1,
                                                 5,
                                                 rep_cap), 2, "Reference Wfm");       

    viCheckParm (rsrtm_SetAttributeViReal64 (instrSession, rep_cap,
        RSRTM_ATTR_REFERENCE_WAVEFORM_HORIZONTAL_SCALE, scale), 3, "Scale");
    viCheckParm (rsrtm_SetAttributeViReal64 (instrSession, rep_cap,
        RSRTM_ATTR_REFERENCE_WAVEFORM_HORIZONTAL_POSITION, horizontalPosition), 4, "Horizontal Position");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}


/// HIFN  This function returns the source waveform - channel, math or reference waveform.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR referenceWfm/This control selects the reference waveform.
/// HIPAR sourceWaveform/Returns the source waveform - channel, math or reference waveform.
ViStatus _VI_FUNC rsrtm_ReferenceReadWaveformCatalog (ViSession instrSession,
                                                        ViInt32 referenceWfm,
                                                        ViInt32 *sourceWaveform)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsrtm_GetAttributeRepCapName (instrSession,
                                                 RSRTM_ATTR_REFERENCE_WAVEFORM_CATALOG,
                                                 "Reference Wfm",
                                                 referenceWfm - 1,
                                                 5,
                                                 rep_cap), 2, "Reference Wfm");       

    viCheckParm (rsrtm_GetAttributeViInt32 (instrSession, rep_cap,
        RSRTM_ATTR_REFERENCE_WAVEFORM_CATALOG, sourceWaveform), 3, "Source Waveform");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function initiates an acquisition on the channels that you enable 
/// HIFN  with the rsrtm_ConfigureChannel function.  It then waits for the 
/// HIFN  acquisition to complete, and returns the waveform for the channel you 
/// HIFN  specify.  You call the rsrtm_FetchWaveform function to obtain the 
/// HIFN  waveforms for each of the remaining enabled channels without initiating 
/// HIFN  another acquisition.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.
/// HIPAR channel/This control selects the channel.
/// HIPAR waveformSize/Pass the number of elements in the Waveform Array parameter.
/// HIPAR maximumTime/Pass the maximum length of time in which to allow the read waveform 
/// HIPAR maximumTime/operation to complete.    
/// HIPAR waveformArray/Returns the waveform that the oscilloscope acquires.  
/// HIPAR actualPoints/Indicates the number of points the function places in the Waveform 
/// HIPAR actualPoints/Array parameter.
/// HIPAR initialX/Indicates the time of the first point in the Waveform Array.  The time 
/// HIPAR initialX/is relative to the trigger event.  
/// HIPAR xIncrement/Indicates the length of time between points in the Waveform Array.  
ViStatus _VI_FUNC rsrtm_ReadWaveform(
    ViSession   instrSession,
    ViInt32 channel,
    ViInt32 waveformSize,
    ViInt32 maximumTime,
    ViReal64  waveformArray[],
    ViInt32*    actualPoints,
    ViReal64*   initialX,
    ViReal64*   xIncrement
)
{
    ViStatus    error = VI_SUCCESS;
	ViInt32     old_tout = 0;

	checkErr (Rs_LockSession (instrSession, VI_NULL));
	checkErr (rsrtm_GetOPCTimeout (instrSession, &old_tout));
	checkErr (rsrtm_SetOPCTimeout (instrSession, maximumTime));

    checkErr (rsrtm_InitiateAcquisition (instrSession, RSRTM_VAL_INIT_SINGLE));
    checkErr (rsrtm_FetchWaveform (instrSession, channel, waveformSize,
                                     waveformArray, actualPoints, initialX,
                                     xIncrement));
    
Error:
	rsrtm_SetOPCTimeout (instrSession, old_tout);
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function initiates an acquisition on the channels that you enable 
/// HIFN  with the rsrtm_ConfigureChannel function.  It then waits for the 
/// HIFN  acquisition to complete, obtains a main waveform measurement on the channel 
/// HIFN  you specify, and returns the measurement value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR measurementFunction/Defines or queries the main measurement.
/// HIPAR maximumTime/Pass the maximum length of time in which to allow the read waveform 
/// HIPAR maximumTime/operation to complete.    
/// HIPAR measurementResult/Returns the measured value.  
ViStatus _VI_FUNC rsrtm_ReadMainWaveformMeasurement (ViSession instrSession,
                                                          ViInt32 measurement,
                                                          ViInt32 measurementFunction,
                                                          ViInt32 maximumTime,
                                                          ViReal64 *measurementResult)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    if (rsrtm_invalidViInt32Range (measurement, RSRTM_VAL_MEASUREMENT_1, 
                                                  RSRTM_VAL_MEASUREMENT_4) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Measurement");
    }

    checkErr (rsrtm_SendSoftwareTrigger (instrSession, maximumTime));

    checkErr (rsrtm_FetchMainWaveformMeasurement (instrSession, 
                                                    measurement,
                                                    measurementFunction,
                                                    measurementResult));
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}


/// HIFN  This function initiates a waveform acquisition.  After you call this 
/// HIFN  function, the oscilloscope leaves the Idle state and waits for a 
/// HIFN  trigger.  The oscilloscope acquires a waveform for each channel you 
/// HIFN  have enabled with the  rsrtm_ConfigureChannel function.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.
/// HIPAR action/Select action to be performed.
ViStatus _VI_FUNC rsrtm_InitiateAcquisition(
    ViSession   instrSession,
    ViInt32 action
)
{
    ViStatus    error = VI_SUCCESS;
    ViUInt32    attr = RSRTM_ATTR_RUN_SINGLE;

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    switch (action)
    {
        case RSRTM_VAL_INIT_SINGLE:
            attr = RSRTM_ATTR_RUN_SINGLE;
            break;
        case RSRTM_VAL_INIT_CONTINUOUS:
            attr = RSRTM_ATTR_RUN_CONTINUOUS;
            break;
        default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Action");
    }

    viCheckErr (rsrtm_SetAttributeViString (instrSession, "", attr, NULL));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  Sends software trigger and waits for operation complete.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.
/// HIPAR maximumTime/Pass the maximum length of time (in milliseconds) in which to allow 
/// HIPAR maximumTime/the read waveform operation to complete.    
ViStatus _VI_FUNC rsrtm_SendSoftwareTrigger(
    ViSession   instrSession,
    ViInt32 maximumTime
)
{
    ViStatus    error = VI_SUCCESS;
	ViInt32     old_tout = 0;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

	checkErr (rsrtm_GetOPCTimeout (instrSession, &old_tout));
	checkErr (rsrtm_SetOPCTimeout (instrSession, maximumTime));
    viCheckErr (rsrtm_SetAttributeViString (instrSession, "",
                                              RSRTM_ATTR_SEND_SW_TRIGGER,
                                              NULL));

Error:
	rsrtm_SetOPCTimeout (instrSession, old_tout);
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function returns the waveform the oscilloscope acquires for the 
/// HIFN  channel you specify.  The waveform is from an acquisition that you 
/// HIFN  previously initiated.  
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR waveformSize/Pass the number of elements in the Waveform Array parameter.
/// HIPAR waveformArray/Returns the waveform that the oscilloscope acquires.  
/// HIPAR actualPoints/Indicates the number of points the function places in the Waveform 
/// HIPAR actualPoints/Array parameter.
/// HIPAR initialX/Indicates the time of the first point in the Waveform Array. The time 
/// HIPAR initialX/is relative to the trigger event.  
/// HIPAR xIncrement/Indicates the length of time between points in the Waveform Array.  
ViStatus _VI_FUNC rsrtm_FetchWaveform(
    ViSession   instrSession,
    ViInt32 channel,
    ViInt32 waveformSize,
    ViReal64    waveformArray[],
    ViInt32*    actualPoints,
    ViReal64*   initialX,
    ViReal64*   xIncrement
)
{
    ViStatus error = VI_SUCCESS;
    ViChar   buffer[RSRTM_IO_BUFFER_SIZE] = "";
    ViChar   *p2data  = NULL;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    sprintf (buffer, "CH%ld", channel);
    viCheckParm(rsrtm_GetAttributeViReal64(instrSession, buffer,
        RSRTM_ATTR_WAVEFORM_CONVERSION_X_START, initialX), 6, "Initial X");
    viCheckParm(rsrtm_GetAttributeViReal64(instrSession, buffer,
        RSRTM_ATTR_WAVEFORM_CONVERSION_X_INCREMENT, xIncrement), 7, "X Increment");

    sprintf (buffer, "FORM REAL,32;CHAN%ld:DATA?", channel);
    checkErr (rsrtm_dataReadTrace (instrSession, buffer, waveformSize, waveformArray, actualPoints));
Error:
    if (p2data) free (p2data);
	viPrintf (instrSession, "FORM ASCII");
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function returns the waveform the oscilloscope acquires for the 
/// HIFN  channel you specify.  The waveform is from an acquisition that you 
/// HIFN  previously initiated. To speed up the transmission, data are transfered as ViUInt16 array 
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR waveformSize/Pass the number of elements in the Waveform Array parameter.
/// HIPAR waveformArray/Returns the waveform that the oscilloscope acquires.  
/// HIPAR actualPoints/Indicates the number of points the function places in the Waveform 
/// HIPAR actualPoints/Array parameter.
/// HIPAR initialX/Indicates the time of the first point in the Waveform Array. The time 
/// HIPAR initialX/is relative to the trigger event.  
/// HIPAR xIncrement/Indicates the length of time between points in the Waveform Array.  
ViStatus _VI_FUNC rsrtm_FetchWaveformInteger (ViSession instrSession,
                                              ViInt32 channel,
                                              ViInt32 waveformSize,
                                              ViUInt16 waveformArray[],
                                              ViInt32 *actualPoints,
                                              ViReal64 *initialX,
                                              ViReal64 *xIncrement)
{
    ViStatus error = VI_SUCCESS;
    ViChar   buffer[RSRTM_IO_BUFFER_SIZE] = "";
	ViChar   *ext_buf = NULL;
	ViUInt32 ret_cnt = 0;
	ViUInt32 count = 0;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    sprintf (buffer, "CH%ld", channel);
    viCheckParm(rsrtm_GetAttributeViReal64(instrSession, buffer,
        RSRTM_ATTR_WAVEFORM_CONVERSION_X_START, initialX), 6, "Initial X");
    viCheckParm(rsrtm_GetAttributeViReal64(instrSession, buffer,
        RSRTM_ATTR_WAVEFORM_CONVERSION_X_INCREMENT, xIncrement), 7, "X Increment");

    viPrintf (instrSession, "FORM UINT,16;CHAN%ld:DATA?", channel);
	checkErr (rsrtm_readBinaryDataBlock (instrSession, &ext_buf, &ret_cnt));
	count = ret_cnt/2;
	
	rsrtm_SWAP_16 ((ViPBuf)ext_buf, count);
	
	*actualPoints = (count > (ViUInt32) waveformSize) ? waveformSize : count;
	memcpy (waveformArray, ext_buf, *actualPoints * 2);
	
Error:
    if (ext_buf) free (ext_buf);
	viPrintf (instrSession, "FORM ASCII");
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function returns the waveform the oscilloscope acquires for the 
/// HIFN  channel you specify.  The waveform is from an acquisition that you 
/// HIFN  previously initiated. To speed up the transmission, data are transfered as byte array 
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR waveformSize/Pass the number of elements in the Waveform Array parameter.
/// HIPAR waveformArray/Returns the waveform that the oscilloscope acquires.  
/// HIPAR actualPoints/Indicates the number of points the function places in the Waveform 
/// HIPAR actualPoints/Array parameter.
/// HIPAR initialX/Indicates the time of the first point in the Waveform Array. The time 
/// HIPAR initialX/is relative to the trigger event.  
/// HIPAR xIncrement/Indicates the length of time between points in the Waveform Array.  
ViStatus _VI_FUNC rsrtm_FetchWaveformByte (ViSession instrSession,
                                           ViInt32 channel, ViInt32 waveformSize,
                                           ViByte waveformArray[],
                                           ViInt32 *actualPoints,
                                           ViReal64 *initialX,
                                           ViReal64 *xIncrement)
{
    ViStatus error = VI_SUCCESS;
    ViChar   buffer[RSRTM_IO_BUFFER_SIZE] = "";
	ViChar   *ext_buf = NULL;
	ViUInt32 ret_cnt = 0;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    sprintf (buffer, "CH%ld", channel);
    viCheckParm(rsrtm_GetAttributeViReal64(instrSession, buffer,
        RSRTM_ATTR_WAVEFORM_CONVERSION_X_START, initialX), 6, "Initial X");
    viCheckParm(rsrtm_GetAttributeViReal64(instrSession, buffer,
        RSRTM_ATTR_WAVEFORM_CONVERSION_X_INCREMENT, xIncrement), 7, "X Increment");

    viPrintf (instrSession, "FORM UINT,8;CHAN%ld:DATA?", channel);
	checkErr (rsrtm_readBinaryDataBlock (instrSession, &ext_buf, &ret_cnt));
	
	*actualPoints = (ret_cnt > (ViUInt32)waveformSize) ? waveformSize : ret_cnt;
	memcpy (waveformArray, ext_buf, *actualPoints);
	
Error:
    if (ext_buf) free (ext_buf);
	viPrintf (instrSession, "FORM ASCII");
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN Returns information on the waveform.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR xStart/X Start in seconds.
/// HIPAR xStop/X Stop in seconds.
/// HIPAR recordLength/Record length of the waveform in Samples.
/// HIPAR valuesPerSample/Number of values per sample interval, usually 1.
ViStatus _VI_FUNC rsrtm_FetchWaveformHeader (ViSession instrSession,
                                             ViInt32 channel, ViReal64 *xStart,
                                             ViReal64 *xStop,
                                             ViInt32 *recordLength,
                                             ViInt32 *valuesPerSample)
{
    ViStatus error = VI_SUCCESS;
    ViChar   header[RSRTM_IO_BUFFER_SIZE];
    ViChar   *p2buf  = NULL;
    ViChar   *p2data  = NULL;
    ViUInt32 count = 0;

    checkErr (Rs_LockSession (instrSession, VI_NULL));    
    
    sprintf (header, "CHAN%ld:DATA:HEAD?\n", channel);
    checkErr (viWrite (instrSession, (ViBuf) header, (ViUInt32) strlen (header), NULL));
    checkErr (Rs_ReadDataUnknownLength (instrSession, &p2data, &count));
    checkErr (rsrtm_CheckStatus (instrSession));
    
    
    p2buf = strtok (p2data, ",");
    if (p2buf == NULL)
        checkErr (VI_ERROR_INV_RESPONSE);
    if (xStart != NULL) *xStart = atof (p2buf);
    
    p2buf = strtok (NULL, ",");
    if (p2buf == NULL)
        checkErr (VI_ERROR_INV_RESPONSE);
    if (xStop != NULL) *xStop = atof (p2buf);

    p2buf = strtok (NULL, ",");
    if (p2buf == NULL)
        checkErr (VI_ERROR_INV_RESPONSE);
    if (recordLength != NULL) *recordLength = atol (p2buf);

    p2buf = strtok (NULL, ",");
    if (p2buf == NULL)
        checkErr (VI_ERROR_INV_RESPONSE);
    if (valuesPerSample != NULL) *valuesPerSample = atol (p2buf);

Error:
    if (p2data) free (p2data);
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN Returns the waveform conversion data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR xStart/Return the time of the first sample of the indicated waveform.
/// HIPAR xIncrement/Return the time difference between two adjacent samples of the indicated waveform.
/// HIPAR yStart/Return the voltage value for binary value 0 of the indicated waveform.
/// HIPAR yIncrement/Return the voltage value per bit of the indicated waveform.
/// HIPAR yResolution/Return the vertical bit resolution of the indicated waveform.
ViStatus _VI_FUNC rsrtm_FetchWaveformConversionData (ViSession instrSession,
                                                     ViInt32 channel,
                                                     ViReal64 *xStart,
                                                     ViReal64 *xIncrement,
                                                     ViReal64 *yStart,
                                                     ViReal64 *yIncrement,
                                                     ViInt32 *yResolution)
{
	ViStatus	error = VI_SUCCESS;
	ViChar      rep_cap[15] = "";
    checkErr (Rs_LockSession (instrSession, VI_NULL));    

    viCheckParm (rsrtm_GetAttributeRepCapName (instrSession,
                                                 RSRTM_ATTR_WAVEFORM_CONVERSION_X_START,
                                                 "Channel",
                                                 channel - 1,
                                                 15,
                                                 rep_cap), 2, "Channel");

	
    
	viCheckParm(rsrtm_GetAttributeViReal64(instrSession, rep_cap,
        RSRTM_ATTR_WAVEFORM_CONVERSION_X_START, xStart), 3, "X Start");
	viCheckParm(rsrtm_GetAttributeViReal64(instrSession, rep_cap,
        RSRTM_ATTR_WAVEFORM_CONVERSION_X_INCREMENT, xIncrement), 4, "X Increment");
	viCheckParm(rsrtm_GetAttributeViReal64(instrSession, rep_cap,
        RSRTM_ATTR_WAVEFORM_CONVERSION_Y_START, yStart), 5, "Y Start");
	viCheckParm(rsrtm_GetAttributeViReal64(instrSession, rep_cap,
        RSRTM_ATTR_WAVEFORM_CONVERSION_Y_INCREMENT, yIncrement), 6, "Y Increment");
	viCheckParm(rsrtm_GetAttributeViInt32(instrSession, rep_cap,
        RSRTM_ATTR_WAVEFORM_CONVERSION_Y_RESOLUTION, yResolution), 7, "Y Resolution");
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  This function returns the data of the envelope.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR waveformSize/Pass the number of elements in the Waveform Array parameter.
/// HIPAR waveformArray/Returns the waveform that the oscilloscope acquires.  
/// HIPAR actualPoints/Indicates the number of points the function places in the Waveform 
/// HIPAR actualPoints/Array parameter.
/// HIPAR initialX/Indicates the time of the first point in the Waveform Array.  The time 
/// HIPAR initialX/is relative to the trigger event.  
/// HIPAR xIncrement/Indicates the length of time between points in the Waveform Array.  
ViStatus _VI_FUNC rsrtm_FetchEnvelopeWaveform (ViSession instrSession,
                                                 ViInt32 channel,
                                                 ViInt32 waveformSize,
                                                 ViReal64 waveformArray[],
                                                 ViInt32 *actualPoints,
                                                 ViReal64 *initialX,
                                                 ViReal64 *xIncrement)
{
    ViStatus error = VI_SUCCESS;
    ViChar   buffer[RSRTM_IO_BUFFER_SIZE] = "";
	ViChar   rep_cap[15] = "";
	

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    if (rsrtm_invalidViInt32Range (channel, 1, 4) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel");
    }
	/*
    checkErr (rsrtm_FetchEnvelopeWaveformHeader (instrSession, channel,
                                                 initialX, NULL,
                                                 actualPoints, NULL));
	*/																	
	 viCheckParm (rsrtm_GetAttributeRepCapName (instrSession,
                                                 RSRTM_ATTR_ENVELOPE_WAVEFORM_CONVERSION_X_START,
                                                 "Channel",
                                                 channel - 1,
                                                 15,
                                                 rep_cap), 2, "Channel");
	
	viCheckParm(rsrtm_GetAttributeViReal64(instrSession, rep_cap,
        RSRTM_ATTR_ENVELOPE_WAVEFORM_CONVERSION_X_START, initialX), 6, "Initial X");
	
	viCheckParm(rsrtm_GetAttributeViReal64(instrSession, rep_cap,
        RSRTM_ATTR_ENVELOPE_WAVEFORM_CONVERSION_X_INCREMENT, xIncrement), 7, "X Increment");
	
    sprintf (buffer, "CHAN%ld:DATA:ENV?", channel);
    checkErr (rsrtm_dataReadTrace (instrSession, buffer, waveformSize, waveformArray, actualPoints));
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN Returns information on the envelope waveform.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR xStart/X Start in seconds.
/// HIPAR xStop/X Stop in seconds.
/// HIPAR recordLength/Record length of the waveform in Samples.
/// HIPAR valuesPerSample/Number of values per sample interval, usually 2.
ViStatus _VI_FUNC rsrtm_FetchEnvelopeWaveformHeader (ViSession instrSession,
                                                     ViInt32 channel,
                                                     ViReal64 *xStart,
                                                     ViReal64 *xStop,
                                                     ViInt32 *recordLength,
                                                     ViInt32 *valuesPerSample)
{
    ViStatus error = VI_SUCCESS;
    ViChar   header[RSRTM_IO_BUFFER_SIZE];
    ViChar   *p2buf  = NULL;
    ViChar   *p2data  = NULL;
    ViUInt32 count = 0;

    checkErr (Rs_LockSession (instrSession, VI_NULL));    
    
    sprintf (header, "CHAN%ld:DATA:ENV:HEAD?\n", channel);
    checkErr (viWrite (instrSession, (ViBuf) header, (ViUInt32) strlen (header), NULL));
    checkErr (Rs_ReadDataUnknownLength (instrSession, &p2data, &count));
    checkErr (rsrtm_CheckStatus (instrSession));
    
    
    p2buf = strtok (p2data, ",");
    if (p2buf == NULL)
        checkErr (VI_ERROR_INV_RESPONSE);
    if (xStart != NULL) *xStart = atof (p2buf);
    
    p2buf = strtok (NULL, ",");
    if (p2buf == NULL)
        checkErr (VI_ERROR_INV_RESPONSE);
    if (xStop != NULL) *xStop = atof (p2buf);

    p2buf = strtok (NULL, ",");
    if (p2buf == NULL)
        checkErr (VI_ERROR_INV_RESPONSE);
    if (recordLength != NULL) *recordLength = atol (p2buf);

    p2buf = strtok (NULL, ",");
    if (p2buf == NULL)
        checkErr (VI_ERROR_INV_RESPONSE);
    if (valuesPerSample != NULL) *valuesPerSample = atol (p2buf);

Error:
    if (p2data) free (p2data);
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN Returns the envelope waveform conversion data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR xStart/Return the time of the first sample of the indicated waveform.
/// HIPAR xIncrement/Return the time difference between two adjacent samples of the indicated waveform.
/// HIPAR yStart/Return the voltage value for binary value 0 of the indicated waveform.
/// HIPAR yIncrement/Return the voltage value per bit of the indicated waveform.
/// HIPAR yResolution/Return the vertical bit resolution of the indicated waveform.
ViStatus _VI_FUNC rsrtm_FetchEnvelopeWaveformConversionData (ViSession instrSession,
                                                             ViInt32 channel,
                                                             ViReal64 *xStart,
                                                             ViReal64 *xIncrement,
                                                             ViReal64 *yStart,
                                                             ViReal64 *yIncrement,
                                                             ViInt32 *yResolution)
{
	ViStatus	error = VI_SUCCESS;
	ViChar      rep_cap[15] = "";
    checkErr (Rs_LockSession (instrSession, VI_NULL));    

    viCheckParm (rsrtm_GetAttributeRepCapName (instrSession,
                                                 RSRTM_ATTR_ENVELOPE_WAVEFORM_CONVERSION_X_START,
                                                 "Channel",
                                                 channel - 1,
                                                 15,
                                                 rep_cap), 2, "Channel");

	
    
	viCheckParm(rsrtm_GetAttributeViReal64(instrSession, rep_cap,
        RSRTM_ATTR_ENVELOPE_WAVEFORM_CONVERSION_X_START, xStart), 3, "X Start");
	viCheckParm(rsrtm_GetAttributeViReal64(instrSession, rep_cap,
        RSRTM_ATTR_ENVELOPE_WAVEFORM_CONVERSION_X_INCREMENT, xIncrement), 4, "X Increment");
	viCheckParm(rsrtm_GetAttributeViReal64(instrSession, rep_cap,
        RSRTM_ATTR_ENVELOPE_WAVEFORM_CONVERSION_Y_START, yStart), 5, "Y Start");
	viCheckParm(rsrtm_GetAttributeViReal64(instrSession, rep_cap,
        RSRTM_ATTR_ENVELOPE_WAVEFORM_CONVERSION_Y_INCREMENT, yIncrement), 6, "Y Increment");
	viCheckParm(rsrtm_GetAttributeViInt32(instrSession, rep_cap,
        RSRTM_ATTR_ENVELOPE_WAVEFORM_CONVERSION_Y_RESOLUTION, yResolution), 7, "Y Resolution");
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  This function returns the data of the math waveform points.  
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR mathNumber/Selects the math waveform.
/// HIPAR waveformSize/Pass the number of elements in the Waveform Array parameter.
/// HIPAR waveformArray/Returns the vertical values - voltages of recorded waveform samples, 
/// HIPAR waveformArray/or magnitudes of a spectrum.  
/// HIPAR actualPoints/Indicates the number of points the function places in the Waveform 
/// HIPAR actualPoints/Array parameter.
ViStatus _VI_FUNC rsrtm_FetchMathWaveform(
    ViSession    instrSession,
    ViInt32        mathNumber,
    ViInt32        waveformSize,
    ViReal64    waveformArray[],
    ViInt32*    actualPoints
)
{
    ViStatus error = VI_SUCCESS;
    ViChar   header[RSRTM_IO_BUFFER_SIZE] = "";
    ViChar   *p2data  = NULL;

	checkErr (Rs_LockSession (instrSession, VI_NULL));

    // Use header to fetch horizontal parameters

    sprintf (header, "CALC:MATH%d:DATA?", mathNumber);
    checkErr (rsrtm_dataReadTrace (instrSession, header, waveformSize, waveformArray, actualPoints));
    
    
Error:
    if (p2data) free (p2data);
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  Returns the header of math waveform data. The header contains  
/// HIFN  attributes of the waveform.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR xStart/X Start in seconds.
/// HIPAR xStop/X Stop in seconds.
/// HIPAR recordLength/Record length of the waveform in Samples.
/// HIPAR valuesPerSample/Number of values per sample interval, usually 1.
ViStatus _VI_FUNC rsrtm_FetchMathWaveformHeader (ViSession instrSession,
                                                 ViInt32 mathNumber,
                                                 ViReal64 *xStart,
                                                 ViReal64 *xStop,
                                                 ViInt32 *recordLength,
                                                 ViInt32 *valuesPerSample)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";
    ViChar      buffer[RSRTM_IO_BUFFER_SIZE] = "";
    ViChar      *p2buf = NULL;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsrtm_GetAttributeRepCapName (instrSession,
                                                 RSRTM_ATTR_MATH_WAVEFORM_HEADER,
                                                 "Math",
                                                 mathNumber - 1,
                                                 5,
                                                 rep_cap), 2, "Math Number");       

    viCheckParm (rsrtm_GetAttributeViString (instrSession, rep_cap,
                                                RSRTM_ATTR_MATH_WAVEFORM_HEADER,
                                                RSRTM_IO_BUFFER_SIZE,
                                                buffer), 3, "Math Waveform Header");
    
    p2buf = strtok (buffer, ",");
    if (p2buf == NULL)
        checkErr (VI_ERROR_INV_RESPONSE);
    if (xStart != NULL) *xStart = atof (p2buf);
    
    p2buf = strtok (NULL, ",");
    if (p2buf == NULL)
        checkErr (VI_ERROR_INV_RESPONSE);
    if (xStop != NULL) *xStop = atof (p2buf);

    p2buf = strtok (NULL, ",");
    if (p2buf == NULL)
        checkErr (VI_ERROR_INV_RESPONSE);
    if (recordLength != NULL) *recordLength = atol (p2buf);

    p2buf = strtok (NULL, ",");
    if (p2buf == NULL)
        checkErr (VI_ERROR_INV_RESPONSE);
    if (valuesPerSample != NULL) *valuesPerSample = atol (p2buf);

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN Returns the math waveform conversion data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR xStart/Return the time of the first sample of the indicated waveform.
/// HIPAR xIncrement/Return the time difference between two adjacent samples of the indicated waveform.
/// HIPAR yStart/Return the voltage value for binary value 0 of the indicated waveform.
/// HIPAR yIncrement/Return the voltage value per bit of the indicated waveform.
/// HIPAR yResolution/Return the vertical bit resolution of the indicated waveform.
ViStatus _VI_FUNC rsrtm_FetchMathWaveformConversionData (ViSession instrSession,
                                                         ViInt32 channel,
                                                         ViReal64 *xStart,
                                                         ViReal64 *xIncrement,
                                                         ViReal64 *yStart,
                                                         ViReal64 *yIncrement,
                                                         ViInt32 *yResolution)
{
	ViStatus	error = VI_SUCCESS;
	ViChar      rep_cap[15] = "";
    checkErr (Rs_LockSession (instrSession, VI_NULL));    

    viCheckParm (rsrtm_GetAttributeRepCapName (instrSession,
                                                 RSRTM_ATTR_WAVEFORM_CONVERSION_X_START,
                                                 "Channel",
                                                 channel - 1,
                                                 15,
                                                 rep_cap), 2, "Channel");

	
    
	viCheckParm(rsrtm_GetAttributeViReal64(instrSession, rep_cap,
        RSRTM_ATTR_MATH_WAVEFORM_CONVERSION_X_START, xStart), 3, "X Start");
	viCheckParm(rsrtm_GetAttributeViReal64(instrSession, rep_cap,
        RSRTM_ATTR_MATH_WAVEFORM_CONVERSION_X_INCREMENT, xIncrement), 4, "X Increment");
	viCheckParm(rsrtm_GetAttributeViReal64(instrSession, rep_cap,
        RSRTM_ATTR_MATH_WAVEFORM_CONVERSION_Y_START, yStart), 5, "Y Start");
	viCheckParm(rsrtm_GetAttributeViReal64(instrSession, rep_cap,
        RSRTM_ATTR_MATH_WAVEFORM_CONVERSION_Y_INCREMENT, yIncrement), 6, "Y Increment");
	viCheckParm(rsrtm_GetAttributeViInt32(instrSession, rep_cap,
        RSRTM_ATTR_MATH_WAVEFORM_CONVERSION_Y_RESOLUTION, yResolution), 7, "Y Resolution");
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  This function returns the data of the reference waveform points.  
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR referenceWfm/Selects the reference waveform, the internal reference storage.
/// HIPAR waveformSize/Pass the number of elements in the Waveform Array parameter.
/// HIPAR waveformArray/Returns the vertical values - voltages of recorded waveform samples, 
/// HIPAR waveformArray/or magnitudes of a spectrum.  
/// HIPAR actualPoints/Indicates the number of points the function places in the Waveform 
/// HIPAR actualPoints/Array parameter.
ViStatus _VI_FUNC rsrtm_FetchReferenceWaveform (ViSession instrSession,
                                                     ViInt32 referenceWfm,
                                                     ViInt32 waveformSize,
                                                     ViReal64 waveformArray[],
                                                     ViInt32 *actualPoints)
{
    ViStatus error = VI_SUCCESS;
    ViChar   *p2buf  = NULL;
    ViChar   *p2data  = NULL;
    ViUInt32 count = 0;
    ViInt32  i = 0;

    checkErr (Rs_LockSession (instrSession, VI_NULL));    
    
    if (rsrtm_invalidViInt32Range (referenceWfm, 
            RSRTM_VAL_REF_WAVEFORM_1, RSRTM_VAL_REF_WAVEFORM_4) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Reference Wfm");
    }

    viCheckErr (viPrintf(instrSession, "REFC%ld:DATA?\n", referenceWfm));

    checkErr (Rs_ReadDataUnknownLength (instrSession, &p2data, &count));
    checkErr (rsrtm_CheckStatus (instrSession));
    
    p2buf = strtok (p2data, ",");
    while (p2buf && (i < waveformSize))
    {
        if (i<waveformSize)
            waveformArray[i++] = atof (p2buf);
        else
            i++;
        p2buf = strtok (NULL, ",");
    }
    *actualPoints = i;
    
Error:
    if (p2data) free (p2data);
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  Returns the header of math waveform data. The header contains  
/// HIFN  attributes of the waveform.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR xStart/X Start in seconds.
/// HIPAR xStop/X Stop in seconds.
/// HIPAR recordLength/Record length of the waveform in Samples.
/// HIPAR valuesPerSample/Number of values per sample interval, usually 1.
ViStatus _VI_FUNC rsrtm_FetchReferenceWaveformHeader (ViSession instrSession,
                                                      ViInt32 channel,
                                                      ViReal64 *xStart,
                                                      ViReal64 *xStop,
                                                      ViInt32 *recordLength,
                                                      ViInt32 *valuesPerSample)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";
    ViChar      buffer[RSRTM_IO_BUFFER_SIZE] = "";
    ViChar      *p2buf = NULL;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsrtm_GetAttributeRepCapName (instrSession,
                                                 RSRTM_ATTR_REFERENCE_WAVEFORM_HEADER,
                                                 "ReferenceWfm",
                                                 channel - 1,
                                                 5,
                                                 rep_cap), 2, "Reference Wfm");       

    viCheckParm (rsrtm_GetAttributeViString (instrSession, rep_cap,
                                                RSRTM_ATTR_REFERENCE_WAVEFORM_HEADER,
                                                RSRTM_IO_BUFFER_SIZE,
                                                buffer), 3, "Reference Waveform Header");
    
    p2buf = strtok (buffer, ",");
    if (p2buf == NULL)
        checkErr (VI_ERROR_INV_RESPONSE);
    if (xStart != NULL) *xStart = atof (p2buf);
    
    p2buf = strtok (NULL, ",");
    if (p2buf == NULL)
        checkErr (VI_ERROR_INV_RESPONSE);
    if (xStop != NULL) *xStop = atof (p2buf);

    p2buf = strtok (NULL, ",");
    if (p2buf == NULL)
        checkErr (VI_ERROR_INV_RESPONSE);
    if (recordLength != NULL) *recordLength = atol (p2buf);

    p2buf = strtok (NULL, ",");
    if (p2buf == NULL)
        checkErr (VI_ERROR_INV_RESPONSE);
    if (valuesPerSample != NULL) *valuesPerSample = atol (p2buf);

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN Returns the reference waveform conversion data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR xStart/Return the time of the first sample of the indicated waveform.
/// HIPAR xIncrement/Return the time difference between two adjacent samples of the indicated waveform.
/// HIPAR yStart/Return the voltage value for binary value 0 of the indicated waveform.
/// HIPAR yIncrement/Return the voltage value per bit of the indicated waveform.
/// HIPAR yResolution/Return the vertical bit resolution of the indicated waveform.
ViStatus _VI_FUNC rsrtm_FetchReferenceWaveformConversionData (ViSession instrSession,
                                                              ViInt32 channel,
                                                              ViReal64 *xStart,
                                                              ViReal64 *xIncrement,
                                                              ViReal64 *yStart,
                                                              ViReal64 *yIncrement,
                                                              ViInt32 *yResolution)
{
	ViStatus	error = VI_SUCCESS;
	ViChar      rep_cap[15] = "";
    checkErr (Rs_LockSession (instrSession, VI_NULL));    

    viCheckParm (rsrtm_GetAttributeRepCapName (instrSession,
                                                 RSRTM_ATTR_WAVEFORM_CONVERSION_X_START,
                                                 "Channel",
                                                 channel - 1,
                                                 15,
                                                 rep_cap), 2, "Channel");

	
    
	viCheckParm(rsrtm_GetAttributeViReal64(instrSession, rep_cap,
        RSRTM_ATTR_REFERENCE_WAVEFORM_CONVERSION_X_START, xStart), 3, "X Start");
	viCheckParm(rsrtm_GetAttributeViReal64(instrSession, rep_cap,
        RSRTM_ATTR_REFERENCE_WAVEFORM_CONVERSION_X_INCREMENT, xIncrement), 4, "X Increment");
	viCheckParm(rsrtm_GetAttributeViReal64(instrSession, rep_cap,
        RSRTM_ATTR_REFERENCE_WAVEFORM_CONVERSION_Y_START, yStart), 5, "Y Start");
	viCheckParm(rsrtm_GetAttributeViReal64(instrSession, rep_cap,
        RSRTM_ATTR_REFERENCE_WAVEFORM_CONVERSION_Y_INCREMENT, yIncrement), 6, "Y Increment");
	viCheckParm(rsrtm_GetAttributeViInt32(instrSession, rep_cap,
        RSRTM_ATTR_REFERENCE_WAVEFORM_CONVERSION_Y_RESOLUTION, yResolution), 7, "Y Resolution");
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  This function obtains a main waveform measurement on the channel you 
/// HIFN  specify, and returns the measurement value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR measurementFunction/Defines or queries the main measurement.
/// HIPAR measurementResult/Returns the measured value.  
ViStatus _VI_FUNC rsrtm_FetchMainWaveformMeasurement (ViSession instrSession,
                                                           ViInt32 measurement,
                                                           ViInt32 measurementFunction,
                                                           ViReal64 *measurementResult)
{
    ViStatus error = VI_SUCCESS;
    ViChar   rep_cap_measurement[25] = "";
    ViChar   rep_cap_meas_function[25] = "";
    ViChar   rep_cap[75] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsrtm_GetAttributeRepCapName (instrSession,
                                                 RSRTM_ATTR_MAIN_MEASUREMENT_RESULT,
                                                 "Measurement",
                                                 measurement - 1,
                                                 25,
                                                 rep_cap_measurement), 2, "Measurement");
    viCheckParm (rsrtm_GetAttributeRepCapName (instrSession,
                                                 RSRTM_ATTR_MAIN_MEASUREMENT_RESULT,
                                                 "MeasType",
                                                 measurementFunction - 1,
                                                 25,
                                                 rep_cap_meas_function), 3, "Result Type");
    strcpy (rep_cap, rep_cap_measurement);
    strcat (rep_cap, ",");
    strcat (rep_cap, rep_cap_meas_function);

    viCheckParm(rsrtm_GetAttributeViReal64(instrSession, rep_cap,
        RSRTM_ATTR_MAIN_MEASUREMENT_RESULT, measurementResult), 4, "Result");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function aborts an acquisition and returns the oscilloscope to 
/// HIFN  the Idle state.  You initiate an acquisition with the 
/// HIFN  rsrtm_InitiateAcquisition function.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.
ViStatus _VI_FUNC rsrtm_Abort(
    ViSession   instrSession
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckErr (rsrtm_SetAttributeViString (instrSession, "",
                                              RSRTM_ATTR_STOP, NULL));
Error:
    (void) Rs_UnlockSession (instrSession, VI_NULL);
    return error;
}

/// HIFN This function resets the instrument to a known state and sends initialization commands to the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.
ViStatus _VI_FUNC rsrtm_reset (ViSession instrSession)
{
    ViStatus    error   = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (Rs_Simulating(instrSession) == 0)                /* call only when locked */
        {
        viCheckErr( viPrintf (instrSession, "*RST\n"));
        }

    checkErr( rsrtm_DefaultInstrSetup (instrSession));
    checkErr( rsrtm_CheckStatus (instrSession));

Error:

    (void) Rs_UnlockSession (instrSession, VI_NULL);

    return error;
}

/// HIFN This function runs the instrument's self test routine and returns the test result(s).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.
/// HIPAR testResult/Contains the value returned from the instrument self test.
/// HIPAR testMessage/Returns the self-test response string from the instrument.
ViStatus _VI_FUNC rsrtm_self_test (ViSession instrSession, ViInt16 *testResult, 
                                      ViChar testMessage[])
{
    ViStatus    error   = VI_SUCCESS;
    ViInt32     result = 0;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (testResult == NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Null address for Test Result");
    if (testMessage == NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Null address for Test Message");

    viCheckErr (rsrtm_GetAttributeViInt32 (instrSession, NULL, RSRTM_ATTR_SELF_TEST_QUERY, &result));
    if (*testResult == 0)
        strcpy (testMessage, "Self-Test Passed.");
    else
        strcpy (testMessage, "Self-Test Failed.");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN This function returns the driver and instrument revisions.
/// HIRET /// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.
/// HIPAR driverRev/Returns the instrument driver software revision numbers in the form of a string.
/// HIPAR instrRev/Returns the instrument firmware revision numbers in the form of a string.
ViStatus _VI_FUNC rsrtm_revision_query (ViSession instrSession, ViChar driverRev[], 
                                           ViChar instrRev[])
{
    ViStatus    error   = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (driverRev == VI_NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Null address for Driver Revision");
    if (instrRev == VI_NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Null address for Instrument Revision");

    checkErr( Rs_GetAttribute (instrSession, VI_NULL, RSRTM_ATTR_SPECIFIC_DRIVER_REVISION,
                                        0, 256, driverRev));
    checkErr( Rs_GetAttribute (instrSession, "", RSRTM_ATTR_INSTRUMENT_FIRMWARE_REVISION,
                                        0, 256, instrRev));
    checkErr( rsrtm_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN This function reads an error code and a message from the
/// HIFN instrument's error queue.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.
/// HIPAR errCode/Returns the error code read from the instrument's error queue.
/// HIPAR errMessage/Returns the error message string read from the instrument's error message queue.
ViStatus _VI_FUNC rsrtm_error_query (ViSession instrSession, ViInt32 *errCode, 
                                        ViChar errMessage[])
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSRTM_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (errCode == NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Null address for Error Code");
    if (errMessage == NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Null address for Error Message");

    if (Rs_Simulating(instrSession) == 0)                /* call only when locked */
        {
        viCheckErr (rsrtm_GetAttributeViString( instrSession, "", RSRTM_ATTR_SYSTEM_ERROR, RSRTM_IO_BUFFER_SIZE, buffer));
        if  (sscanf (buffer, "%ld,\"%256[^\"]", errCode, errMessage) != 2)
            viCheckErr(RS_ERROR_UNEXPECTED_RESPONSE);

        }
    else
        {
            /* Simulate Error Query */
        *errCode = 0;
        strcpy (errMessage, "No error.");
        }

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/// HIFN Queries the error/event queue for all unread items and removes them from the queue.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.
/// HIPAR bufferSize/Sets the size of Errors string.
/// HIPAR errors/Returns the error message string read from the instrument's error message queue.
ViStatus _VI_FUNC rsrtm_ErrorList (ViSession instrSession,
                                   ViInt32 bufferSize, ViChar errors[])
{
	ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

	viCheckParm (rsrtm_GetAttributeViString (instrSession, NULL,
                                                 RSRTM_ATTR_SYST_ERR_LIST, bufferSize,
                                                 errors), 3, "Errors"); 
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN This function converts a status code returned by an instrument driver function into a user-readable string.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.
/// HIPAR errorCode/Pass the Status parameter that is returned from any of the instrument driver functions.
/// HIPAR errorMessage/Returns the user-readable message string that corresponds to the status code you specify.
ViStatus _VI_FUNC rsrtm_error_message (ViSession instrSession, ViStatus errorCode,
                                          ViChar errorMessage[256])
{
    ViStatus    error   = VI_SUCCESS;

    static      RsStringValueTable errorTable =
        {
            RSRTM_ERROR_CODES_AND_MSGS,
            {VI_NULL, VI_NULL}
        };

    (void) Rs_LockSession(instrSession, VI_NULL);

        /* all VISA and RS error codes are handled as well as codes in the table */
    if (errorMessage == VI_NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Null address for Error Message");

    checkErr( Rs_GetSpecificDriverStatusDesc(instrSession, errorCode, errorMessage, errorTable));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function resets all STATUS:QUESTIONALBLE registers.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
ViStatus _VI_FUNC rsrtm_ResetStatusRegisters (ViSession instrSession)
{
	ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

	viCheckErr (rsrtm_SetAttributeViString (instrSession, NULL, RSRTM_ATTR_RESET_STATUS_REGISTER, NULL)); 
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function sets the status reporting system.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR registerType/This control selects type of the registers that will be configured.
/// HIPAR statusRegister/This control selects the registers that will be configured.
/// HIPAR registerPart/This control selects the registers that will be configured.
/// HIPAR registerValue/Value of selected register.
ViStatus _VI_FUNC rsrtm_SetStatusRegister (ViSession instrSession,
                                             ViInt32 registerType,
                                             ViInt32 statusRegister,
                                             ViInt32 registerPart,
                                             ViInt32 registerValue)
{
    ViStatus error = VI_SUCCESS;
    ViChar   rep_cap_part[25] = "";
    ViChar   rep_cap[75] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsrtm_GetAttributeRepCapName (instrSession,
                                                 RSRTM_ATTR_REGISTER,
                                                 "RegisterType",
                                                 registerType,
                                                 25,
                                                 rep_cap), 2, "Register Type");
    viCheckParm (rsrtm_GetAttributeRepCapName (instrSession,
                                                 RSRTM_ATTR_REGISTER,
                                                 "Register",
                                                 statusRegister,
                                                 25,
                                                 rep_cap_part), 3, "Status Register");
    strcat (rep_cap, ",");
    strcat (rep_cap, rep_cap_part);

    viCheckParm (rsrtm_GetAttributeRepCapName (instrSession,
                                                 RSRTM_ATTR_REGISTER,
                                                 "RegisterPart",
                                                 registerPart,
                                                 25,
                                                 rep_cap_part), 4, "Register Part");

    strcat (rep_cap, ",");
    strcat (rep_cap, rep_cap_part);

    viCheckParm(rsrtm_SetAttributeViInt32(instrSession, rep_cap,
        RSRTM_ATTR_REGISTER, registerValue), 5, "Register Value");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function queries the status reporting system.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR registerType/This control selects type of the registers that will be queried.
/// HIPAR statusRegister/This control selects the registers that will be queried.
/// HIPAR registerPart/This control selects the registers that will be queried.
/// HIPAR registerValue/Returns value of selected register.
ViStatus _VI_FUNC rsrtm_GetStatusRegister (ViSession instrSession,
                                                ViInt32 registerType,
                                                ViInt32 statusRegister,
                                                ViInt32 registerPart,
                                                ViInt32 *registerValue)
{
    ViStatus error = VI_SUCCESS;
    ViChar   rep_cap_part[25] = "";
    ViChar   rep_cap[75] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsrtm_GetAttributeRepCapName (instrSession,
                                                 RSRTM_ATTR_REGISTER,
                                                 "RegisterType",
                                                 registerType,
                                                 25,
                                                 rep_cap), 2, "Register Type");
    viCheckParm (rsrtm_GetAttributeRepCapName (instrSession,
                                                 RSRTM_ATTR_REGISTER,
                                                 "Register",
                                                 statusRegister,
                                                 25,
                                                 rep_cap_part), 3, "Status Register");
    strcat (rep_cap, ",");
    strcat (rep_cap, rep_cap_part);

    viCheckParm (rsrtm_GetAttributeRepCapName (instrSession,
                                                 RSRTM_ATTR_REGISTER,
                                                 "RegisterPart",
                                                 registerPart,
                                                 25,
                                                 rep_cap_part), 4, "Register Part");

    strcat (rep_cap, ",");
    strcat (rep_cap, rep_cap_part);

    viCheckParm(rsrtm_GetAttributeViInt32 (instrSession, rep_cap,
        RSRTM_ATTR_REGISTER, registerValue), 5, "Register Value");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN This function retrieves and then clears the error information
/// HIFN for the session or the current execution thread.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.
/// HIPAR errorCode/Returns the error code for the session or execution thread.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the Description parameter.
/// HIPAR description/Returns the error description for the session or execution thread. If there is no description, the function returns an empty string.
ViStatus _VI_FUNC rsrtm_GetError (ViSession instrSession, 
                                     ViStatus *errorCode, 
                                     ViInt32 bufferSize,
                                     ViChar description[])  
{
    ViStatus    error       = VI_SUCCESS;
    ViStatus    primary     = VI_SUCCESS,
                secondary   = VI_SUCCESS;
    ViChar      elaboration[RS_MAX_MESSAGE_BUF_SIZE]    = "";
    ViChar      errorMessage[RS_MAX_MESSAGE_BUF_SIZE + 256]  = "";
    ViChar     *appendPoint = errorMessage;
    ViInt32     actualSize  = 0;
    ViBoolean   locked      = VI_FALSE;

    /* Lock Session */
    checkErr( Rs_LockSession(instrSession, &locked));

    /* Test for nulls and acquire error data */
    if (bufferSize != 0)
        {
        if (errorCode == VI_NULL)
            {
            viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Null address for Error");
            }
        checkErr( Rs_GetErrorInfo (instrSession, &primary, &secondary, elaboration));
        }

    else
        {
        checkErr( Rs_GetAttribute (instrSession, VI_NULL, RS_ATTR_ERROR_ELABORATION, 0, 256, elaboration));
        checkErr( Rs_GetAttribute (instrSession, VI_NULL, RS_ATTR_SECONDARY_ERROR, 0, (ViInt32) sizeof (secondary), &secondary));
        checkErr( Rs_GetAttribute (instrSession, VI_NULL, RS_ATTR_PRIMARY_ERROR, 0, (ViInt32) sizeof (primary), &primary));
        }

    if (description == VI_NULL)
        {
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 4, "Null address for Description");
        }

    /* Format data */
    if (primary != VI_SUCCESS)
        {
        ViChar msg[256] = "";
        checkErr( rsrtm_error_message (instrSession, primary, msg));
        appendPoint += sprintf(appendPoint, "Primary Error: (Hex 0x%08X) %s\n", (unsigned int) primary, msg);
        }

    if (secondary != VI_SUCCESS)
        {
        ViChar msg[256] = "";
        checkErr( rsrtm_error_message (instrSession, secondary, msg));
        appendPoint += sprintf(appendPoint, "Secondary Error: (Hex 0x%08X) %s\n", (unsigned int) secondary, msg);
        }

    if (strlen (elaboration) > 0)
        {
        sprintf(appendPoint, "Elaboration: %s", elaboration);
        }

    actualSize = (ViInt32) (strlen(errorMessage) + 1);

    /* Prepare return values */
    if (bufferSize == 0)
        {
        error = (ViStatus) actualSize;
        }
    else
        {
        if (bufferSize > 0)
            {
            if (actualSize > bufferSize)
                {
                error = actualSize;
                actualSize = bufferSize;
                }
            }
        memcpy(description, errorMessage, (size_t)(actualSize-1));
        description[actualSize-1] = '\0';
        }

    if (errorCode)
        {
        *errorCode = primary;
        }

Error:
    /* Unlock Session */
    (void) Rs_UnlockSession (instrSession, &locked);
    return error;
}

/// HIFN This function clears the error code and error description for
/// HIFN the session.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.
ViStatus _VI_FUNC rsrtm_ClearError (ViSession instrSession)                                                        
{                                                                                                           
    return Rs_ClearErrorInfo (instrSession);                                                                             
}

/// HIFN This function writes a user-specified string to the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.
/// HIPAR writeBuffer/Pass the string to be written to the instrument.
ViStatus _VI_FUNC rsrtm_WriteInstrData (ViSession instrSession, ViString writeBuffer)
{
    return Rs_WriteInstrData (instrSession, writeBuffer);
}

/// HIFN Document your function here.
/// HIFN You may use multiple lines for documentation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.
/// HIPAR numBytes/Pass the maximum number of bytes to read from the instruments.
/// HIPAR rdBuf/After this function executes, this parameter contains the data that was read from the instrument.
/// HIPAR bytesRead/Returns the number of bytes actually read from the instrument and stored in the Read Buffer.
ViStatus _VI_FUNC rsrtm_ReadInstrData (ViSession instrSession, ViInt32 numBytes,
                                          ViChar rdBuf[], ViUInt32 *bytesRead)
{
    return Rs_ReadInstrData (instrSession, numBytes, rdBuf, bytesRead);
}

/// HIFN This function queries the ViBoolean value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.
/// HIPAR command/The query to be sent to the instrument.
/// HIPAR value/Returns the ViBoolean value.
ViStatus _VI_FUNC rsrtm_QueryViBoolean (ViSession instrSession,
                                           ViString command, 
                                           ViBoolean *value)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    if (strlen(command)==0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Command");
    if (value==VI_NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Value");
    viCheckErr (viQueryf (instrSession, "%s\n", "%d", command, value));
    checkErr( rsrtm_CheckStatus (instrSession)); 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN This function queries the ViInt32 value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.
/// HIPAR command/The query to be sent to the instrument.
/// HIPAR value/Returns the ViInt32 value.
ViStatus _VI_FUNC rsrtm_QueryViInt32 (ViSession instrSession,
                                         ViString command, 
                                         ViInt32 *value)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    if (strlen(command)==0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Command");
    if (value==VI_NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Value");
    viCheckErr (viQueryf (instrSession, "%s\n", "%ld", command, value));
    checkErr( rsrtm_CheckStatus (instrSession)); 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN This function queries the ViReal64 value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.
/// HIPAR command/The query to be sent to the instrument.
/// HIPAR value/Returns the ViReal64 value.
ViStatus _VI_FUNC rsrtm_QueryViReal64 (ViSession instrSession,
                                          ViString command, 
                                          ViReal64 *value)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    if (strlen(command)==0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Command");
    if (value==VI_NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Value");
    viCheckErr (viQueryf (instrSession, "%s\n", "%le", command, value));
    checkErr( rsrtm_CheckStatus (instrSession)); 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN This function queries the ViString value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.
/// HIPAR command/The query to be sent to the instrument.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the Value parameter.
/// HIPAR value/Returns the string value.
ViStatus _VI_FUNC rsrtm_QueryViString (ViSession instrSession,
                                          ViString command, 
                                          ViInt32 bufferSize,
                                          ViChar _VI_FAR value[])
{
    ViStatus    error = VI_SUCCESS;
    ViChar      *pbuffer = NULL;
    ViUInt32        bytesRead;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    if (strlen(command)==0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Command");
    if (value==VI_NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Value");
    viCheckErr (viPrintf (instrSession, "%s\n", command));
    viCheckErr(Rs_ReadDataUnknownLength (instrSession, &pbuffer, &bytesRead));  
    if (bytesRead>=(ViUInt32)bufferSize)
    {
        strncpy(value, pbuffer, (size_t) (bufferSize-1));
        value[bufferSize-1]='\0';
        error = (ViStatus) (bytesRead+1);
    } 
    else
    {
        strncpy(value, pbuffer, (size_t) bytesRead); 
        value[bytesRead]='\0';   
    }
    checkErr( rsrtm_CheckStatus (instrSession)); 
Error:
    if (pbuffer) free (pbuffer);
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function provides mass storage capabilities for the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR operation/This control selects the type of memory operation to be performed.
/// HIPAR source/This control specifies the source of the operation to be performed. If 
/// HIPAR source/the operation requires a single parameter, this control specifies that 
/// HIPAR source/parameter.
/// HIPAR destination/This control specifies the destination of the operation to be 
/// HIPAR destination/performed.
ViStatus _VI_FUNC rsrtm_FileManagerOperations(
    ViSession   instrSession,
    ViInt32 operation,
    ViString    source,
    ViString    destination
)
{
    ViStatus error = VI_SUCCESS;
    ViChar   buffer[RSRTM_IO_BUFFER_SIZE] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    switch (operation)
    {
        case RSRTM_VAL_FILE_COPY:
            sprintf (buffer, "MMEM:COPY '%s','%s'\n", source, destination);
            checkErr (viWrite (instrSession, (ViBuf) buffer, (ViUInt32) strlen (buffer), NULL));
            checkErr (rsrtm_CheckStatus (instrSession));
            break;
        case RSRTM_VAL_FILE_MOVE:
            sprintf (buffer, "MMEM:MOVE '%s','%s'\n", source, destination);
            checkErr (viWrite (instrSession, (ViBuf) buffer, (ViUInt32) strlen (buffer), NULL));
            checkErr (rsrtm_CheckStatus (instrSession));
            break;
        case RSRTM_VAL_FILE_DELETE:
            viCheckParm (rsrtm_SetAttributeViString(instrSession, "",
                RSRTM_ATTR_DELETE_FILE, source), 3, "Source");
            break;
        case RSRTM_VAL_FILE_NEW_DIR:
            viCheckParm (rsrtm_SetAttributeViString(instrSession, "",
                RSRTM_ATTR_CREATE_DIRECTORY, source), 3, "Source");
            break;
        case RSRTM_VAL_FILE_CDIR:
            viCheckParm (rsrtm_SetAttributeViString(instrSession, "",
                RSRTM_ATTR_CURRENT_DIRECTORY, source), 3, "Source");
            break;
        case RSRTM_VAL_FILE_RDIR:
            viCheckParm (rsrtm_SetAttributeViString(instrSession, "",
                RSRTM_ATTR_REMOVE_DIRECTORY, source), 3, "Source");
            break;
        case RSRTM_VAL_FILE_CDRIVE:
            viCheckParm (rsrtm_SetAttributeViString(instrSession, "",
                RSRTM_ATTR_CHANGE_DRIVE, source), 3, "Source");
            break;
        default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Operation");
    }
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  Returns the number of files and subdirectories of the current or of a 
/// HIFN  specified directory. The number includes the parent directory strings 
/// HIFN  "." and ".." so that it corresponds to the number of strings returned 
/// HIFN  by the rsrtm_FileDirectoryContent function after the initial numeric 
/// HIFN  parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR directory/This control specifies the directory in which the command will be 
/// HIPAR directory/performed.
/// HIPAR numberofFilesAndFolders/Returns the number of files and subdirectories of the current or of a 
/// HIPAR numberofFilesAndFolders/specified directory.
ViStatus _VI_FUNC rsrtm_FileCount(
    ViSession   instrSession,
    ViString    directory,
    ViInt32*    numberofFilesAndFolders
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar  buffer[RSRTM_IO_BUFFER_SIZE] = "";
    ViChar *p2buf = NULL;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckAlloc (numberofFilesAndFolders);
    
    p2buf = buffer + sprintf (buffer, "MMEM:CAT:LENG?");
    
    if ((directory != NULL) && (strlen (directory) > 0))
        p2buf += sprintf (p2buf, " '%s'", directory);
    
    strcat (buffer, "\n");
    checkErr (viWrite (instrSession, (ViBuf) buffer, (ViUInt32) strlen (buffer), NULL));
    checkErr (viScanf (instrSession, "%ld", numberofFilesAndFolders));
    checkErr (rsrtm_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function reads the indicated directory. According to DOS 
/// HIFN  convention, wild card characters can be entered in order to query eg. a 
/// HIFN  list of all file of a certain type.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR directory/This control specifies the directory in which the command will be 
/// HIPAR directory/performed.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the 
/// HIPAR bufferSize/Output parameter.  
/// HIPAR output/This indicator returns the files in the indicated directory as comma 
/// HIPAR output/separated strings, for example: 'SPOOL1.WMF','SPOOL2.WMF','SPOOL3.WMF'
ViStatus _VI_FUNC rsrtm_FileDirectoryContent(
    ViSession   instrSession,
    ViString    directory,
    ViInt32 bufferSize,
    ViChar    output[]
)
{
    ViStatus error = VI_SUCCESS;
    ViChar   buffer[RSRTM_IO_BUFFER_SIZE] = "";
    ViChar   *p2buf = NULL;
    ViUInt32  count = 0;
    ViUInt32  length = 0;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckAlloc (output);
    
    p2buf = buffer + sprintf (buffer, "MMEM:CAT?");
    
    if ((directory != NULL) && (strlen (directory) > 0))
        p2buf += sprintf (p2buf, " '%s'", directory);
    
    strcat (buffer, "\n");
    checkErr (viWrite (instrSession, (ViBuf) buffer, (ViUInt32) strlen (buffer), NULL));
    p2buf = NULL;
    checkErr (Rs_ReadDataUnknownLength (instrSession, &p2buf, &count));
    
    length = ((ViInt32)count > bufferSize) ? bufferSize : count;
    memcpy (output, p2buf, length);
    
    checkErr (rsrtm_CheckStatus (instrSession));

Error:
    if (p2buf != NULL) free (p2buf);
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  Returns the number of subdirectories of the current or of a specified 
/// HIFN  directory. The number includes the parent directory strings "." and 
/// HIFN  ".." so that it corresponds to the number of strings returned by the 
/// HIFN  rsrtm_FileSubdirectories.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR directory/This control specifies the directory in which the command will be 
/// HIPAR directory/performed.
/// HIPAR numberofSubdirectories/Returns the number of files and subdirectories of the current or of a 
/// HIPAR numberofSubdirectories/specified directory.
ViStatus _VI_FUNC rsrtm_FileSubdirectoriesCount(
    ViSession   instrSession,
    ViString    directory,
    ViInt32*    numberofSubdirectories
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar  buffer[RSRTM_IO_BUFFER_SIZE] = "";
    ViChar *p2buf = NULL;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckAlloc (numberofSubdirectories);
    
    p2buf = buffer + sprintf (buffer, "MMEM:DCAT:LENG?");
    
    if ((directory != NULL) && (strlen (directory) > 0))
        p2buf += sprintf (p2buf, " '%s'", directory);
    
    strcat (buffer, "\n");
    checkErr (viWrite (instrSession, (ViBuf) buffer, (ViUInt32) strlen (buffer), NULL));
    checkErr (viScanf (instrSession, "%ld", numberofSubdirectories));
    checkErr (rsrtm_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  Returns the subdirectories of the current or of a specified directory.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR directory/This control specifies the directory in which the command will be 
/// HIPAR directory/performed.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the 
/// HIPAR bufferSize/Output parameter.  
/// HIPAR output/This indicator returns the files in the indicated directory as comma 
/// HIPAR output/separated strings, for example: 'SPOOL1.WMF','SPOOL2.WMF','SPOOL3.WMF'
ViStatus _VI_FUNC rsrtm_FileSubdirectories(
    ViSession   instrSession,
    ViString    directory,
    ViInt32 bufferSize,
    ViChar    output[]
)
{
    ViStatus error = VI_SUCCESS;
    ViChar   buffer[RSRTM_IO_BUFFER_SIZE] = "";
    ViChar   *p2buf = NULL;
    ViUInt32  count = 0;
    ViUInt32  length = 0;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckAlloc (output);
    
    p2buf = buffer + sprintf (buffer, "MMEM:DCAT?");
    
    if ((directory != NULL) && (strlen (directory) > 0))
        p2buf += sprintf (p2buf, " '%s'", directory);
    
    strcat (buffer, "\n");
    checkErr (viWrite (instrSession, (ViBuf) buffer, (ViUInt32) strlen (buffer), NULL));
    p2buf = NULL;
    checkErr (Rs_ReadDataUnknownLength (instrSession, &p2buf, &count));
    
    length = ((ViInt32)count > bufferSize) ? bufferSize : count;
    memcpy (output, p2buf, length);
    
    checkErr (rsrtm_CheckStatus (instrSession));

Error:
    if (p2buf != NULL) free (p2buf);
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  Returns a list of the drives of the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR output/Returns a list of the drives of the instrument.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the 
/// HIPAR bufferSize/Output parameter.  
ViStatus _VI_FUNC rsrtm_FileDrives(
    ViSession   instrSession,
    ViInt32   bufferSize,
    ViChar    output[]
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm(rsrtm_GetAttributeViString(instrSession, "",
        RSRTM_ATTR_DRIVES, bufferSize, output), 2, "Drives");
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function is used to read data from the instrument and write it to 
/// HIFN  a user specified file on the host computer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR source/This control selects file for the data transfer from instrument to 
/// HIPAR source/control computer.
/// HIPAR destination/This control defines destination file to which the data transfer from 
/// HIPAR destination/instrument to control computer applies.
ViStatus _VI_FUNC rsrtm_ReadToFileFromInstrument(
    ViSession   instrSession,
    ViString    source,
    ViString    destination
)
{
    ViStatus    error = VI_SUCCESS;
    ViInt32     count = 0;
    ViUInt32    ret_count = 0;
    ViUInt32    length = 0;
    ViChar      buffer[RSRTM_IO_BUFFER_SIZE] = "";
    ViUInt16    intfc = VI_INTF_GPIB;     
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (!source)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Source");
    if (!destination)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Destination");    
    checkErr (viGetAttribute (instrSession, VI_ATTR_INTF_TYPE, &intfc));
    switch (intfc)
    {
        case VI_INTF_ASRL: /* Serial intfc. */
            viCheckErr( viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_NONE));
        break;
    }
    viCheckErr( viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_FALSE));
    
    viCheckErr (viPrintf(instrSession, "MMEM:DATA? '%s'\n", source));
    //Read header       
    viCheckErr (viRead(instrSession, (ViPBuf)buffer, 2,  &ret_count));
    if (sscanf (buffer, "#%1ld", &count) != 1)
        checkErr(RS_ERROR_UNEXPECTED_RESPONSE); 
    viCheckErr (viRead (instrSession, (ViPBuf)buffer, (ViUInt32) count, &ret_count));
    if (sscanf (buffer, "%lu", &length)  != 1)
        checkErr(RS_ERROR_UNEXPECTED_RESPONSE);     
    viCheckErr(Rs_ReadToFile(instrSession, destination, (ViInt32) length, RS_VAL_TRUNCATE, &count)); 
    //Read \n and \r
    //viCheckErr (viRead (instrSession, (ViPBuf)buffer, 2, &ret_count)); 
    if (count!=(ViInt32) length)
        checkErr(RS_ERROR_UNEXPECTED_RESPONSE);         
    viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_TRUE);        
    checkErr( rsrtm_CheckStatus (instrSession));
  
Error:
    viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_TRUE);
    switch (intfc)
    {
        case VI_INTF_ASRL:
            viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_TERMCHAR);
        break;    
    }  
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    
    return error;
}

/// HIFN  This function is used to read data from the host computer and write it 
/// HIFN  to a user specified file in the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR source/This control selects file for the data transfer from control computer 
/// HIPAR source/to instrument.
/// HIPAR destination/This control defines destination file to which the data transfer from 
/// HIPAR destination/control computer to instrument applies.
ViStatus _VI_FUNC rsrtm_WriteFromFileToInstrument(
    ViSession   instrSession,
    ViString    source,
    ViString    destination
)
{
    ViStatus    error = VI_SUCCESS;
    ViInt32     count = 0;
    ViUInt32    digits = 0;
    FILE        *sourceFile = NULL;
    ViUInt32    fileSize = 0;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (!source)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Source");
    if (!destination)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Destination");  
        
    /* Open source file from host controller */
    if ((sourceFile = fopen (source, "rb")) == NULL)
        viCheckErr(RS_ERROR_CANNOT_OPEN_FILE);

    /* Get size of file */
    (void) fseek (sourceFile, 0, SEEK_END);
    fileSize = (ViUInt32) ftell (sourceFile);
    (void) fseek (sourceFile, 0, SEEK_SET);

    /* Create command header */
    digits = (ViUInt32) ((int)floor(log10((double)fileSize))) + 1;
    viCheckErr (viPrintf (instrSession, ":MMEM:DATA '%s',#%ld%ld", destination, digits, fileSize));
    
    viCheckErr (Rs_WriteFromFile (instrSession, source, (ViInt32) fileSize, 0, &count));
	viCheckErr (viWrite (instrSession, (ViBuf)"\n", 1, NULL));
    if (count!=(ViInt32) fileSize)
        viCheckErr (RS_ERROR_WRITING_FILE);     
        
    checkErr( rsrtm_CheckStatusTimed (instrSession));
 
 Error:
    if (sourceFile) (void) fclose (sourceFile);
    
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    
    return error;
}

/// HIFN  This function allows to provide data set operation for Save/Recall 
/// HIFN  items of memory subsystem.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR operation/This control selects the type of memory operation to be performed.
/// HIPAR stateNumber/Defines save and recall state position.
/// HIPAR path/This control specifies the source of the operation to be performed. 
ViStatus _VI_FUNC rsrtm_DataSetFileOperations(
    ViSession   instrSession,
    ViInt32 operation,
    ViInt32 stateNumber,
    ViString    path
)
{
    ViStatus error = VI_SUCCESS;
    ViChar   buffer[RSRTM_IO_BUFFER_SIZE] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    switch (operation)
    {
        case RSRTM_VAL_MEM_SAVE:
            sprintf (buffer, "MMEM:STOR:STAT %ld,'%s'\n", stateNumber, path);
            checkErr (viWrite (instrSession, (ViBuf) buffer, (ViUInt32) strlen (buffer), NULL));
            checkErr (rsrtm_CheckStatus (instrSession));
            break;                
        case RSRTM_VAL_MEM_RECALL:
            sprintf (buffer, "MMEM:LOAD:STAT %ld,'%s'\n", stateNumber, path);
            checkErr (viWrite (instrSession, (ViBuf) buffer, (ViUInt32) strlen (buffer), NULL));
            checkErr (rsrtm_CheckStatus (instrSession));
            break;
        default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Operation");
    }
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN This function closes the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.
ViStatus _VI_FUNC rsrtm_close (ViSession instrSession)
{
    ViStatus    error   = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    checkErr( rsrtm_RsClose (instrSession));

Error:

    (void) Rs_UnlockSession (instrSession, VI_NULL);
    (void) Rs_Dispose (instrSession);

    return error;
}

/// HIFN This function performs all of the drivers clean-up operations
/// HIFN except for closing the RS session.  This function is called by
/// HIFN rsrtm_close or by an RS class driver.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm_init or 
/// HIPAR instrSession/rsrtm_InitWithOptions function.
ViStatus _VI_FUNC rsrtm_RsClose (ViSession instrSession)
{
    ViStatus    error   = VI_SUCCESS;
    ViSession   io      = VI_NULL;

    /* Do not lock here. The caller manages the lock. */

    checkErr( Rs_GetAttribute (instrSession, VI_NULL, RS_ATTR_IO_SESSION, 0, (ViInt32) sizeof (io), &io));

Error:

    (void) Rs_SetAttribute (instrSession, VI_NULL, RS_ATTR_IO_SESSION, 0, VI_NULL);
    
    if(io != 0)
        {
        ViSession   rmSession = VI_NULL;

        checkErr( Rs_GetAttribute (instrSession, VI_NULL, RS_ATTR_VISA_RM_SESSION, 0, (ViInt32) sizeof (rmSession), &rmSession));
        
        (void) viClose (io);
        (void) viClose (rmSession);
        }
    
    return error;
}

/*****************************************************************************
 *------------------- End Instrument Driver Source Code ---------------------*
 *****************************************************************************/
